//============================================================================
// FILE: Argument.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe générique des arguments
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _ARGUMENT_HH
#define _ARGUMENT_HH

// SYSTEM INCLUDES
//
//#include <>                                    //

// PROJECT INCLUDES
//
#include "CoreTypes.h"

//-X///////////////X/////////////////////////////X////////////////////////////


/**@name CDPFORTEXArguments des méthodes
Les méthodes (au sens d'instances de [C]Method[/C], voir page 'Attributs fonctionnels : les méthodes') décrivent des connaissances fonctionnelles sur les entités, les processus ou les événements. \\
Elles possèdent un attribut (appelé le {\em corps}) qui décrit la procédure exécutée lors de l'invocation de la méthode (par le message [Y]Body[/Y] adressé à la méthode ou le message [Y]Exec..Method[/Y] adressé à l'entité, le processus ou l'événement).\\
Ce que fait ce {\em corps} et/ou la valeur qu'il renvoie peuvent dépendre du contexte de l'appel. Ces élements de contexte sont transmis au {\em corps} de la méthode sous forme de paramètres. La liste des paramètres est une donnée membre de la classe [C]Method[/C]. Ainsi, leurs valeurs sont accessibles à l'intérieur du {\em corps}.\\
En fait, un paramètre est une instances de la présente classe [C]Argument[/C].\\

Dans DIESE, le type d'un paramètre, désormais appelé argument, peut être :
\begin{itemize}
\item un des types de descripteurs (sous-classes de [C]Descriptor[/C]), à savoir un entier ou un réel (resp. les types {\em int}, {\em float} et {\em double} de C), un tableau d'entiers ou de flottants, une chaîne de caractères (le type {\em char*} de C), un pointeur sur un tableau d'intervalles sur les entiers (le type {\em pIntIntervalTab*}), une instance des sous-classes de [C]Entity[/C], un tableau d'instances d'entités, un tableau d'instances de spécifications de domaines de valeurs.\\
Le type {\em bool} n'est pas fourni : un argument booléen doit être représenté par un argument de type {\em int} dont les valeurs possibles sont FALSE (0) et TRUE (1).
\item le type {\em Event*} quand la valeur est une instance des sous-classes de [C]Event[/C]
\item le type {\em Descriptor*} quand la valeur est une instance des sous-classes de [C]VariableDescriptor[/C], à l'usage exclusif des méthodes de démons.
\item le type {\em DescValueSpec*} quand la valeur est une instance de [C]DescValueSpec[/C]
\end{itemize}
A chacun de ces types correspond une sous-classe prédéfinie de [C]Argument[/C]. Seules ces sous-classes peuvent avoir des instances (autrement dit, on ne crée jamais d'instance directe de [C]Argument[/C]).\\

De façon générale : 
\begin{itemize}
\item Les constructeurs de base des sous-classes de [C]Argument[/C] affectent un symbole et un nom de classe, ainsi qu'une valeur par défaut, précisés dans la documentation de chaque sous-classe.
\item Les constructeurs par copie copient le type, le symbole, le nom et les valeurs.
\item Les opérateurs d'affectation font que l'argument à gauche a la même valeur que celui à droite.
\item les services [Y]GetValue[/Y] renvoient la valeur de l'instance d'[C]Argument[/C]. \index{méthodes!GetValue - Argument} 
\item les services [Y]SetValue[/Y], argumentées par une valeur, affectent cette valeur à l'instance d'[C]Argument[/C]. \index{méthodes!SetValue - Argument}\\

Lors de la création d'un argument, la valeur par défaut, définie pour les différents types conformément au tableau suivant, est éventuellement assignée à l'argument\index{valeur par défaut} :\\

\begin{tabular}{ccl}

{\tiny classe} & {\tiny type} & {\tiny valeur par défaut} & \\
\hline

 {\tiny IntArgument} & {\tiny int} & {\tiny aucune} \\
 {\tiny FloatArgument} & {\tiny float} & {\tiny aucune} \\
 {\tiny DoubleArgument} & {\tiny double} & {\tiny aucune} \\
 {\tiny StringArgument} & {\tiny char*} & {\tiny ""}\\
 {\tiny EntityArgument} & {\tiny BasicEntity*} & {\tiny NULL}\\
 {\tiny IntTabArgument} & {\tiny pIntTab*} & {\tiny NULL}\\
 {\tiny FloatTabArgument} & {\tiny pFloatTab*} & {\tiny NULL}\\
 {\tiny DoubleTabArgument} & {\tiny pDoubleTab*} & {\tiny NULL}\\
 {\tiny EntityTabArgument} & {\tiny pEntityTab*} & {\tiny aucune} \\
 {\tiny IntDomainArgument} & {\tiny pIntIntervalTab*} & {\tiny un [C]pIntIntervalTab[/C] vide} \\
 {\tiny DescValueSpecArgument} & {\tiny DescValueSpec*} & {\tiny aucune} \\
 {\tiny DescValueSpecTabArgument} & {\tiny pDescValueSpecTab*} & {\tiny aucune} \\
 {\tiny EventArgument} & {\tiny Event*} & {\tiny aucune} \\
 {\tiny DescriptorArgument} & {\tiny Descriptor*} & {\tiny aucune} \\
\end{tabular}
     
\end{itemize}
*/

//@{  // Argument + CoreArgument

//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Les classes de base pour représenter les arguments
\begin{itemize} \item Directive d'inclusion : 
[P]
\#include Argument.h
[/P]
\end{itemize}
 */
//@{  // classe de base + ss-classes par types de valeur

//-X///////////////X/////////////////////////////X////////////////////////////

/**@name La classe de base */
//@{
          /// 
class Argument
{
public:

// @Man: LIFECYCLE
// @{
          // Constructeur de base. N'est normalement appelé que par les sous-classes prédéfinies, et non dans les classes définies par l'utilisateur.
   Argument();       
          // Destructeur.                
   ~Argument();
// @}

// @Man: OPERATORS
// @{
         // Opérateurs < et > (pour la forme).
  Boolean         operator < (const Argument&) const {return TRUE;};
  Boolean         operator > (const Argument&) const {return TRUE;};
// @}                   

/**@name ACCESS */ 
//@{
         /// Affecte un type à l'argument qui reçoit ce message. \index{méthodes!ClassType - Argument}
  void               ClassType(ValueTypeId); // _no swig_
         /// Renvoie le type de l'argument.
   ValueTypeId       ClassType(); // _no swig_
         /// Affecte un symbole de classe à l'argument qui reçoit ce message. \index{méthodes!ClassSymbol - Argument}
   void              ClassSymbol(UserArgumentId); // _no swig_
         /// Renvoie le symbole de classe de l'argument.
   UserArgumentId    ClassSymbol(); // _no swig_
         /// Affecte un nom de classe à l'argument qui reçoit ce message. \index{méthodes!ClassName - Argument}
   void              ClassName(char*); // _no swig_
         /// Renvoie le nom de classe de l'argument.
   char*             ClassName(); // _no swig_
//@}

    // comptage
    static int   getNbConst() {return _nbConst;}
    static int   getNbDest() {return _nbDest;} 

// INQUIRY

protected:

   int static   _nbConst;
   int static   _nbDest;

   ValueTypeId      mArgumentType;
   UserArgumentId   mArgumentClassId;
   char*            mArgumentClassName;
   

private:
}; // _end class_

//@} // fin classe de base

//-X///////////////X/////////////////////////////X////////////////////////////
//
// Argument à valeur INT
//
//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Argument à valeur enti\`ere */
//@{
  /// 
class IntArgument : public Argument
{
 public:

         /// Constructeur. Affecte le type INT. La valeur par défaut est 0.
  IntArgument(UserArgumentId, char*);      
         /// Constructeur par copie. 
    IntArgument(const IntArgument&);  // _no swig_
         /// Opérateur d'affectation.
   const  IntArgument&      operator=(const IntArgument&); 
         ///
  void               SetValue(int);
         ///
  int                GetValue();

 protected:
  int                mValue;   
};  // _end class_

//@} // fin classe 


//-X///////////////X/////////////////////////////X////////////////////////////
//
// Argument à valeur FLOAT
//
//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Argument à valeur flottante */
//@{
  /// 
class FloatArgument : public Argument
{
 public:
         /// Constructeur. Affecte le type FLOAT. La valeur par défaut est 0.
  FloatArgument(UserArgumentId, char*);      
         /// Constructeur par copie. 
  FloatArgument(const FloatArgument&);  // _no swig_ 
         /// Opérateur d'affectation.
   const  FloatArgument&      operator=(const FloatArgument&); 
         ///
  void               SetValue(float);
         ///
  float              GetValue();

 protected:
  float              mValue;   
}; // _end class_

//@} // fin classe 


//-X///////////////X/////////////////////////////X////////////////////////////
//
// Argument à valeur DOUBLE
//
//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Argument à valeur flottante en double précision */
//@{
  /// 
class DoubleArgument : public Argument
{
 public:
         /// Constructeur. Affecte le type DOUBLE. La valeur par défaut est 0.
  DoubleArgument(UserArgumentId, char*);      
         /// Constructeur par copie. 
  DoubleArgument(const DoubleArgument&);  // _no swig_ 
         /// Opérateur d'affectation.
   const  DoubleArgument&      operator=(const DoubleArgument&); 
         ///
  void               SetValue(double);
         ///
  double             GetValue();
  
 protected:
  double             mValue;   
}; // _end class_ 

//@} // fin classe


//-X///////////////X/////////////////////////////X////////////////////////////
//
// Argument à valeur pointeur sur une ENTITY
//
//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Argument à valeur 'pointeur sur une entit\'e' */
//@{
  /// 
class EntityArgument : public Argument
{
 public:
         /// Constructeur. Affecte le type P\_ENTITY. La valeur par défaut est le pointeur NULL.
  EntityArgument(UserArgumentId, char*);      
         /// Constructeur par copie. 
  EntityArgument(const EntityArgument&);   // _no swig_
         /// Opérateur d'affectation.
   const  EntityArgument&      operator=(const EntityArgument&); 
         ///
  void               SetValue(BasicEntity*);
         ///
  BasicEntity*            GetValue();
  
 protected:
  BasicEntity*            mValue;   
}; // _end class_ 

//@} // fin classe


//-X///////////////X/////////////////////////////X////////////////////////////
//
// Argument à valeur pointeur sur un tableau d' ENTITY
//
//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Argument à valeur 'tableau de pointeurs sur des entit\'es' */
//@{
  /// 
class EntityTabArgument : public Argument
{
 public:
         /// Constructeur. Affecte le type ENTITY\_TAB. La valeur par défaut est le pointeur NULL.
  EntityTabArgument(UserArgumentId, char*);      
         /// Constructeur par copie. 
  EntityTabArgument(const EntityTabArgument&);  // _no swig_ 
         /// Opérateur d'affectation.
   const  EntityTabArgument&      operator=(const EntityTabArgument&); 
         ///
  void               SetValue(pEntityTab*);
         ///
  pEntityTab*        GetValue();
  
 protected:
  pEntityTab*        mValue;   
}; // _end class_ 

//@} // fin classe


//-X///////////////X/////////////////////////////X////////////////////////////
//
// Argument à valeur pointeur sur un tableau d'intervalles sur les entiers
//
//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Argument à valeur 'tableau d'intervalles sur les entiers' */
//@{
  /// 
class IntDomainArgument : public Argument
{
 public:
         /// Constructeur. Affecte le type INTDOMAIN. La valeur par défaut est le pointeur NULL.
  IntDomainArgument(UserArgumentId, char*);      
         /// Constructeur par copie. 
  IntDomainArgument(const IntDomainArgument&);  // _no swig_   
         // Destructeur.                
  ~IntDomainArgument();      
         /// Opérateur d'affectation.
   const  IntDomainArgument&      operator=(const IntDomainArgument&); 
         ///
  void               SetValue(pIntIntervalTab*); 
         ///
  void               AddValue(IntInterval*);
         ///
  pIntIntervalTab*        GetValue();
  
 protected:
  pIntIntervalTab*        mValue;   
}; // _end class_ 

//@} // fin classe


//-X///////////////X/////////////////////////////X////////////////////////////
//
// Argument à valeur CHAINE de CARACTERES
//
//-X///////////////X/////////////////////////////X////////////////////////////

 /**@name Argument à valeur 'chaine de caract\`eres' */
//@{
 /// 
class StringArgument : public Argument
{
 public:
         /// Constructeur. Affecte le type STRING. La valeur par défaut est une chaîne vide.
  StringArgument(UserArgumentId, char*);   
         // Destructeur.                
  ~StringArgument();      
         /// Constructeur par copie.    
  StringArgument(const StringArgument&);   // _no swig_
         /// Opérateur d'affectation.
   const  StringArgument&      operator=(const StringArgument&); 
         ///
  void               SetValue(char*);
         ///
  char*              GetValue();
  
 protected:
  char*              mValue;   
}; // _end class_ 

//@} // fin classe


//-X///////////////X/////////////////////////////X////////////////////////////
//
// Argument à valeur pointeur sur un DESCVALUESPEC
//
//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Argument à valeur 'sp\'ecification de domaine de valeur' */
//@{
  /// 
class DescValueSpecArgument : public Argument
{
 public:
         /// Constructeur. Affecte le type P\_DESCVALUESPEC. La valeur par défaut est le pointeur NULL.
  DescValueSpecArgument(UserArgumentId, char*);      
         /// Constructeur par copie. 
  DescValueSpecArgument(const DescValueSpecArgument&);  // _no swig_ 
         /// Opérateur d'affectation.
   const  DescValueSpecArgument&      operator=(const DescValueSpecArgument&); 
         ///
  void               SetValue(DescValueSpec*);
         ///
  DescValueSpec*        GetValue();
  
 protected:
  DescValueSpec*        mValue;   
}; // _end class_ 

//@} // fin classe


//-X///////////////X/////////////////////////////X////////////////////////////
//
// Argument à valeur pointeur sur un tableau de DESCVALUESPEC
//
//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Argument à valeur 'tableau de sp\'ecifications de domaine de valeur' */
//@{
  /// 
class DescValueSpecTabArgument : public Argument
{
 public:
         /// Constructeur. Affecte le type DESCVALUESPEC\_TAB. La valeur par défaut est le pointeur NULL.
  DescValueSpecTabArgument(UserArgumentId, char*);      
         /// Constructeur par copie. 
  DescValueSpecTabArgument(const DescValueSpecTabArgument&);  // _no swig_ 
         /// Opérateur d'affectation.
   const  DescValueSpecTabArgument&      operator=(const DescValueSpecTabArgument&); 
         ///
  void               SetValue(pDescValueSpecTab*);
         ///
  pDescValueSpecTab*        GetValue();
  
 protected:
  pDescValueSpecTab*        mValue;   
}; // _end class_ 

//@} // fin classe


//-X///////////////X/////////////////////////////X////////////////////////////
//
// Argument à valeur pointeur sur un tableau d'entiers
//
//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Argument à valeur 'tableau de entiers' */
//@{
  /// 
class IntTabArgument : public Argument        // JPR 051114
{
 public:
         /// Constructeur. Affecte le type INT\_TAB. La valeur par défaut est le pointeur NULL.
  IntTabArgument(UserArgumentId, char*);      
         /// Constructeur par copie. 
  IntTabArgument(const IntTabArgument&);  // _no swig_   
         // Destructeur.                
  ~IntTabArgument();      
         /// Opérateur d'affectation.
   const  IntTabArgument&      operator=(const IntTabArgument&); 
         ///
  void               SetValue(pIntTab*);
         ///
  pIntTab*        GetValue();
  
 protected:
  pIntTab*        mValue;   
}; // _end class_ 

//@} // fin classe


//-X///////////////X/////////////////////////////X////////////////////////////
//
// Argument à valeur pointeur sur un tableau de flottants
//
//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Argument à valeur 'tableau de flottants' */
//@{
  /// 
class FloatTabArgument : public Argument        // JPR 051114
{
 public:
         /// Constructeur. Affecte le type FLOAT\_TAB. La valeur par défaut est le pointeur NULL.
  FloatTabArgument(UserArgumentId, char*);      
         /// Constructeur par copie. 
  FloatTabArgument(const FloatTabArgument&);  // _no swig_   
         // Destructeur.                
  ~FloatTabArgument();      
         /// Opérateur d'affectation.
   const  FloatTabArgument&      operator=(const FloatTabArgument&); 
         ///
  void               SetValue(pFloatTab*);
         ///
  pFloatTab*        GetValue();
  
 protected:
  pFloatTab*        mValue;   
}; // _end class_ 

//@} // fin classe


//-X///////////////X/////////////////////////////X////////////////////////////
//
// Argument à valeur pointeur sur un tableau de flottants double précision
//
//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Argument à valeur 'tableau de flottants' */
//@{
  /// 
class DoubleTabArgument : public Argument        // JPR 051114
{
 public:
         /// Constructeur. Affecte le type DOUBLE\_TAB. La valeur par défaut est le pointeur NULL.
  DoubleTabArgument(UserArgumentId, char*);      
         /// Constructeur par copie. 
  DoubleTabArgument(const DoubleTabArgument&);  // _no swig_   
         // Destructeur.                
  ~DoubleTabArgument();      
         /// Opérateur d'affectation.
   const  DoubleTabArgument&      operator=(const DoubleTabArgument&); 
         ///
  void               SetValue(pDoubleTab*);
         ///
  pDoubleTab*        GetValue();
  
 protected:
  pDoubleTab*        mValue;   
}; // _end class_ 

//@} // fin classe


//-X///////////////X/////////////////////////////X////////////////////////////
//
// Argument à valeur pointeur sur un EVENT
//
//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Argument à valeur 'pointeur sur un \'ev\'enement */
//@{
  /// 
class EventArgument : public Argument
{
 public:
         /// Constructeur. Affecte le type EVENT. La valeur par défaut est le pointeur NULL.
  EventArgument(UserArgumentId, char*);      
         /// Constructeur par copie. 
  EventArgument(const EventArgument&);   // _no swig_
         /// Opérateur d'affectation.
   const  EventArgument&      operator=(const EventArgument&); 
         ///
  void               SetValue(Event*);
         ///
  Event *            GetValue();
  
 protected:
  Event*             mValue;   
}; // _end class_ 

//@} // fin classe


//-X///////////////X/////////////////////////////X////////////////////////////
//
// Argument à valeur pointeur sur un VARIABLE DESCRIPTOR
//
//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Argument à valeur 'pointeur sur un descripteur' */
//@{
  /// 
class DescriptorArgument : public Argument
{
 public:
         /// Constructeur. Affecte le type P\_DESCRIPTOR. La valeur par défaut est le pointeur NULL.
  DescriptorArgument(UserArgumentId, char*);      
         /// Constructeur par copie. 
  DescriptorArgument(const DescriptorArgument&);  // _no swig_ 
         /// Opérateur d'affectation.
   const  DescriptorArgument&      operator=(const DescriptorArgument&); 
         ///
  void               SetValue(Descriptor*);
         ///
  Descriptor *            GetValue();
  
 protected:
  Descriptor*             mValue;   
}; // _end class_ 

//@} // fin classe

//@} // fin classe de base + par types d'objet

#endif  // _ARGUMENT_HH
