//============================================================================
// FILE: CoreFunction.h
//
// TYPE: C-LIBRARY
// DESCRIPTION: Miscellaneous C functions
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _COREFUNCTION_HH
#define _COREFUNCTION_HH

// SYSTEM INCLUDES
//
//#include <stdarg.h>
//#include <stdlib.h>


// PROJECT INCLUDES
//
#include "Method.h"
#include "File.h"
#include "Tableau.h" // pour les fonctions  procart

/**@name CDPFORTEXFonctions globales (hors gestion du temps)
\begin{itemize} \item Directive d'inclusion : 
[P]
\#include CoreFunction.h
[/P]
\end{itemize} 
On présente ici les fonctions, au sens du langage {\em C}, c'est-à-dire les procédures (avec effets de bord) et les fonctions au sens strict (retournant une valeur) qui ne sont pas attachées à des classes d'objets sous le vocable et avec le r\^ole de "méthodes".\\

Elles sont globales parce qu'on peut les invoquer dans l'ensemble du code, celui de \Db et celui spécifique au domaine d'application.
*/ 
//@{ 


/**@name Interprétation des arguments de la commande de simulation
La commande d'exécution du simulateur ([P]./main ...[/P]) possède des arguments, pour spécifier des données en entrée, ou pour préciser des options d'exécution. \Db prédéfinit quelques arguments standards et propose, par la fonction [Y]ParseMainArguments[/Y], un moyen d'interpréter leurs valeurs dans la ligne de commande. Cette fonction, ou toute autre écrite par l'utilisateur qui aurait le même rôle, doit être appelée en début de programme principal. Dans le cas contraire, les arguments seraient ignorés (conduisant normalement à une erreur d'exécution) ou absents (ce qui dénoterait une exécution non paramétrée, d'un intérêt quasi nul).\\

Outre ceux prédéfinis par \D, l'utilisateur peut ajouter des arguments qui ont une utilité propre à son application. La fonction [Y]ParseMainArguments[/Y] fournit par \Db ne suffit alors plus et l'utilisateur doit écrire lui-même la fonction d'analyse de la ligne de commande. Une pratique conseillée est de toujours invoquer [Y]ParseMainArguments[/Y] en préambule de cette fonction particulière à l'application, laquelle n'a plus, dans sa suite, qu'à traiter l'interprétation des arguments spécifiques.\\

Le premier argument de [Y]ParseMainArgument[/Y] (et donc de toute fonction écrite par l'utilisateur venant la compléter) est le nombre d'arguments de la commande exécutable + 1. Par exemple, cet argument recevra la valeur 2 si la commande est [P]main -h[/P]. Le second argument de [Y]ParseMainArgument[/Y] est le tableau des arguments de la commande exécutable. La manière la plus simple de passer ces deux valeurs à [Y]ParseMainArgument[/Y] est d'utiliser les arguments de la fonction [Y]main()[/Y] (typiquement nommés {\small {\tt argc}} et {\small {\tt argv}}). Dans ce cas, le nom de la commande ({\small {\tt main}}) est l'élément de rang 0 de {\small {\tt argv}} :\\

[P]
\begin{verbatim}
int main (int argc, char* argv[]) {
  App_ParseMainArguments(argc, argv);
  ...
};

void App_ParseMainArguments (int argc, char* argv[]) {
//---------------------------------------------
  Boolean stdArgs_ok = ParseMainArguments(argc, argv);

  if(stdArgs_ok)  {
	for (int i=6;i<argc;i++) {
	  if (!strcmp(argv[i], "-w"))
		gMyTraceWarnings  = TRUE;
	  if (!strcmp(argv[i], "-o"))
		gMyTraceOperations  = TRUE;
	}
  }
}
\end{verbatim}
[/P]

{\small Rôle de ParseMainArguments}\\

Dans sa version courante, [Y]ParseMainArguments[/Y] teste la présence dans la ligne de commande des arguments suivants, et donne le cas échéant la valeur mentionnée aux variables globales correspondantes :
{\small
\begin{verbatim}
    -V       g_BD_TraceActionMode      TRACE_ACTION_ON
    -M       g_BD_TraceNewDelMode      TRACE_NEWDEL_ON
    -MR      g_BD_MemoryReportMode     MEM_REPORT_ON   
    -Warn    g_BD_TraceWarnings        TRACE_WARN_OFF
    -T       g_BD_TraceTableauMode     TRACE_TABLEAU_ON
    -L       g_BD_TraceListeMode       TRACE_LISTE_ON
    -IDN n   gCurrentSimNumId          n
\end{verbatim}
}
Si la ligne de commande est [P]main -h[/P], le résultat est l'affichage de la manière d'invoquer la commande, avec un rappel du sens des arguments obligatoires et optionnels.\\

Les autres arguments reconnus sont les suivants\label{commentParseMainArguments} (on rappelle que l'argument de rang 0 est le module exécutable lui-même) :
\begin{itemize} 
\item La présence parmi les arguments à partir du rang 6 de l'option {\small {\tt -V}} (verbose) permet d'éditer sur la sortie standard des messages sur les actions mises en oeuvre par le moteur de simulation, notamment relatives aux événements occurrents et aux processus mis en jeu. La variable globale [Y]gTraceActionMode[/Y]\index{gTraceActionMode} prend la valeur TRACE_ACTION_ON\index{TRACE_ACTION_ON} (équivalente à TRUE). 
\item La présence parmi les arguments à partir du rang 6 de l'option {\small {\tt -M}} (memory)\label{optionMemory} permet d'éditer sur la sortie standard les messages relatifs aux constructions et destructions d'entités. La variable globale [Y]gTraceNewDelMode[/Y]\index{gTraceNewDelMode} prend la valeur TRACE_NEWDEL_ON\index{TRACE_NEWDEL_ON} (équivalente à TRUE). 
\item La présence parmi les arguments à partir du rang 6 de l'option {\small {\tt -MR}} (memory report)\label{optionMemoryreport} permet d'éditer sur la sortie standard un rapport synthétique que les nombres de constructions/destructions d'objets. La variable globale [Y]g_BD_MemoryReportMode[/Y]\index{g_BD_MemoryReportMode} prend la valeur MEM_REPORT_ON\index{MEM_REPORT_ON} (équivalente à TRUE).
\item La présence de l'option {\small {\tt -Warn}} permet d'éditer sur la sortie standard des messages d'avertissement sur une situation sinon anormale, du moins atypique, et qui mérite une analyse pour justification ou correction.
\item La présence de l'option {\small {\tt -T}} (Tableau) parmi les arguments à partir du rang 6 demande au moteur de simulation d'éditer sur la sortie standard des messages relatifs aux créations et suppressions de tableaux. La variable globale [Y]gTraceTableauMode[/Y]\index{gTraceTableauMode} prend la valeur TRACE_TABLEAU_ON\index{TRACE_TABLEAU_ON} (équivalente à TRUE). 
\item La présence de l'option {\small {\tt -L}} (Liste) parmi les arguments à partir du rang 6 demande au moteur de simulation d'éditer sur la sortie standard des messages relatifs aux créations et suppressions de listes. La variable globale [Y]gTraceListeMode[/Y]\index{gTraceListeMode} prend la valeur TRACE_LISTE_ON\index{TRACE_LISTE_ON} (équivalente à TRUE).  
\item La présence parmi les arguments à partir du rang 6 de l'option {\small {\tt -IDN <val>}} (identifiant numérique) attribue un numéro à la simulation. La variable globale [Y]gCurrentSimNumId[/Y]\index{gCurrentSimNumId} prend la valeur <val>. Cette valeur pourra figurer dans les lignes de trace d'état écrites sur un fichier ou une base de données. Le constructeur d'une simulation lui attribue la valeur de [Y]gCurrentSimNumId[/Y] comme identifiant numérique, celui renvoyé par le service [Y]GetNumId[/Y].
\end{itemize}

{\small Rôle complémentaire de App_ParseMainArguments}\\

Elle est placée, normalement, dans les fichiers [F]UserFunction.h[/F] et [F].cc[/F]. Dans l'exemple ci-dessus, on tient compte de la présence, généralement vérifiée, des 5 arguments correspondant aux fichiers d'entrée de la simulation (voir page 'Les fichiers en entrée de type déclaratif'). Sachant que la commande 'main' est, par convention C, le premier élément du tableau {\small {\tt argv}}, on n'examine que les arguments à partir du rang 6 et au-delà. Une option de commande spécifique est prévue :
{\small
\begin{verbatim}
    -w       gMyTraceWarnings  TRUE
\end{verbatim}
}
[Y]gMyTraceWarnings[/Y] est une variable globale définie dans l'application. Elle peut servir dans des instructions conditionnées à la présence de l'option dans la ligne de commande, ici des instructions de trace d'avertissements.

{\small Exemples de lignes de commandes :\\

[P]main f.par f.sim f.osp f.str f.dir -V -M[/P] \\
demande une simulation avec impression des deux types de messages.\\

[P]main  f.par f.sim f.osp f.str f.dir -V -M[/P] \\
demande une simulation avec impression des messages de construction/destruction seulement.

[P]main  f.par f.sim f.osp f.str f.dir -V -M -IDN 18[/P] \\ 
demande une simulation identifiée par le numéro 18.
*/
//@{
///Interprète les arguments de la commande exécutable. Renvoie FALSE si la commande n'a pas pu être complètement interprétée. La ligne de commande est affichée sur la sortie standard.\index{fonctions!ParseMainArguments}
Boolean ParseMainArguments(int, char*[ ]);  // _no swig_ // modif 100518
						
///Interprète les arguments de la commande exécutable. Renvoie FALSE si la commande n'a pas pu être complètement interprétée. La ligne de commande n'est affichée sur la sortie standard que si le troisième argument est TRUE.
Boolean ParseMainArguments(int, char*[ ], Boolean);  // _no swig_ // + JPR 110720

/// Imprime la ligne de commande sur la sortie standard.\index{fonctions!PrintCommandLine}
void PrintCommandLine(int, char*[ ]);  // _no swig_ // + JPR 100616
//@}
 
 
/**@name Gestion de la hiérarchie des classes d'entités

{\large {\tt Mode d'installation de la hiérarchie :}}\\

\index{hiérarchie de classes d'entités}Les entités du domaine sont regroupées par l'utilisateur en classes et sous-classes, qui héritent de la classe prédéfinie [C]BasicEntity[/C] (et généralement de sa sous-classe [C]Entity[/C]), directement (pour les classes) ou indirectement (pour les sous-classes). \label{hierarchie-de-classes}\\

A chaque classe d'entité propre à l'application, l'utilisateur doit associer une instance d'une classe prédéfinie dédiée à la gestion des classes d'entités : c'est la classe [C]MetaClass[/C], une "classe sur les classes", appelée ci-après "métaclasse". Pour la classe d'entités [C]A[/C], cette instance encapsule des informations générales telles que le nombre et la liste des instances de [C]A[/C] déjà créées et, c'est l'objet de la présente documentation, la place de [C]A[/C] dans la hiérarchie des classes d'entités.\\ 

Ainsi, pour installer une hiérarchie de classes d'entités, l'utilisateur opére normalement de la façon suivante :
\begin{enumerate}

\item {\small Les classes du domaine.} Un fichier [F]UserEntity.h[/F] contient la déclaration des classes du domaine, avec pour le moins un constructeur sans argument, et avec spécification de l'héritage. Dans cet exemple, la classe [C]A[/C] hérite de la classe [C]B[/C] :

[P]
\begin{verbatim}
class A : public Entity { public: A();};
class B : public A { public: B();};
\end{verbatim}
[/P]

\item {\small Les métaclasses}. Quelque part dans le code, mais normalement dans une fonction dédiée, couramment appelée [Y]App_InstallMetaClasses[/Y], on crée les instances de [/C]MetaClass[/C] associées à [C]A[/C] et [C]B[/C] ... :

[P]
\begin{verbatim}
void App_InstallMetaClasses() {
  MetaClass* pMC;
  pMC = CreateMetaClass(ID_A, "a"); pMC->SetParentClassId(BD_ENTITY);
  pMC = CreateMetaClass(ID_B, "b"); pMC->SetParentClassId(ID_A);
}
\end{verbatim}
[/P]
... où ID_A et ID_B (resp. "a" et "b") sont les symboles (resp. les noms) de classes choisis pour [C]A[/C] et [C]B[/C].\\

Cette fonction est invoquée au début du code de toute application (mais après la création d'une instance de [C]Simulation[/C] et après l'invocation de [Y]D_InstallMetaClasses[/Y], qui a le même rôle pour les classes prédéfinies de \D). \index{fonctions!App_InstallMetaClasses}\label{commentInstallMetaClasses} 

[P]
\begin{verbatim}
int main(int argc, char argv[]) {
  ...
  pCurrentSim = new Simulation();
  D_InstallMetaClasses();
  App_InstallMetaClasses();
  SetHierarchy();
  ...
}
\end{verbatim}
[/P]

\item {\small Les constructeurs des classes du domaine}. Le fichier [F]UserEntity.cc[/F] contient la réalisation des constructeurs des classes d'entités. Chaque constructeur installe le lien entre la classe d'entités et sa métaclasse. En outre , le constructeur précise les types des composants ou le type des éléments de l'entité, les types des sur-ensembles, les classes des descripteurs constants et variables (voir page 'Les entités'). \label{hierarchie-de-classes-add} 

[P]
\begin{verbatim}
A::A() {
  SetMetaClass(ID_A);
  AddInstanceToClass();
  ...
  ElementClassId(ID_A_ELEM);
  AddSuperSetClassIdID_SET_OF_A);
  Descriptor* pD = new X(); AddConstantDescriptor(pD);
  ...
}
B::B() {
  SetMetaClass(ID_B);
  AddInstanceToClass();
  ...
}
\end{verbatim}
[/P]

[P]SetMetaClass(ID_A)[/P] installe dans l'instance de [C]A[/C]  un pointeur vers la métaclasse (toutes les instances de [C]A[/C] pointent vers la même métaclasse).\\

[P]AddInstanceToClass()[/P] ajoute dans la métaclasse un pointeur vers l'instance de [C]A[/C]. Donne en outre un nom propre à l'instance (on l'appelle le "nom d'instance"), construit comme la concaténation du nom de classe et d'un numéro d'ordre global unique (par exemple {\small {\tt "a_118"}}). 
De manière plus générale, par [Y]AddInstanceToClass[/Y], l'instance créée est ajoutée à la liste des instances de toutes ses classes ascendantes jusqu'à la classe de base [C]BasicEntity[/C]\\

\item {\small Les symboles des classes}. Le fichier [F]UserTypes.h[/F] contient l'énumération (nommée par exemple {\em App_ClassId}) des symboles de classe. Les nombres doivent être tous plus grand que la valeur de la variable globale [Y]g_BD_ReservedUserClassId[/Y].

[P]
\begin{verbatim}
 enum App_ClassId {ID_A=11, ID_B};
\end{verbatim}
[/P]

\item {\small Destructeurs}. L'écriture d'un destructeur explicite n'est pas exigée. L'appel au destructeur implicite (par un [P]delete <pointeur>;[/P] explicite ou bien dans [Y]DeleteInDepth[/Y]) fait appel aux destructeurs implicites des classes ascendantes, jusqu'à celui de la classe de base des entités ([C]BasicEntity[/C]). Ce dernier retire l'instance en cours de suppression de la liste des instances de toutes les métaclasses dont elle hérite, jusqu'à la métaclasse de base (voir page 'La classe des entités avec descripteurs et méthodes').\\
\end{enumerate}
*/
   
//@{
/// Fonction d'installation des métaclasses, instances de [C]Metaclass[/C], pour chaque sous-classes de [C]BasicEntity[/C] propres à \D. Cette fonction doit être invoquée au début du code de toute application (mais après la création d'une instance de [C]Simulation[/C]), et avant l'invocation d'une fonction [Y]App_InstallMetaClasses[/Y]. \index{fonctions!D_InstallMetaClasses} 
void             D_InstallMetaClasses();

// Crée une instance de chaque classe devant figurer dans le fichier [F]*.cls[/F], c'est-à-dire celles dont l'utilisateur de Solfege pourra développer des spécialisations. Exploité seulement dans mainMakeCls.cc.
void             D_InstantiateClassesForCls(); // + JPR 090819

/// Fonction d'installation des relations d'ascendance et de descendance entre les classes. Cette fonction doit être invoquée au début du code de toute application (mais après la création d'une instance de [C]Simulation[/C], et l'installtion des classes par [Y]D_InstallMetaClasses[/Y] et les fonctions de même rôle dans les sous-couches de \Db (notamment [Y]App_InstallMetaClasses[/Y]). \index{fonctions!SetHierarchy} 
void             SetHierarchy();

        // Crée la métaclasse de symbole {\small {\tt id}} et de nom {\small {\tt nom}} dans la table des métaclasses. Un appel à cette fonction doit être réalisé pour chaque classe d'entités. Et ceci avant la première instanciation d'une quelconque classe. ces appels sont normalement faits dans [Y]App_InstallMetaClasses[/Y]). Le troisième argument indique si la classe est instantiable par un développeur (cas général), ou bien seulement par le programmeur d'une sur-couche générique. Le quatrième argument indique si on construit la liste des instances de la métaclasse (cas général) ou non.
MetaClass*       CreateMetaClass(UserClassId id, char* nom,
				 Boolean userInstantiable,
				 Boolean instanceList); // _no swig_ 

        // Crée la métaclasse de symbole {\small {\tt id}} et de nom {\small {\tt nom}} dans la table des métaclasses. Un appel à cette fonction doit être réalisé pour chaque classe d'entités. Et ceci avant la première instanciation d'une quelconque classe. ces appels sont normalement faits dans [Y]App_InstallMetaClasses[/Y]). Le troisième argument indique si la classe est instantiable par un développeur (cas général), ou bien seulement par le programmeur d'une sur-couche générique. Le quatrième argument est supposé être TRUE.
MetaClass*       CreateMetaClass(UserClassId id, char* nom,
				 Boolean userInstantiable); // _no swig_  

        /// Crée la métaclasse de symbole {\small {\tt id}} et de nom {\small {\tt nom}} dans la table des métaclasses. Un appel à cette fonction doit être réalisé pour chaque classe d'entités. Et ceci avant la première instanciation d'une quelconque classe. ces appels sont normalement faits dans [Y]App_InstallMetaClasses[/Y]).\index{fonctions!CreateMetaClass}
MetaClass*       CreateMetaClass(UserClassId id, char* nom); // _no swig_

///Affiche les informations sur les sous-classes d'entités pour lesquelles au moins une instance a été créée par l'utilisateur. \index{fonctions!DisplayClasses} 
void             DisplayClasses();  // _no swig_

///Renvoie {\small {\tt vrai}} si la classe passée en argument (par son symbole) est une classe 'terminale', au sens où aucune autre classe n'en est une spécialisation, et {\small {\tt faux}} sinon. \index{fonctions!IsTerminalClass} 
   Boolean          IsTerminalClass(UserClassId);   

///Etablit le tableau des symboles de classe des classes terminales qui héritent de celle dont le symbole est passée en argument. \index{fonctions!TerminalDescendantIdTab} 
   void             TerminalDescendantIdTab(UserClassId, UserClassIdTab&);       

///Renvoie le tableau des symboles des classes dont hérite, directement ou non, la classe passée en argument (par son symbole). \index{fonctions!AncestorIdTab} 
   UserClassIdTab   AncestorIdTab(UserClassId); 

///Même rôle que la précédente, mais le résultat devient la valeur du second argument (transmis par référence au programme appelant). 
   void             AncestorIdTab(UserClassId, UserClassIdTab&); // _no swig_         

///Renvoie le tableau des symboles des classes dont hérite directement la classe passée en argument (par son symbole). \index{fonctions!ParentIdTab} 
   UserClassIdTab   ParentIdTab(UserClassId);       

///Renvoie le tableau des symboles des classes qui héritent directement de la classe passée en argument (par son symbole). \index{fonctions!ChildIdTab}    
   UserClassIdTab   ChildIdTab(UserClassId);

///Renvoie le tableau des symboles des classes qui héritent, directement ou non, de la classe passée en argument (par son symbole de classe). \index{fonctions!DescendantIdTab}    
   UserClassIdTab   DescendantIdTab(UserClassId);
    
///Même rôle que la précédente, mais le résultat devient la valeur du second argument (transmis par référence au programme appelant). 
   void             DescendantIdTab(UserClassId, UserClassIdTab&); // _no swig_

///Renvoie le tableau des symboles des classes qui héritent, directement ou non, de la classe passée en argument (par son nom de classe). 
   UserClassIdTab   DescendantIdTab(char*); // _no swig_

///Même rôle que la précédente, mais le résultat devient la valeur du second argument (transmis par référence au programme appelant). 
   void             DescendantIdTab(char*, UserClassIdTab&); // _no swig_

///Renvoie {\small {\tt vrai}} si la classe de symbole {\small {\tt idf}} est une sous-classe, immédiate ou non, de la classe de symbole {\small {\tt idm}}, {\small {\tt faux}} sinon. \index{fonctions!IsAncestorOf}  
   Boolean          IsAncestorOf(UserClassId idm, UserClassId idf);
       
///Renvoie {\small {\tt vrai}} si la classe de symbole {\small {\tt idf}} est une sous-classe immédiate de la classe de symbole {\small {\tt idm}}, {\small {\tt faux}} sinon. \index{fonctions!IsParentOf} 
   Boolean          IsParentOf(UserClassId idm, UserClassId idf);

///Equivalente à {\small {\tt IsParentOf(idm, idf)}}. \index{fonctions!IsChildOf}  
   Boolean          IsChildOf(UserClassId idf, UserClassId idm); 

///Equivalente à {\small {\tt IsAncestorOf(idm, idf)}}. \index{fonctions!IsDescendantOf}  
   Boolean          IsDescendantOf(UserClassId idf, UserClassId idm);  

///Renvoie {\small {\tt vrai}} si les deux classes passées en arguments ont un ancêtre commun (hormis O_ENTITY), et {\small {\tt faux}} sinon. \index{fonctions!HaveCommonAncestor}  
   Boolean          HaveCommonAncestor(UserClassId id1, UserClassId id2);  

///Renvoie le symbole de classe du premier ancêtre commun aux entités du tableau passé en argument. \index{fonctions!FirstCommonAncestorId}  
   UserClassId      FirstCommonAncestorId(const pEntityTab &);   
//@}
 
/**@name Création d'instances de classes (entités, processus, etc.)
{\large {\tt Modes de création d'instances d'entités}}\\

\index{instanciation d'entités}Il existe au moins trois manières de créer des instances d'[C]Entity[/C].
\begin{enumerate}
\item La manière la plus simple consiste à invoquer directement un constructeur quelconque de la classe pour laquelle on désire créer une instance.
[P]
\begin{verbatim}
A* p = new A([argument_list]);
\end{verbatim}
[/P]

\item Les fichiers [F]UserFunction.h[/F] et [F].cc[/F] peuvent contenir l'entête et la réalisation de fonctions plus ou moins élaborées de création d'instances, spécifiques des différentes sous-classes d'[C]Entity[/C]. Elles renvoient un pointeur sur l'instance créée, sous la forme d'un pointeur d'instance de la classe considérée. 

[P]
\begin{verbatim}
A* CreateA(argument_list) {
  A* p = new A();
  ...
  return p;
}
\end{verbatim}
[/P]

Outre l'appel au constructeur de la classe, une telle fonction ... :
\begin{itemize}
\item ... peut contenir des instructions de trace de la création (voir le commentaire de la fonction [Y]ParseMainArguments[/Y], page 'Interprétation des arguments de la commande de simulation') :

[P]
\begin{verbatim}
  ...
  if(gTraceNewDelMode)
  printf("\n+Construction du %d %x", p->ClassId(), p);
\end{verbatim}
[/P]

\item ... permet de donner des valeurs particulières aux attributs de l'instance créée (les valeurs sont alors passées en arguments : il s'agit donc d'une construction paramétrée) : 

[P]
\begin{verbatim}
A* CreateA(float xval, char* name) {
  A* p = new A();
  int descId = AREA;
  p->GetConstantDescriptor(descId)->SetDefaultValue(xval);
  p->InstanceName(name);
  ...
}
\end{verbatim}
[/P]

\item ... permet d'enchasser la construction des composants de l'entité créée, par appel à une fonction de création ou au constructeur de la classe :

[P]
\begin{verbatim}
A* CreateA(float xval, char* name) {
  A* p = new A();
  AElem* pObj = new AElem();
  p->AddElement(pObj);
  ...
}
\end{verbatim}
[/P]
\end{itemize}


\item \label{refNewInstFunctions}Le fichiers [F]UserFunction.h[/F] et [F].cc[/F] peuvent, en général, et doivent dans des cas particuliers (voir plus loin) contenir en outre l'entête et la réalisation de la fonction [Y]App_NewEntityFromString[/Y], qui renvoie une instance d'entité (plus précisément un pointeur sur cette instance) à partir du nom d'une classe d'[C]Entity[/C] ; dans ce cas, une invocation de [Y]Assign_App_NewEntityFromStringFunction[/Y] affecte la fonction utilisateur comme valeur du pointeur sur fonction [Y]pApp_NewEntityFromString}\index{fonctions!App_NewEntityFromString}\index{fonctions!App_NewEntityFromId}\\

La fonction utilisateur doit être programmée de telle sorte qu'elle invoque le constructeur sans argument de la classe dont le symbole ou le nom de classe est passé en argument, et qu'elle renvoie l'instance créée, sous la forme d'un pointeur d'instance d'[C]Entity[/C], ou bien le pointeur NULL si aucune instanciation n'a pu être faite. 

[P]
\begin{verbatim}
Entity* App_NewEntityFromString(char* s) \{}
  Entity* pInst = NULL;
  if(! strcmp(s, "a") pInst = new A();
  if(! strcmp(s, "b") pInst = new B();
  ...
  return pInst;
}
\end{verbatim}
[/P]
\end{enumerate}

Le même principe s'applique à la création d'instances de [C]Process[/C], d'[C]Event[/C] et de [C]DescValueSpec[/C], plus précisément de sous-classes propres à l'application. La fonction appropriée est alors [/Y]App_NewProcessFromStringFunction[/Y] pour les processus, \newline [Y]App_NewEventFromStringFunction[/Y] pour les événements, et [/]App_NewDescValueSpecFromStringFunction[/Y] pour les spécifications de domaines de valeur. Il s'applique aussi à la création d'instances de [C]SerialDataFile[/C] avec la fonction [Y]App_NewSerialDataFileFromStringFunction[/Y].\\

Par invocation d'une fonction d'assignation appropriée (de nom [Y]AssignNew..Function[/Y], parmi celles définies ci-dessous), les fonctions 'utilisateur' deviennent les valeurs des fonctions d'intantiation prédéfinies de \Db [Y]pApp_NewEntityFromString[/Y] \ldots, [Y]pApp_NewProcessFromStringFunction[/Y], \ldots, elles aussi définies ci-dessous.\\

Les appels aux fonctions [Y]Assign_App_New..Function[/Y] peuvent être inclus dans la fonction \newline [Y]App_AssignInstantiators[/Y], déclarée dans \D, mais obligatoirement réalisée dans les fichiers [F]UserFunction[/F] de l'application. En tout état de cause, ces fonctions doivent être invoquées avant toute invocation des fonctions prédéfinies de \Db correspondantes. Même si ces fonctions ne sont jamais invoquées, on doit déclarer et réaliser une fonction [Y]App_AssignInstantiators[/Y] avec un corps éventuellement vide.\\

Les cas particuliers necéssitant l'existence de fonctions d'instanciation sont les suivants.
\begin{itemize}
\item Ces fonctions sont invoquées lors de la lecture et l'interprétation du fichier de déclaration de la structure du système simulé (voir page 'Le fichier de description de la structure du système'). En conséquence, si le code de l'application (typiquement dans son [Y]main()[/Y]) contient un appel de [Y]ParseStructureFile[/Y], il est nécessaire d'invoquer au préalable les fonctions d'assignation. Noter à ce propos que l'interprétation du fichier de spécification de la simulation invoque la fonction [Y]App_AssignInstantiators[/Y] lors de l'instanciation de la simulation courante. Il suffit alors d'avoir inclus les appels aux fonctions d'instanciation dans le code de [Y]App_AssignInstantiators[/Y]\label{refAssignUserInst}.
\item La méthode [Y]Expand[/Y] de la classe [C]EntitySpec[/C] génère une liste d'entités satisfaisant une certaine spécification. Si le {\em mode} de création de la liste demande de lister des instances non pas déjà existantes, mais à créer, alors [Y]Expand[/Y] invoque la fonction [Y]pApp_NewEntityFromId[/Y].
\end{itemize}

Le code de la fonction [Y]App_AssignInstantiators[/Y] sera normalement le suivant :
[P]
\begin{verbatim}
void App_AssignInstantiators() {
  Assign_App_NewEntityFromStringFunction(App_NewEntityFromString);
  Assign_App_NewEntityFromIdFunction(App_NewEntityFromId);
  Assign_App_NewProcessFromStringFunction(App_NewProcessFromString);
  Assign_App_NewEventFromStringFunction(App_NewEventFromString);
  Assign_App_NewEntitySpecFunction(App_NewEntitySpecFromString);
  Assign_App_NewDescValueSpecFunction(App_NewDescValueSpecFromString);
  Assign_App_NewSerialDataFileFunction(App_NewSerialDataFileFromString);
  Assign_App_NewSerialDataFileFromIdFunction(App_NewSerialDataFileFromId);
  Assign_App_SerialDataFileIdFromStringFunction(App_SerialDataFileIdFromString);
};
\end{verbatim}
[P/]
 */
//@{

// ======================
// INSTANCIATION d'ENTITY
// ======================

///Crée et renvoie une instance de la sous-classe d'[C]Entity[/C] dont le nom de classe est passé en argument. Cette fonction invoque d'autres fonctions exécutant le travail aux niveaux applicatifs, où les sous-classes de [C]Entity[/C] sont définies (voir ci-dessous).\index{fonctions!NewEntity}\label{utilNewEntity} 
Entity* D_NewEntityFromString(char*);  // _no swig_ 060818

// Pointeur sur une fonction d'instanciation d'entité. Ce pointeur est valué dans le code de l'application par un appel à [Y]Assign_App_NewEntityFromStringFunction[/Y].
extern Entity* (*pApp_NewEntityFromString)(char*);  // _no swig_

// Pointeur sur une fonction d'instanciation d'entité. Ce pointeur est valué dans le code de l'application par un appel à [Y]Assign_App_NewEntityFromIdFunction[/Y].
extern Entity* (*pApp_NewEntityFromId)(UserClassId);  // _no swig_ 091231

///Affecte au pointeur sur fonction [Y]pApp_NewEntityFromString[/Y] la valeur{\small {\tt f}}. \index{fonctions!Assign_App_NewEntityFromStringFunction}
void Assign_App_NewEntityFromStringFunction(Entity* (*f)(char*));  // _no swig_

///Affecte au pointeur sur fonction [Y]pApp_NewEntityFromId[/Y] la valeur {\small {\tt f}}. \index{fonctions!Assign_App_NewEntityFromIdFunction}
void Assign_App_NewEntityFromIdFunction(Entity* (*f)(UserClassId)); // _no swig_

// ========================
// INSTANCIATION de PROCESS
// ========================

///Crée et renvoie une instance de la sous-classe de [C]Process[/C] dont le nom de classe est passé en argument.\index{fonctions!NewProcess} 
Process* D_NewProcessFromString(char*);  // _no swig_ 060818

// Pointeur sur une fonction d'instanciation de processus. Ce pointeur est valué dans le code de l'application par un appel à [Y]Assign_App_NewProcessFromStringFunction[/Y].
extern Process* (*pApp_NewProcessFromString)(char*);  // _no swig_

///Affecte à la fonction [Y]pApp_NewProcessFromString[/Y] la valeur {\small {\tt f}}. 
void            Assign_App_NewProcessFromStringFunction(Process* (*f)(char*)); // _no swig_

// ========================
// INSTANCIATION de EVENT
// ========================

///Crée et renvoie une instance de la sous-classe d'[C]Event[/C] dont le nom de classe est passé en argument. 
Event* D_NewEventFromString(char * s); // + JPR 060121

// Pointeur sur une fonction d'instanciation d'événement. Ce pointeur est valué dans le code de l'application par un appel à [Y]Assign_App_NewEventFromStringFunction[/Y].
extern Event* (*pApp_NewEventFromString)(char*);  // _no swig_

///Affecte à la fonction [Y]pApp_NewEventFromString[/Y] la valeur {\small {\tt f}}. 
void            Assign_App_NewEventFromStringFunction(Event* (*f)(char*)); // _no swig_

// ============================
// INSTANCIATION de ENTITY_SPEC
// ============================

///Crée et renvoie une instance de la sous-classe de [C]EntitySpec[/C] dont le nom de classe est passé en argument. 
EntitySpec* D_NewEntitySpecFromString(char * s); // + JPR 060121

// Pointeur sur une fonction d'instanciation de spécification d'ensemble d'entités. Ce pointeur est valué dans le code de l'application par un appel à [Y]Assign_App_NewEntitySpecFromStringFunction[/Y].
extern EntitySpec* (*pApp_NewEntitySpecFromString)(char*);  // _no swig_

///Affecte à la fonction [Y]pApp_NewEntitySpecFromString[/Y] la valeur {\small {\tt f}}. 
void            Assign_App_NewEntitySpecFromStringFunction(EntitySpec* (*f)(char*)); // _no swig_

// ================================
// INSTANCIATION de DESC_VALUE_SPEC
// ================================

///Crée et renvoie une instance de la sous-classe de [C]DescValueSpec[/C] dont le nom de classe est passé en argument. 
DescValueSpec* D_NewDescValueSpecFromString(char * s); // + JPR 060121

// Pointeur sur une fonction d'instanciation de domaine de valeurs. Ce pointeur est valué dans le code de l'application par un appel à [Y]Assign_App_NewDescValueSpecFromStringFunction[/Y].
extern DescValueSpec* (*pApp_NewDescValueSpecFromString)(char*);  // _no swig_

///Affecte à la fonction [Y]pApp_NewDescValueSpecFromString[/Y] la valeur {\small {\tt f}}. 
void            Assign_App_NewDescValueSpecFromStringFunction(DescValueSpec* (*f)(char*)); // _no swig_

// =====================================
// INSTANCIATION de SERIAL_DATA_FILE
// =====================================

///Crée et renvoie une instance de la sous-classe de [C]SerialDataFile[/C] dont le nom de classe est passé en argument. 
SerialDataFile* D_NewSerialDataFileFromString(char * s); // + JPR 060121
///Crée et renvoie une instance de la sous-classe de [C]SerialDataFile[/C] dont le symbole de classe est passé en argument. 
SerialDataFile* D_NewSerialDataFileFromId(UserFileId); // + JPR 060121

// Pointeur sur une fonction d'instanciation de fichier séquentiel. Ce pointeur est valué dans le code de l'application par un appel à [Y]Assign_App_NewSerialDataFileFromStringFunction[/Y].
extern SerialDataFile* (*pApp_NewSerialDataFileFromString)(char*);  // _no swig_

// Pointeur sur une fonction d'instanciation de fichier séquentiel. Ce pointeur est valué dans le code de l'application par un appel à [Y]Assign_App_NewSerialDataFileFromIdFunction[/Y].
extern SerialDataFile* (*pApp_NewSerialDataFileFromId)(UserFileId);  // _no swig_

///Affecte à la fonction [Y]pApp_NewSerialDataFileFromString[/Y] la valeur {\small {\tt f}}. 
void            Assign_App_NewSerialDataFileFromStringFunction(SerialDataFile* (*f)(char*)); // _no swig_

///Affecte à la fonction [Y]pApp_NewSerialDataFileFromId[/Y] la valeur {\small {\tt f}}. 
void            Assign_App_NewSerialDataFileFromIdFunction(SerialDataFile* (*f)(UserFileId)); // _no swig_

// =====================================

/// Renvoie le symbole de la sous-classe de [C]SerialDataFile[/C] dont le nom de classe est passé en argument. 
UserFileId D_SerialDataFileIdFromString(char * s); // + JPR 060121

// Pointeur sur une fonction qui renvoie le symbole d'un fichier séquentiel de nom de classe donné. Ce pointeur est valué dans le code de l'application par un appel à [Y]Assign_AppSerialDataFileIdFromStringFunction[/Y].
extern UserFileId (*pApp_SerialDataFileIdFromString)(char*);  // _no swig_

///Affecte à la fonction [Y]pApp_SerialDataFileIdFromString[/Y] la valeur {\small {\tt f}}. 
void Assign_App_SerialDataFileIdFromStringFunction(UserFileId (*f)(char*));  // _no swig_

// =====================================

///Si l'argument est FALSE, invoque [Y]MemoryReport()[/Y] seulement si l'option {\small {\tt -m}} a été prise dans la ligne de commande. Invoque [Y]MemoryReport()[/Y] sans condition si l'argument est TRUE. 
void              MemoryReport (Boolean); // _no swig_ 

/**@name Bilan constructions / destructions

Affiche un rapport sur les constructions des instances des classes prédéfinies de \D. On place typiquement un appel à cette fonction en fin de la fonction 'main'. Les classes en question ne doivent pas avoir été supprimées par un appel approprié à [Y]DeleteEntityClasses[/Y]. Une séquence stantard est :
[P]
\begin{verbatim}
{
  ...
  DeleteEntityInstances();
  MemoryReport();
  DeleteEntityClasses();
  ...
 }
\end{verbatim}
[/P]
\index{fonctions!MemoryReport} 
*/
//@{
  ///
void              MemoryReport (); // _no swig_  
//@}           

//@}   // fin création entités

 
/**@name Recherche d'instances (entité, processus, événement, etc.)

 */
//@{

/**@name GetNamedEntity
Renvoie l'instance d'entité dont le nom d'instance et/ou la classe est /sont donné(es) en argument(s).
*/
//@{

/**@name GetNamedEntity(char* ni)
*/
//@{
///Renvoie l'instance dont le nom d'instance est donné en argument ou NULL si elle n'a pas été trouvée parmi les sous-classes (strictement) de [C]Entity[/C]. \index{fonctions!GetNamedEntity(ni)}\index{recherche d'entité nommée} 
Entity*        GetNamedEntity(char*);  // _no swig_
//@}   

/**@name GetNamedEntity(char* nc, char* ni)
*/
//@{
///Renvoie l'instance dont le nom de classe est donné en premier argument et le nom d'instance en second argument. Si le nom de classe est la chaîne vide, la méthode ci-dessus est invoquée avec le nom d'instance. seul Renvoie NULL si l'instance n'a pas été trouvée.  \index{fonctions!GetNamedEntity(nc, ni)}
Entity*        GetNamedEntity(char*, char*);
//@}   

/**@name GetNamedEntity(UserClassId, char* ni)
*/
//@{
///Renvoie l'instance dont le symbole de classe est donné en premier argument et le nom d'instance en second argument. Si le symbole de classe est -9, la méthode ci-dessus est invoquée avec le nom d'instance seul. Renvoie NULL si l'instance n'a pas été trouvée.  \index{fonctions!GetNamedEntity(idc, ni)}
Entity*        GetNamedEntity(UserClassId, char*); // + JPR 110520
//@} 
  
//@}   



  

///Renvoie le tableau des instances libres. Les instances libres sont celles qui ne sont ni composant ni élément d'un autre entité.   \index{fonctions!GetFreeEntityList()}
pEntityTab*       GetFreeEntityList();

///Renvoie le premier élément, dans le tableau donné en 1\er argument, instance (1) d'une classe descendante de celle dont le symbole de classe est donné en 2\eme argument si le 3\eme argument est {\tt "ancestor"}, (2) de la classe dont le symbole est donné en 2\eme argument si le 3\eme argument est {\tt "itself"}. \index{fonctions!SelectBasicEntity}  
BasicEntity* SelectBasicEntity(const pEntityTab &, UserClassId, char*);

///Renvoie les éléments, dans le tableau donné en 1\er argument, instances (1) d'une classe descendante de celle dont le symbole de classe est donné en 2\eme argument si le 3\eme argument est {\tt "ancestor"}, (2) de la classe dont le symbole est donné en 2\eme argument si le 3\eme argument est {\tt "itself"}. \index{fonctions!SelectBasicEntities}  
pEntityTab* SelectBasicEntities(const pEntityTab & tab, UserClassId id, const char* s);

/// Si on trouve une même entité (pointeur) en position d'élément dans plusieurs entités du tableau en argument, on la renvoie. Si on n'en trouve pas, on renvoie NULL.
// + JPR 061011 pour C_CoreMonitor
BasicEntity* CommonElementInSeveralEntities(pEntityTab*);

///Renvoie l'unique instance de spécification d'entités dont le nom de classe est donné en argument ou NULL si elle n'a pas été trouvée. \index{fonctions!GetNamedEntitySpec}\index{recherche de spécification d'entités nommée} 
EntitySpec*    GetNamedEntitySpec(char*);  // _no swig_

///Renvoie la première instance de spécification d'entités dont le symbole de la classe de référence est passé en argument. \index{fonctions!GetFirstEntitySpec}\index{recherche de spécification d'entités sur la classe de référence} 
EntitySpec*    GetFirstEntitySpec(UserClassId refClassId);

///Renvoie le processus dont le nom d'instance est donné en argument ou NULL si il n'a pas été trouvé. \index{fonctions!GetNamedProcess}\index{recherche de processus nommé} 
Process*       GetNamedProcess(char*);

///Renvoie le processus dont le nom de classe est donné en argument ou NULL si il n'a pas été trouvé. \index{fonctions!GetClassNamedProcess} 
Process*       GetClassNamedProcess(char*);

// Renvoie la liste des  processus dont le nom de classe est donné en argument. \index{fonctions!GetClassNamedProcessTab} 
//pProcessTab*    GetClassNamedProcessTab(char*);  // - JPR 071109

///Renvoie l'événement dont le nom d'instance est donné en argument ou NULL si il n'a pas été trouvé. \index{fonctions!GetNamedEvent}\index{recherche d'événement nommé} 
Event*         GetNamedEvent(char*);

///Renvoie l'événement dont le nom de classe est donné en argument ou NULL si il n'a pas été trouvé. \index{fonctions!GetClassNamedEvent} 
Event*         GetClassNamedEvent(char*);

///Renvoie la liste des événements dont le nom de classe est donné en argument. \index{fonctions!GetClassNamedEventTab} 
pEventTab*     GetClassNamedEventTab(char*);

///Renvoie la méthode dont le nom de classe est donné en troisième argument et qui est un attribut fonctionnel de l'objet (entité, processus, événement, selon la valeur du premier argument) dont le nom d'instance (s'il s'agit d'une entité) ou le nom de classe (s'il s'agit d'un processus ou d'un événement) est donné en deuxième argument. Renvoie le pointeur NULL si la méthode n'a pas été trouvée. \index{fonctions!GetNamedMethod}\index{recherche de méthode nommée} 
Method*        GetNamedMethod(ObjectClassId type, char* s, char* methodName);  // _no swig_

///Renvoie le symbole de la méthode dont le nom est {\small {\tt methodName}}. Ce nom doit avoir été donné à une méthode de la sous-classe de [C]Entity[/C] dont le symbole est{\small  {ttm classId}}. Une instance de cette classe doit déjà avoir été créée. Renvoie -9 si ce n'est pas le cas. \index{fonctions!GetMethodSymbol} 
UserMethodId   GetMethodSymbol(UserClassId classId, char* methodName);  // _no swig_

//Parmi les instances de fichiers de données séquentielles, renvoie la première trouvée dont le symbole de classe correspond à la chaine passée en argument ou NULL si cette classe n'a pas encore d'instance. La correspondance est établie par la fonction [Y]SerialDataFileIdFromString[/Y].\index{fonctions!GetClassNamedSDF}\index{recherche de fichier nommé} 
SerialDataFile* GetClassNamedSDF(char*);

//Parmi les instances de fichiers de données séquentielles, renvoie la première trouvée dont le symbole de classe correspond au symbole passé en argument ou NULL si cette classe n'a pas encore d'instance.
SerialDataFile* GetClassNamedSDF(UserFileId); // + JPR 081027

//Parmi les instances de fichiers de données séquentielles, renvoie la première trouvée dont le  chemin du fichier externe correspondant est la chaine passée en argument ou NULL si aucun fichier externe n'a ce chemin. \index{fonctions!GetPathNamedSDF}\index{recherche de fichier par chemin} 
SerialDataFile* GetPathNamedSDF(char*); // + JPR 101021
 
//@}

/**@name Suppression d'instances (entités, processus, événements, etc.)

 */
//@{

///Désalloue la mémoire occupée par une entité, en fonction de son type. A utiliser dans le même cas que [Y]CopyEntity[/Y]. \index{fonctions!DeleteEntity} 
void         DeleteEntity(BasicEntity*); 

/**@name DeleteEntityClasses
Désalloue une seule ou toutes les sous-classes "utilisateur" de [C]BasicEntity[/C]. En fait, c'est l'instance de [C]MetaClass[/C] associée à la classe d'entités qui est désallouée.
*/
//@{
/**@name DeleteEntityClasses()
Désalloue toutes les sous-classes "utilisateur" de [C]BasicEntity[/C]. On recommande de récupérer au préalable la mémoire occupée par les instances des classes (par [Y]DeleteEntityInstances()[/Y]). \index{fonctions!DeleteEntityClasses} 
*/
//@{
void         DeleteEntityClasses(); // _no swig_ 
//@} 

/**@name DeleteEntityClasses(UserClassId id)
Supprime la sous-classe "utilisateur" de [C]BasicEntity[/C] dont l'entier correspondant au symbole de classe est égal à {\em id}. On recommande de récupérer au préalable la mémoire occupée par les instances de la classe (par {\em DeleteEntityInstances(id)}). 
*/
//@{
void         DeleteEntityClasses(UserClassId id); // _no swig_
//@}
//@}



/**@name DeleteEntityInstances
Supprime toutes les instances d'une ou de toutes les sous-classes "utilisateur" de [C]BasicEntity[/C].
*/
//@{
/**@name DeleteEntityInstances()
Supprime toutes les instances (celles non encore désallouées au moment de l'appel de cette fonction) des sous-classes "utilisateur" de [C]BasicEntity[/C] (voir page 'La classe des simulations'). Un appel de [Y]DeleteEntityInstances[/Y] ne doit pas suivre immédiatement un appel de [Y]DeleteEntityClasses[/Y]. \index{fonctions!DeleteEntityInstances} 
 */
//@{
void         DeleteEntityInstances(); // _no swig_ 
//@} 

/**@name DeleteEntityInstances(UserClassId id)
Supprime toutes les instances (celles non encore désallouées au moment de l'appel de cette fonction) de la sous-classe "utilisateur" de [C]BasicEntity[/C] dont l'entier correspondant au symbole de classe est égal à {\em id}.
 */
//@{  
void         DeleteEntityInstances(UserClassId id); // _no swig_
//@}

//@}




// Désalloue la mémoire occupée par un processus, en fonction de sa classe. \index{fonctions!DeleteProcess} 
void         DeleteProcess(Process*);

///Supprime toutes les instances (celles non encore désallouées) de [C]EntitySpec[/C]. \index{fonctions!DeleteEntitySpecs} 
void         DeleteEntitySpecs(); // _no swig_

///Supprime les démons. Supprime toutes les instances (celles non encore désallouées) des sous-classes "utilisateur" de [C]StructureMonitor[/C] et de [C]DescValueMonitor[/C] (voir page 'La classe des simulations'). \index{fonctions!DeleteMonitors}   
void         DeleteMonitors(); // _no swig_

// Supprime toutes les instances (celles non encore désallouées) de [C]DescValueSpec[/C]. 
// DIESE.4.4 : on ne gère plus la liste des instances : un double appel à DeleteEntityInstances() et à DeleteEntitySpecs() rend inutile un appel à DeleteDescValueSpecs(), puisque les premiers désallouent les descripteurs et les spécifications d'ensembles d'entités, ce qui provoque la désallocation des spécifications de domaines de valeurs.
//void         DeleteDescValueSpecs(); // _no swig_

///Supprime toutes les instances (celles non encore désallouées) de toutes les classes, et affiche un rapport par [Y]MemoryReport[/Y]. Supprime aussi l'instance de [C]Simulation[/C] pointée par [Y]pCurrentSim[/Y]. \index{fonctions!DeleteAll} 
void         DeleteAll(); // _no swig_

//@}  // fin suppression


/**@name Copie d'instances d'entités 
Ces fonctions renvoient une copie de l'entité passée en argument, en fonction de son type. On utilisera une de ces fonctions, et non un constructeur par copie, lorsqu'on ne sait pas si l'entité à copier est une [C]DescriptedEntity[/C], une [C]Entity[/C], ou bien seulement une [C]BasicEntity[/C]. \index{fonctions!CopyEntity}  
 */
//@{
/**@name CopyEntity(BasicEntity*)
 */
//@{
        ///
BasicEntity* CopyEntity(BasicEntity*);
//@}

/**@name CopyEntity(BasicEntity*, bool triggerMonitor)
 */
//@{
       /// Même rôle qu'avec un seul argument, mais les moniteurs installés sur l'instanciation ne seront pas déclenchés si le second argument est {\small {\tt faux}}.
BasicEntity* CopyEntity(BasicEntity*, bool triggerMonitor);
//@}

//Boolean      App_DeleteEntity(BasicEntity*); // _no swig_

//@}   // fin copie entités


/**@name Accès argumenté par une classe d'entités

 */
//@{
        // Renvoie un pointeur sur la structure [C]MetaClass[/C] contenant les informations sur la classe dont le symbole est donné en argument. \index{fonctions!GetMetaClass}
MetaClass*       GetMetaClass(UserClassId); // _no swig_  

///Renvoie le nombre d'instances en vie pour la classe d'entités dont le symbole est donné en argument, ou -9 si la liste des instances n'a pas encoré été créée par [Y]AddInstanceToClass[/Y], ou si la classe désignée n'existe pas. \index{fonctions!GetNumberOfEntities} 
int              GetNumberOfEntities(UserClassId);

///Renvoie le nombre d'instances libres en vie pour la classe d'entités dont le symbole est donné en argument, ou -9 si la liste des instances n'a pas encoré été créée par [Y]AddInstanceToClass[/Y], ou si la classe désignée n'existe pas. Les instances libres sont celles qui ne sont ni composant ni élément d'un autre entité. \index{fonctions!GetNumberOfFreeEntities} 
int              GetNumberOfFreeEntities(UserClassId);

///Renvoie le nombre de créations d'instances pour la classe d'entités dont le symbole est donné en argument, ou -9 si la liste des instances n'a pas encoré été créée par [Y]AddInstanceToClass[/Y]. \index{fonctions!GetEntityCreations} 
int              GetEntityCreations(UserClassId);

///Renvoie le nombre de suppressions d'instances pour la classe d'entités dont le symbole est donné en argument, ou -9 si la liste des instances n'a pas encoré été créée par [Y]AddInstanceToClass[/Y]. \index{fonctions!GetEntityDeletions} 
int              GetEntityDeletions(UserClassId);

///Renvoie le tableau des instances de la classe dont le symbole est donné en argument, ou le pointeur NULL si la liste des instances n'a pas encoré été créée par [Y]AddInstanceToClass[/Y], ou si la classe n'existe pas. \index{fonctions!GetEntityList}\index{instances d'une classe} 
pEntityTab*      GetEntityList(UserClassId);

///Renvoie la première instance trouvée dont le symbole de classe est donné en argument ou NULL si cette classe n'a pas encore d'instance. \index{fonctions!GetFirstBasicEntity}\index{recherche ciblée d'entité} 
BasicEntity*     GetFirstBasicEntity(UserClassId);

///Idem ci-dessus, avec simple changement du type de la valeur retournée. \index{fonctions!GetFirstDescriptedEntity}
DescriptedEntity* GetFirstDescriptedEntity(UserClassId);

///Idem ci-dessus, avec simple changement du type de la valeur retournée. \index{fonctions!GetFirstEntity}
Entity*           GetFirstEntity(UserClassId);

///Retourne la première instance de la classe entityId ayant val comme valeur du descripteur constant descId. \index{fonctions!GetFirstEntityOnDescInt...Value}
DescriptedEntity* GetFirstEntityOnDescIntConstValue(UserClassId entityId, UserClassId descId, int val);

///Retourne la première instance de la classe entityId ayant val comme valeur du descripteur variable descId.
DescriptedEntity* GetFirstEntityOnDescIntVarValue(UserClassId entityId, UserClassId descId, int val);

///Retourne la première instance de la classe entityId ayant val comme valeur du descripteur descId.
DescriptedEntity* GetFirstEntityOnDescIntValue (UserClassId entityId, UserClassId descId, int val);  // 080520

///Renvoie le tableau des instances libres de la classe dont le symbole est donné en argument, ou le pointeur NULL si la liste des instances n'a pas encoré été créée par [Y]AddInstanceToClass[/Y]. Les instances libres sont celles qui ne sont ni composant ni élément d'un autre entité. \index{fonctions!GetFreeEntityList}\index{instances libres d'une classe} 
pEntityTab* GetFreeEntityList(UserClassId); 

///Affiche la liste des pointeurs sur les instances de la classe dont le symbole est donné en argument. \index{fonctions!DisplayEntities}  
void             DisplayEntities(UserClassId);

///Renvoie le symbole de la classe dont le nom est donné en argument. Une instance de la classe de ce nom doit déjà avoir été créée. Renvoie -9 si ce n'est pas le cas. Si l'argument est {\tt "Domain_Entity"}, c'est le symbole (O_ENTITY) de la classe de base [C]Entity[/C] qui est renvoyé. \index{fonctions!GetClassSymbol}
UserClassId      GetClassSymbol(char*);

///Renvoie le nom de la classe dont le symbole est donné en argument. Une instance de la classe de ce symbole doit déjà avoir été créée. \index{fonctions!GetClassName} 
char*            GetClassName(UserClassId);

///Renvoie le symbole du descripteur d'une classe d'entités dont le nom est {\small {\tt descName}}. Ce nom doit avoir été donné à un descripteur (constant ou variable) de la sous-classe de [C]Entity[/C] dont le symbole est {\small {\tt classId}}. Une instance de cette classe doit déjà avoir été créée. Renvoie -9 si ce n'est pas le cas. \index{fonctions!GetDescriptorSymbol} 
UserDescriptorId  GetDescriptorSymbol(UserClassId classId, char* descName);

///Même rôle que la fonction ci-dessus, mais renvoie le type de valeur du descripteur ou -9. \index{fonctions!GetDescriptorType} 
ValueTypeId       GetDescriptorType(UserClassId classId, char* descName);

//@}


/**@name Tri d'un tableau d'entités 
*/
//@{

///Renvoie l'entité du tableau passé en premier argument qui possède la plus grande valeur pour le descripteur passé en second argument. Les entités doivent être des instances de [C]DescriptedEntity[/C] et posséder ce descripteur. \index{fonctions!MaxValued} 
DescriptedEntity* MaxValued(pDescriptedEntityTab*, UserDescriptorId);   // + JPR 041019

///Renvoie l'entité du tableau passé en premier argument qui possède la plus petite valeur pour le descripteur passé en second argument. Les entités doivent être des instances de [C]DescriptedEntity[/C] et posséder ce descripteur. \index{fonctions!MinValued} 
DescriptedEntity* MinValued(pDescriptedEntityTab*, UserDescriptorId);   // + JPR 041019

/// Trie les entités du tableau par ordre croissant des valeurs du descripteur spécifié par son symbole de classe.\label{fonctionsTri} Les entités doivent être des instances de [C]DescriptedEntity[/C] et posséder ce descripteur. \index{fonctions!SortUp} 
void             SortUp(pDescriptedEntityTab*, UserDescriptorId);   // + JPR 040603

/// Trie les entités du tableau par ordre décroissant des valeurs du descripteur spécifié par son symbole de classe. Les entités doivent être des instances de [C]DescriptedEntity[/C] et posséder ce descripteur. \index{fonctions!SortDown} 
void             SortDown(pDescriptedEntityTab*, UserDescriptorId);   // + JPR 040603 

/// Renvoie un tableau dont les éléments sont nécessairement des instances de [C]DescriptedEntity[/C], pour être admis comme argument des fonctions de tri. \index{fonctions!EntityTabToDescriptedEntityTab}
pDescriptedEntityTab* EntityTabToDescriptedEntityTab(pEntityTab); // + JPR 120511
//@}  // fin tri



/**@name Fonctions à valeur aléatoire 
*/
//@{

///Renvoie un entier aléatoire positif inférieur à l'argument. Par exemple [Y]RandomInteger(10)[/Y] renvoie un nombre entre 0 et 9. Normalement, le générateur de nombres doit avoir été initialisé par un appel à la fonction [Y]srand(<seed>)[/Y]. Ceci peut être réalisé par une directive {\tt INITRAND} dans les fichiers de spécifications ou de directives de la simulation (voir page 'Le fichier des spécifications de simulation').\index{fonctions!RandomUniformInteger} 
int               RandomUniformInteger (int);
// ci-dessous pour compatibilité ascendante 
int               RandomInteger (int);

/// Tire, selon la loi uniforme, un nombre entier entre les deux bornes qui sont les valeurs des arguments INT_LOW_BOUND_ARGUMENT et INT_UPPER_BOUND_ARGUMENT de la méthode.\index{fonctions!Sampling_Uniform} 
int Sampling_Uniform (EventMethod*);

 
///Renvoie un nombre réel aléatoire tiré uniformément dans l'intervalle [O  1].\index{fonctions!RandomNormalized} 
double            RandomNormalized();

///Renvoie un réel tiré selon une loi Normale d'espérance {\small {\tt mu}} et d'écart type {\small {\tt sigma}}. Ainsi une série suffisamment grande de nombres ainsi générés aura une moyenne observée proche de mu, et respectera les critères de distribution de la loi Normale (par exemple, environ 68% des nombres seront compris dans un intervalle autour de la moyenne de longueur égale à environ 2 fois l'écart-type).\newline Normalement, le générateur de nombres doit avoir été initialisé par un appel à la fonction [Y]srand(<seed>)[/Y]. Ceci peut être réalisé par une directive {\tt INITRAND} dans les fichiers de spécifications ou de directives de la simulation (voir page 'Le fichier des spécifications de simulation').\index{fonctions!RandomNormalDouble} 
double            RandomNormalDouble(double mu, double sigma);
//@}


  /**@name GetPosition : recherche, dans un tableau, d'un élément désigné par son pointeur
Les fonctions ci-dessous renvoient l'indice, dans le tableau de pointeurs donné en premier argument, du {\em pointeur} spécifié, d'une manière ou d'une autre, en second argument. \index{fonctions!GetPosition}

[Y]
\begin{verbatim}
int GetPosition(const pEntityTab &,             BasicEntity*);

int GetPosition(const pEntityTab &,             char*);

int GetPosition(const TableauString &,          char*);

int GetPosition(const pConstantDescriptorTab &, ConstantDescriptor*);

int GetPosition(const pVariableDescriptorTab &, VariableDescriptor*);

int GetPosition(const pProcessTab &,            Process*);

int GetPosition(const pMethodTab &,             Method*);

int GetPosition(const pStructureMonitorTab &,   StructureMonitor*);

int GetPosition(const pDescValueMonitorTab &,   DescValueMonitor*);

int GetPosition(const pDescValueSpecTab &,      DescValueSpec*);

int GetPosition(const pEntitySpecTab &,         EntitySpec*);
\end{verbatim}
[/Y]
*/
  //@{
int GetPosition(const pEntityTab &, BasicEntity*); // _no swig_

int GetPosition(const pEntityTab &, char*); // _no swig_ 

int GetPosition(const TableauString &, char*); // _no swig_ 

int GetPosition(const pConstantDescriptorTab &, ConstantDescriptor*); // _no swig_ 

int GetPosition(const pVariableDescriptorTab &, VariableDescriptor*); // _no swig_ 
int GetPosition(const pProcessTab &, Process*); // _no swig_
int GetPosition(const pMethodTab &, Method*); // _no swig_ 
int GetPosition(const pStructureMonitorTab &, StructureMonitor*); // _no swig_ 
int GetPosition(const pDescValueMonitorTab &, DescValueMonitor*); // _no swig_ 
int GetPosition(const pDescValueSpecTab &, DescValueSpec*); // _no swig_ 
int GetPosition(const pEntitySpecTab &, EntitySpec*); // _no swig_ 
  //@}

  /**@name GetPosition : recherche, dans un tableau, d'un élément désigné par un symbole de classe
Les fonctions ci-dessous renvoient l'indice, dans le tableau de pointeurs donné en premier argument ... : \index{fonctions!GetPosition} \index{fonctions!GetPositionAncestor} 

- du premier pointeur sur une instance dont le symbole de classe est donné en second argument :
 
[Y]
\begin{verbatim}
  int GetPosition(const pEntityTab &,             UserClassId);
 
  int GetPosition(const pConstantDescriptorTab &, UserDescriptorId);

  int GetPosition(const pVariableDescriptorTab &, UserDescriptorId);
\end{verbatim}
[/Y]

- du premier élément égal au symbole donné en second argument

[Y]
\begin{verbatim}
  int GetPosition(const UserClassIdTab &,         UserClassId);
\end{verbatim}
[/Y]

- du premier élément égal au symbole donné en second argument ou du premier symbole d'une classe ancêtre de la classe dont le symbole est donné en second argument.
 
[Y]
\begin{verbatim}
  int GetPositionAncestor(const UserClassIdTab &, UserClassId);
\end{verbatim}
[/Y]
  */
  //@{
int GetPosition(const pEntityTab &, UserClassId); // _no swig_ 
int GetPosition(const pConstantDescriptorTab &, UserDescriptorId); // _no swig_ 
int GetPosition(const pVariableDescriptorTab &, UserDescriptorId); // _no swig_ 
int GetPosition(const UserClassIdTab &, UserClassId); // _no swig_ 
int GetPositionAncestor(const UserClassIdTab &, UserClassId); // _no swig_
  //@}



/**@name Fonctions constantes (à valeur {\em vrai} ou {\em faux})  

Les méthodes booléennes (prédicats) de l'application ont un corps qui renvoie {\em vrai} ou {\em faux} en fonction de la situation courante. Dans certains cas, on souhaite exprimer que la valeur renvoyée n'est pas fonction de la situation courante, mais au contraire constante. On utilise alors les fonctions ci-dessous comme valeur du corps de la méthode booléenne.\\
Par exemple, dans le constructeur d'un prédicat toujours vrai, on écrira : \index{fonctions!gAlwaysTrue} \index{fonctions!gAlwaysFalse}  

[P]
\begin{verbatim}
MyPredicate::MyPredicate() {
  ...
  AssignBody(gAlwaysTrue);
}
\end{verbatim}
[/P]

Si [C]MyPredicate[/C] est attaché à une entité, c'est la première des fonctions ci-dessous qui deviendra son corps.\\

[Y]
\begin{verbatim}
bool gAlwaysTrue (EntityMethod*);
bool gAlwaysTrue (EntitySpecMethod*);
bool gAlwaysTrue (EventMethod*);

bool gAlwaysFalse (EntityMethod*);
bool gAlwaysFalse (EntitySpecMethod*);
bool gAlwaysFalse (EventMethod*);
\end{verbatim}
[/Y]
*/
//@{
 
bool gAlwaysTrue (EntityMethod*);     // Entity :     + JPR 021204 // _no swig_ 
bool gAlwaysTrue (EntitySpecMethod*); // EntitySpec : + JPR 030801 // _no swig_
bool gAlwaysTrue (EventMethod*);      // Event :      + JPR 100630 // _no swig_

bool gAlwaysFalse (EntityMethod*);    // Entity :     + JPR 021204 // _no swig_ 
bool gAlwaysFalse (EntitySpecMethod*);// EntitySpec : + JPR 030801 // _no swig_ 
bool gAlwaysFalse (EventMethod*);     // Event :      + JPR 100630 // _no swig_
//@}



/**@name D'un élément d'énumération à une chaîne de caractères

{\small Pour diverses énumérations} \\

char* [Y]DescriptorSubclassToChar[/Y](UserDescriptorId); \\
     CONSTANT=1 --> la chaîne "CONSTANT" \\ 
     VARIABLE=2 --> la chaîne "VARIABLE" \\

char* [Y]ValueTypeToChar[/Y](ValueTypeId id);\\ \index{fonctions!ValueTypeToChar}
     VOID=0 --> la chaîne "VOID", \\ 
     BOOLEAN=1 --> la chaîne "BOOLEAN", etc. \\

char* [Y]ObjectClassToChar[/Y](ObjectClassId id);\\
     BD_BASIC_ENTITY=0 --> la chaîne "BD_BASIC_ENTITY", \\
     BD_DESCRIPTED_ENTITY=1 --> la chaîne "BD_DESCRIPTED_ENTITY"; etc. \\
\\

{\small Pour les classes d'entités},...\\
... de descripteurs, de méthodes et d'arguments, l'utilisateur peut exploiter, respectivement, les variables globales [Y]ClassIdToCharTab[/Y], [Y]DescIdToCharTab[/Y], [Y]MethodIdToCharTab[/Y] et [Y]ArgumentIdToCharTab[/Y]. Ce sont des tableaux de chaînes de caractères, contenant, à l'indice <classId> la chaîne correspondante. Par exemple :\\

[P]
\begin{verbatim}
printf("\nLe symbole de classe BD_ENTITY, de valeur %d s'affiche '%s'", 
       BD_ENTITY, ClassIdToCharTab(BD_ENTITY));
\end{verbatim}
[/P]
\begin{verbatim}
>>
Le symbole de classe BD_ENTITY, de valeur 2 s'affiche 'BD_ENTITY'
\end{verbatim}

Ce sont les fonctions prédéfinies [Y]MakeClassIdToCharTab[/Y], [Y]MakeDescIdToCharTab[/Y], [Y]MakeMethodIdToCharTab[/Y] et [Y]MakeArgumentIdToCharTab[/Y] qui "remplissent" ces tableaux pour la couche \D. Le programmeur d'une application a intérêt à compléter ces tableaux avec les chaînes correspondant aux symboles de classes de l'application. C'est la raison pour laquelle les entiers associés aux symboles doivent être supérieurs à ceux déjà utilisés par la couche \D (valeur de [Y]gClassIdToCharTabSize[/Y] pour les classes d'entités). A cet effet, l'utilisateur développera des fonctions pouvant être appellées comme ci-dessous :  \index{fonctions!Make...IdToCharTab}

[P]
\begin{verbatim}
void  App_MakeClassIdToCharTab() {
  for(int k=gClassIdToCharTabSize;k<11;k++)  // le premier entier est 11
	ClassIdToCharTab.push_back("@");

  ClassIdToCharTab.push_back("ID_A");
  ClassIdToCharTab.push_back("ID_B");
  ClassIdToCharTab.push_back("ID_A_ELEM");
  ClassIdToCharTab.push_back("ID_SET_OF_A");
  ....
} 
void  App_MakeDescIdToCharTab() {...};
void  App_MakeMethodIdToCharTab() {...};  
void  App_MakeArgumentIdToCharTab() {...};
\end{verbatim}
[/P]

Ainsi :
[P]
\begin{verbatim}
printf("\nLe symbole de classe ID_A, de valeur %d s'affiche '%s'",
       ID_A, ClassIdToCharTab(ID_A));
\end{verbatim}
[/P]
\begin{verbatim}
>>
Le symbole de classe ID_A, de valeur 11 s'affiche 'ID_A'
\end{verbatim}
 */

//@{
char*            DescriptorSubclassToChar(UserDescriptorId);
char*            ValueTypeToChar(ValueTypeId id);
char*            ObjectClassToChar(ObjectClassId id);

void             MakeClassIdToCharTab(); // _no swig_  
//void             App_MakeClassIdToCharTab(); // _no swig_ // - JPR 110630
char*            ClassIdToString(UserClassId id); // + JPR 110823

void             MakeDescIdToCharTab(); // _no swig_  
//void             App_MakeDescIdToCharTab(); // _no swig_  // - JPR 110630 
char*            DescIdToString(UserDescriptorId id); // + JPR 110823

void             MakeMethodIdToCharTab(); // _no swig_  
//void             App_MakeMethodIdToCharTab(); // _no swig_ 
char*            MethodIdToString(UserMethodId id); // + JPR 110823 

void             MakeArgumentIdToCharTab(); // _no swig_  
//void             App_MakeArgumentIdToCharTab(); // _no swig_ 
char*            ArgumentIdToString(UserArgumentId id); // + JPR 110823 
//@}
  

/**@name De la chaîne affichant un élément d'énumération à un entier

Les fichiers externes de données en entrée de la simulation peuvent contenir des chaînes de caractères représantant des éléments d'énumération. Par exemple, déclarer que le descripteur [C]X[/C], d'une instance de la classe [C]B[/C], prend pour valeur le symbole de classe de la classe [C]A[/C], on écrira : \\

[P]
\begin{verbatim}
+ I unB B
    x = ID_A;
\end{verbatim}
[/P]

Dans le fichier externe, le mot 'ID_A' est lu comme une chaîne de caractères, alors que le descripteur [C]X[/C] est à valeur entière (un symbole de classe). L'interpréteur doit donc effectuer une conversion.\\

Pour les classes prédéfinies d'entités, de descripteurs, de méthodes et d'arguments, la fonction [Y]D_EnumToInt[/Y] assure la conversion de la chaîne de caractères en l'entier correspondant. Ainsi, l'interprétation d'une ligne [P]X=BD_ENTITY;[/P] du fichier externe affectera la valeur 2 au descripteur [C]X[/C].  \\
[Y]D_EnumToInt[/Y] assure aussi la conversion pour les énumérations {\em QuantificatorId}, {\em EntitySpecMode} et {\em RelationalOperator}.\\

Pour l'interprétation des chaînes correspondant aux symboles dans la couche applicative, le développeur pourra écrire une fonction globale, nommée par exemple [Y]App_EnumToInt[/Y]. Elle commencera par invoquer [Y]D_EnumToInt[/Y], puis enchaînera la traduction des symboles de l'application : \\ 

[P]
\begin{verbatim}
int App_EnumToInt (char* token) {
  int result;
  int resultFromDiese = D_EnumToInt(token);
  if(resultFromDiese >= 0) {
    result = resultFromDiese;
  }
  else if(!strcmp(token, "ID_A")) result = 11;
  else if(!strcmp(token, "ID_B")) result = 12;
  else if(!strcmp(token, "ID_A_ELEM")) result = 13;
  else if(!strcmp(token, "ID_SET_OF_A")) result = 14;
  return result;
}
\end{verbatim}
[/P]

Certains services de \Db ont intérêt à connaitre la correspondance, installée dans l'application, entre les symboles d'énumération et la valeur entière. C'est notamment dle cas de l'interprétaion des fichiers en entrée, dont la programmation générale doit s'appliquer aux applications particulières. A cet effet \Db possède une fonction globale, [Y]EnumToInt[/Y], surchargeable par une autre fonction, [Y]AssingEnumToInt[/Y]. Il suffit alors au développeur de l'application d'écrire, en début d'exécution ... :\index{fonctions!AssignEnumToInt}\index{fonctions!EnumToInt}

[P]
\begin{verbatim}
int main (...) {
  ...
  AssignEnumToInt(App_EnumToInt);
  ...
}
\end{verbatim}
[/P]
... pour que \Db sache interpréter les chaînes propres à l'application.
 */
//@{
int D_EnumToInt(char* str);  // _no swig_ 
extern int (*EnumToInt)(char*);  // _no swig_ 

/// Fonction de passage entre symboles d'énumération et entiers correspondants. [/]AssignEnumToInt(int (*f)(char*))[/Y] permet d'assigner un corps {\small {\tt f(char*)}} à la fonction prédéfinie [Y]EnumToInt[/Y]. Il suffit alors de coder dans {\small {\tt f(char*)}} le passage du symbole à la valeur entière, puis d'invoquer, dans le code de l'application, [Y]EnumToInt[/Y], argumenté par la chaîne de caractères du symbole, pour obtenir l'entier correspondant.
void AssignEnumToInt(int (*f)(char*));  // _no swig_ 
//@}


/**@name Fonctions diverses */
//@{

///Prédicat prédéfini d'ordre sur les événements placés en arguments. Renvoie {\small {\tt vrai}} si la date de {\small {\tt ev1}} est antérieure à la date de {\small {\tt ev2}}, ou si, les deux dates étant égales, la valeur de priorité de {\small {\tt ev1}} est plus petite que la valeur de priorité de {\small {\tt ev2}}. Dans ce cas, {\small {\tt ev1}} sera placé avant {\small {\tt ev2}} dans l'agenda d'événements. \index{fonctions!BeforeByDatePriority}\label{defBeforeByDatePriority}  
Boolean          BeforeByDatePriority (Event* ev1, Event* ev2);  // _no swig_

///Renvoie renvoie l'argument (action sur un processus) sous la forme d'une chaîne de caractères. Voir l'énumération des valeurs possibles dans la section "Définition de types, variable globales et constantes symboliques". Par exemple : [/Y]StringActionType(INIT)[/Y] renvoie {\small {\tt INIT}}. Sert notament à l'affichage de l'agenda d'événements sur la sortie standard\.index{fonctions!StringActionType} 
char*            StringActionType (ActionType); // _no swig_

///Force l'affichage de la chaîne [Y]gGlobalString[/Y] sur la sortie standard. La chaîne [Y]gGlobalString[/Y] ne doit pas comporter de saut de ligne, sauf éventuellement en dernier caractère. Fonction à n'utiliser normalement que dans le code d'un simulateur destiné à une utilisation interactive. Dans une utilisation en ligne de commande, la chaîne affichée par cette fonction est préfixée par 'STDOUT'. Cette fonction n'ajoute pas de saut de ligne, ni en début ni en fin de chaîne. \index{fonctions!ForceToStdout} 
void             ForceToStdout();
        
/**@name  itoc(int N, char* c) : convertit un entier en chaîne de caractères

Le résultat est placé dans le second argument, préalablement alloué avec une dimension suffisante, et renvoyé. Le code client doit désallouer la mémoire.\\
 
[P]
\begin{verbatim}
{
  ...
  char* c = new char[10];
  char* d = new char[10];
  int N;
  N = 9999;
  d = itoc(N, c);
  printf("\nitoc(%d, c) --> c : '%s'", N, c);
  printf("\nitoc(%d, c) --> d : '%s'", N, d);
  N = 1;
  c = itoc(N, c);
  printf("\nitoc(%d, c) --> c : '%s'", N, c);
  delete[] c;
  //delete[] d; // erreur : 'double free detected'
}
\end{verbatim}
[/P]
\begin{verbatim}
>>

itoc(9999, c) --> c : '9999'
itoc(9999, c) --> d : '9999'
itoc(1, c) --> c : '1'
\end{verbatim}
*/
//@{
char*            itoc(int N, char* c);
//@}
   
/**@name fillLeft(int L, int N, char* c) : ajoute des espaces à gauche d'un entier

Dans un champ de {\em L} caractères, préalablement alloué avec une dimension suffisante, place l'entier {\em N} à droite, et complète à gauche avec des espaces.\\
Le résultat est placé dans le troisième argument, et renvoyé. Le code client doit désallouer la mémoire.\\
[P]
\begin{verbatim}
{
  ...
  char* c = new char[10];
  char* d = new char[10];
  int L, N;
  L = 6;
  N = 9999;
  d = fillLeft(L, N, c);
  printf("\nfillLeft(%d, %d, c) --> c : '%s'", L, N, c);
  printf("\nfillLeft(%d, %d, c) --> d : '%s'", L, N, d);
  L = 9;
  N = 1;
  d = fillLeft(L, N, c);
  printf("\nfillLeft(%d, %d, c) --> c : '%s'", L, N, c);
  printf("\n\n");
  delete[] c;
  //delete[] d; // erreur : 'double free detected'
}'
\end{verbatim}
[/P]
\begin{verbatim}
>>

fillLeft(6, 9999, c) --> c : '  9999'
fillLeft(6, 9999, c) --> d : '  9999'
fillLeft(9, 1, c) --> c : '        1'
\end{verbatim}
*/
//@{
char*            fillLeft(int L, int N, char* c);
//@}

        /// Affiche les noms d'instance des éléments du tableau passé par référence.\index{fonctions!DisplayEntityNames}
void             DisplayEntityNames(const pEntityTab& tab); // _no swig_
void             DisplayEntityNames(const pDescriptedEntityTab& tab); // _no swig_

        // Affiche les noms d'instance des éléments du tableau passé par référence. Si le second argument est TRUE, les valeurs de pointeurs sont associées aux noms.
void             DisplayEntityNames(const pEntityTab& tab, Boolean); // _no swig_ 
void             DisplayEntityNames(const pDescriptedEntityTab& tab, Boolean); // _no swig_ 

        /// Ecrit dans le fichier [F].log[/F] (préalablement créé et ouvert par un appel à [Y]OpenLogFile[/Y], les noms d'instance des éléments du tableau passé par référence.
void             LogEntityNames(const pEntityTab& tab); // _no swig_
 
        /// Crée et ouvre un fichier de trace banalisé, de nom [F]D_logFile[/F].\index{fonctions!OpenLogFile}
void             OpenLogFile();
  

void FirstToUpper(char*, char*);

void WriteToCls(FILE*, FILE*, FILE*, pIntDTab&, pIntDTab&);

FactorialDesignOfExperiment* GetDesignOfExperiment();

//@}  // fin fonctions diverses

void CheckInputFromIHM();

//X///////////////////////////////////////////////////////////////////////////
//
// Utilitaires pour émuler quelques utilitaires Tableau 
// non swiggables si templatisés
//
//X///////////////////////////////////////////////////////////////////////////

/* *@name Utilitaires sur tableaux typés (pour SWIG)
Ces fonctions réalisent certaines opérations sur les tableaux. Elles ont la même fonction que les méthodes correspondantes des classes 'templates' de tableaux (classe [C]Tableau[/C]). Or, ces méthodes-là ne sont pas exploitables dans les routines {\em perl}, parce que l'outil {\em swig} dans sa version courante ne sait pas générer les interfaces nécessaires lorsque les objets (classes et méthodes) sont sous la forme générique des 'templates'. Les présentes fonctions sont des versions non génériques, c'est-à-dire typées, des méthodes génériques. Un objet générique donné (e.g. get_nth_elem) est décliné en autant d'objets que de types sur lesquels la méthode ou la fonction peut s'appliquer dans une routine {\em perl}.
 */
// @{
 // /Renvoie l'élément d'indice i du tableau d'entités tab. 
BasicEntity* pEntity_get_nth_elem(pEntityTab& tab, int i);

// / Renvoie l'élément d'indice i du tableau d'entiers tab.
int         int_get_nth_elem(TableauInt& tab, int i);

// /Renvoie le nombre d'éléments du tableau d'entités tab. 
int         pEntity_get_nb_elem(pEntityTab& tab);

// /Renvoie le nombre d'éléments du tableau d'entiers tab. 
int        int_get_nb_elem(TableauInt& tab);

// @}  // fin Utilitaires spécifiques

//X///////////////////////////////////////////////////////////////////////////

void testExceptionMsg();

char* my_strdup(const char *s);
char* my_strdup1(const char *s);
char* my_strdup2(const char *s);
Tableau<Tableau<UnifiedValue*>*>* my_newDTableauUnifiedValue();

//@}  // fin Fonctions globales et générales

#endif  // COREFUNCTION_HH
