//============================================================================
// FILE: DescValueMonitor.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe des démons
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _DESCVALUEMONITOR_HH
#define _DESCVALUEMONITOR_HH

// SYSTEM INCLUDES
//
//#include <>                                    //

// PROJECT INCLUDES
//
#include "CoreTypes.h"

/**@name CDPFORTEXLes d\'emons 
Un démon\index{démon} permet de spécifier des actions qui doivent être déclenchées lors de certains événements sur les entités : plus précisément des changements de structure (le démon est alors une instance de [C]StructureMonitor[/C]), ou bien des accès ou changement de valeurs de descripteurs (instance de [C]DescValueMonitor[/C]). L'action à déclencher est décrite dans le démon lui-même, puis le démon est attaché à l'entité (cas [C]StructureMonitor[/C]) ou à un de ses descripteurs (cas [C]DescValueMonitor[/C]). Ces deux classes sont présentées dans les liens de la liste en haut de page.\\

Un exemple typique de démon est la surveillance du franchissement d'un seuil lors par la valeur d'une variable d'état. Ce franchissement est peut-être le déclencheur d'une action corrective ou d'accompagnement. Sans le mécanisme de démon, le codage de cette situation est le suivant : 
\begin{verbatim}
[P]int main(...) {
  ...
  Entity* pE = GetNamedEntity(CUVE, "cuve1");
  float val;
  // forme 1
  val = pE->GetFloatVarValue(NIVEAU_COURRANT);
  if(val > (float)20.) {
    // une action corrective ou d'accompagnement
  }
  // forme 2 alternative équivalente
  val = gMyFloatVarValue(pE);
  ...
}

float gMyFloatVarValue(Entity* pE) {
  float val = pE->GetFloatVarValue(NIVEAU_COURRANT);
  if(val > (float)20.) {
    // une action corrective ou d'accompagnement
  }
  return val;
}[/P]
\end{verbatim}

Le mécanisme permet de ne pas avoir à compléter, dans le code de l'application, chacun des appels aux méthodes standard, par exemple [Y]Get...Value()[/Y], par un morceau de code décrivant les actions associées (forme 1 ci-dessus), ni à les récrire en y intégrant l'action associée (forme 2). En d'autres termes, les méthodes standard conservent leur sémantique, mais, au cours d'une simulation, certaines de leurs invocations peuvent générer des effets de bord et les autres non.\\

 {\large {\tt Les étapes de l'installation d'un démon}}\\

Les éléments de code (et les étapes du codage) qui permettent le déclenchement automatique d'actions sont les suivants, en prenant l'exemple de l'accès à la valeur d'un descripteur flottant. Noter d'ores et déjà que ces fonctions sont les corps d'instances de la sous-classe [C]MonitorMethod[/C], et que ces instances sont descripteurs fonctionnels des classes [C]DescValueMonitor[/C] et [C]StructureMonitor[/C].
\begin{enumerate}

\item {\large {\tt La déclaration}} du démon. Par exemple :
\begin{verbatim} 
// normalement dans [F]UserMonitor.h[/F] :
// déclaration 'extern' pour accessibilité dans les autres fichiers
// (constructeurs des entités, notamment)
[P]extern FloatDescValueMonitor* CurrentLevelMonitor;[/P]

// normalement dans [F]UserMonitor.cc[/F] :
[P]FloatDescValueMonitor* CurrentLevelMonitor;[/P]
\end{verbatim}

\item {\large {\tt La spécification}} et {\large {\tt l'instanciation}} du démon. Il s'agit de doter le démon des fonctions qui seront déclenchées lors des événements d'accès ou de modification de valeurs. Il y a là trois sous-étapes :
\begin{enumerate}

\item {\large {\tt La définition}} des fonctions. Par exemple, l'utilisateur définit une fonction à déclencher lors de l'accès à la valeur du descripteur flottant [C]NiveauCourant[/C] de toutes les instances de la classe [C]Cuve[/C] : 
\begin{verbatim}
[P]float CurrentLevel_whenGet(MonitorMethod* pM) {
  Descriptor* pVD = pM->GetDescriptorArgValue(MONITOREDDESCRIPTOR_ARGUMENT);
  //float oldValue = pM->GetFloatArgValue(CURRENTFLOATVALUE_ARGUMENT);
  float candidateValue = pM->GetFloatArgValue(CANDIDATEFLOATVALUE_ARGUMENT);
  if(candidateValue > (float)20.) {
    // une action corrective ou d'accompagnement
  }
  ...
  return candidateValue;
}[/P]
\end{verbatim}
Le corps ci-dessus exploite les arguments prédéfinis (instances d'[C]Argument[/C]) de la méthode {\em pM}, obligatoirement placée en unique argument de la fonction. Leur liste est précisée page 'La classe de base pour tous les types de descripteurs'.

\item {\large {\tt La création de l'instance}} de démon :  
\begin{verbatim}
[P]CurrentLevelMonitor = new FloatDescValueMonitor("currentLevelMonitor");[/P]
\end{verbatim}

\item {\large {\tt La liaison}} entre les fonctions "utilisateur" et les méthodes prédéfinies des sous-classes de [C]DescValueMonitor[/C]. Par exemple :  
\begin{verbatim}
[P]  CurrentLevelMonitor->AssignWhenGetFloatMethod(CurrentLevel_whenGet);[/P]
\end{verbatim}
 \end{enumerate}

Quelle que soit la façon de les programmer, ces trois sous-étapes doivent être exécutées dans cet ordre. Cependant, elles peuvent être codées dans un seul et même fichier ([/F]UserMonitor.cc[/F] par exemple), sous réserve que la définition des fonctions (a) précède leur liaison (c). Les deux dernières sous-étapes (b et c) peuvent être incluses dans une fonction de spécification de l'ensemble des démons actifs pendant la simulation. 
\noLF
Cette fonction peut être nommée [Y]App_DefineMonitors()[/Y]\index{fonctions!App_DefineMonitors}, déclarée dans [F]UserMonitor.h[/F] et réalisée dans [F]UserMonitor.cc[/F] après la définition des fonctions.   
\begin{verbatim}
[P]void App_DefineMonitors() {
  CurrentLevelMonitor = new FloatDescValueMonitor("currentLevelMonitor");
  CurrentLevelMonitor->AssignWhenGetFloatMethod(CurrentLevel_whenGet);
  ...
};[/P]
\end{verbatim}

Normalement la fonction [Y]App_DefineMonitors[/Y] est appelée en début du programme principal, pour que les démons soient actifs dès le début de la période simulée.\\

Noter la recommandation de nommage : l'identificateur C++ du démon possède un suffixe {\small {\tt 'Monitor'}}, lequel est remplacé dans le nom des fonctions utilisateur par la chaîne {\small {\tt '_whenSet'}} ou {\small {\tt '_whenGet'}}.

\item {\large {\tt L'attachement}} du démon à une ou plusieurs entités, ou bien à un ou plusieurs descripteurs d'une ou plusieurs entités. Cet attachement est programmé dans les constructeurs des entités en question (on rappelle qu'à cet effet le démon a été déclaré {\em extern}). Par exemple, dans le constructeur de la classe [C]Cuve[/C], dont le descripteur [C]NiveauCourant[/C] a pour symbole de classe {\small {\tt NIVEAU_COURANT}}, on aura :  
\begin{verbatim}
[P]Cuve::Cuve() {
  ...
  Descriptor* pD = new NiveauCourant();
  CurrentLevelMonitor->AttachToDescriptor(this, NIVEAU_COURANT);
  ...
}[/P]
\end{verbatim}

L'attachement est ensuite réalisé dynamiquement lors de la création (par l'opérateur [Y]new[/Y]) de toute instance de la classe en question. On insiste sur le fait qu'un démon déclaré, spécifié, instancié, équipé, reste inactif tant qu"il n'a pas été attaché à l'objet à surveiller.
\end{enumerate}
 */
//@{  // fermeture dans StructureMonitor

//===========================================================================

/* *@name La fonction d'instanciation des d\'emons
\label{refDefineUserMonitors}
Cette fonction réalise la création des instances de moniteurs propres à l'application et la liaison entre des fonctions écrites par l'utilisateur et les méthodes prédéfinies des démons. Cette fonction est simplement déclarée dans \D. Elle doit être réalisée dans le code de l'application (et bien sûr ne pas y être déclarée à nouveau).\\

Cette fonction est invoquée par la fonction de lecture et d'interprétation du fichier de spécification de la simulation, au moment de la création de l'instance de simulation ([/Y]pCurrentSim[/Y]).
 */
// @{
         // / 
//void App_DefineMonitors();
// @}

//===========================================================================

/**@name CDPFORTEXLes démons sur les descripteurs des entités
\begin{itemize} \item Directive d'inclusion :
[P]
\#include DescValueMonitor.h
[/P]
\end{itemize}

Un démon de cette classe est une structure permettant de spécifier des actions qui doivent être déclenchées lorsqu'on accède à la valeur d'un descripteur variable (par [Y]GetIntValue[/Y], [Y]GetFloatValue[/Y], [Y]GetDoubleValue[/Y], [Y]GetEntityValue[/Y], [Y]GetEntityTabValue[/Y], [Y]GetStringValue[/Y]), ou bien lorsqu'on mo\-difie cette valeur (par [Y]SetValue[/Y], [Y]AddValue[/Y],\ldots). \\

Un tel démon est déclaré et défini comme une instance de la classe [C]DescValueMonitor[/C]. DIESE ne permet que de spécifier des actions à déclencher lors de l'accès ou de la modification des descripteurs {\em variables} des entités. Aucune réaction ne pourra être associée aux descripteurs {\em constants} des entités, même si leurs valeurs peuvent être modifiées en cours de simulation.\\

{\large {\tt Remarques}} :
\begin{itemize}
\item Dans sa spécification, un même démon instance de [C]DescValueSpec[/C] peut comporter des réactions à des événements divers (accès, modification de valeur) sur le descripteur auquel il sera attaché.   
\begin{verbatim}
[P]void App_DefineMonitors() {
  CurrentLevelMonitor = new FloatDescValueMonitor("currentLevelMonitor");
  CurrentLevelMonitor->AssignWhenGetFloatMethod(CurrentLevel_whenGet);
  CurrentLevelMonitor->AssignWhenSetFloatMethod(CurrentLevel_whenSet);
  ...
};[/P]
\end{verbatim}

\item Un même démon peut être attaché à plusieurs descripteurs différents. C'est bien clair lorsqu'il s'agit de descripteurs de même classe, attachés à différentes instances de la même classe d'entités. C'est aussi vrai pour des descripteurs de classes différentes, à condition que l'effet de bord de la fonction ait un sens identique pour les différentes classes. Par exemple, un démon dont l'effet de bord est un changement d'unité métrique, de mètres en yards, lors de l'accès à la valeur, pourra être attaché à tout descripteur ayant le sens d'une mesure en mètres, quelle que soit la classe de l'entité décrite. 
\begin{verbatim}
[P]Cuve::Cuve() {
  ...
  Descriptor* pD = new NiveauCourant();
  CurrentLevelMonitor->AttachToDescriptor(this, NIVEAU_COURANT);
  ...
}
Tank::Tank() {
  ...
  Descriptor* pD = new CurrentLevel();
  CurrentLevelMonitor->AttachToDescriptor(this, CURRENT_LEVEL);
  ...
}[/P]
\end{verbatim}

\item Il n'est pas possible d'attacher plusieurs démons au même descripteur (dans cette éventualité, c'est le dernier démon attaché qui reste le seul attaché). Si l'idée d'attacher plusieurs démons était justifiée par la multiplicité des effets de bords à déclencher, alors le codage de ces différents effets doit être assemblé dans les fonctions liées aux méthodes prédéfinies d'un seul démon.
\begin{verbatim}
[P]Cuve::Cuve() {
  ...
  Descriptor* pD = new NiveauCourant();
  FromMeterToYardMonitor->AttachToDescriptor(this, NIVEAU_COURANT); // inopérant
  CurrentLevelMonitor->AttachToDescriptor(this, NIVEAU_COURANT);
  ...
}

void App_DefineMonitors() {
  CurrentLevelMonitor = new FloatDescValueMonitor("currentLevelMonitor");
  CurrentLevelMonitor->AssignWhenGetFloatMethod(CurrentLevel_otherWhenGet);
  ...
};

float CurrentLevel_otherWhenGet(MonitorMethod* pM) {
  Descriptor* pVD = pM->GetDescriptorArgValue(MONITOREDDESCRIPTOR_ARGUMENT);
  float candidateValue = pM->GetFloatArgValue(CANDIDATEFLOATVALUE_ARGUMENT);
  if(candidateValue > (float)20.) {
    // une action corrective ou d'accompagnement
  }
  ...
  return candidateValue * (float)1.09361;  // conversion en yards
}[/P]
\end{verbatim}

\item On peut vouloir accéder à la valeur d'un descripteur ou la modifier, sans déclencher les effets de bord du démon attaché au descripteur. Il suffit pour cela d'utiliser les méthodes normales d'accès et de modification ([Y]SetValue[/Y], [Y]GetIntValue[/Y], [Y]GetFloatValue[/Y], \ldots) avec l'argument supplémentaire {\em FALSE}. Par exemple :  
\begin{verbatim}
[P]pMyInstance->GetFloatValue(MY_DESC, FALSE);[/P]
\end{verbatim}

\item Il est possible d'ôter une capacité de réaction à un démon, en lui envoyant un message approprié, correspondant à l'événement visé : par exemple [Y]UnassignWhenGetFloatMethod[/Y], sans argument, pour ôter la réaction à l'accès à une valeur flottante :  
\begin{verbatim}
[P]pADescMonitor->UnassignWhenGetFloatMethod();[/P]
\end{verbatim}

\item De même, il est possible de détacher un démon d'un descripteur en envoyant au démon le message [Y]DetachFromDescriptor[/Y], avec l'adresse de l'entité et le symbole de classe du descripteur en arguments :  
\begin{verbatim}
[P]pADescMonitor->DetachFromDescriptor(pMyInstance, MY_DESC);[/P]
\end{verbatim}

\end{itemize}
*/
//@{
/** La classe de base, mère des démons pour tous les types de descripteurs

{\large {\tt Affectation des fonctions déclenchées automatiquement}}

Il s'agit des fonctions écrites par l'utilisateur, et installées comme corps de méthodes prédéfinies des démons (instance de [C]MonitorMethod[/C]), méthodes connue pour être déclenchée lors d'un événement particulier.\\

Ces fonctions sont installées par des messages tels que [Y]AssignWhenSetIntMethod[/Y] : la fonction passée en argument devient celle qui sera exécutée lors de l'affectation d'une valeur entière au descripteur.  
\begin{verbatim}
[P]  CurrentLevelMonitor->AssignWhenGetFloatMethod(CurrentLevel_whenGet);[/P]
\end{verbatim}

Ces fonctions "utilisateur", ont toutes comme seul argument un pointeur sur la méthode dont elles deviennent corps. 
\begin{verbatim}
[P]float CurrentLevel_whenGet(MonitorMethod* pM) { 
  ...
}[/P]
\end{verbatim}

Les méthodes en question (ici {\small {\tt pM}}) ont, par prédéfinition dans \D, plusieurs arguments, mais pas toujours les mêmes : 

\begin{itemize}
\item Toutes les méthodes ont un agument de type {\em VariableDescriptor*}, celui de symbole de classe MONITOREDDESCRIPTOR_ARGUMENT). \Db passe comme valeur de cet argument le descripteur auquel est attaché le démon.
 
\item Les méthodes dont le corps est déclenché lors d'une modification de valeur par [Y]SetValue[/Y], ont un autre argument (celui de symbole de classe CURRENT..VALUE_ARGUMENT) qui sera lié, lors de l'appel, à la valeur courante du descripteur (celle avant la modification), et encore un autre (celui de symbole de classe CANDIDATE..VALUE_ARGUMENT) qui sera lié à la valeur transmise en argument de la méthode de modification ([Y]SetValue[/Y]). La valeur renvoyée par la fonction sera affectée au descripteur. Elle peut être différente de la valeur candidate.

\item Les méthodes dont le corps est déclenché lors d'un ajout de valeur par [Y]AddValue[/Y] ou un retrait de valeur par [Y]RemoveValue[/Y] ont un argument de symbole de classe CURRENT..VALUE_ARGUMENT qui sera lié au tableau d'entité qui est la valeur du descripteur, alors que l'argument de symbole de classe CANDIDATE..VALUE_ARGUMENT sera lié à l'entité qu'on tente d'ajouter ou de retirer. La valeur renvoyée par la fonction sera effectivement ajoutée à ou retirée de la valeur du descripteur. Elle peut être différente de la valeur candidate.

\item Les méthodes dont le corps est déclenché lors de l'accès à une valeur, ont un argument (celui de symbole de classe CURRENT..VALUE_ARGUMENT) qui sera lié, lors de l'appel, à la valeur courante du descripteur (celle au moment de l'accès par [Y]GetIntValue[/Y], [Y]GetFloatValue[/Y], [Y]GetDoubleValue[/Y], [Y]GetEntityValue[/Y], [Y]GetEntityTabValue[/Y], [Y]GetStringValue[/Y]). La valeur renvoyée par la fonction ne sera pas affectée au descripteur. Une méthode d'affectation doit être explicitement invoquée si on veut la réaliser.
\end{itemize} 

{\large {\tt Désaffectation des fonctions déclenchées automatiquement}}

Les méthodes telles que [Y]UnassignWhenSetIntMethod[/Y] défont la liaison entre une fonction écrite par l'utilisateur et une méthode prédéfinie du démon. La fonction jusqu'à présent liée à la modification d'une valeur entière (par un [Y]SetValue[/Y]) ne sera plus exécutée lors des modifications ultérieures. La fonction elle-même n'est pas détruite, c'est-à-dire qu'elle pourra être à nouveau liée (par un [Y]AssignWhenSetIntMethod[/Y]).
*/
//@{
          ///
class DescValueMonitor 
{
public:

/**@name LIFECYCLE */
//@{
          // Constructeur de base. 
  //DescValueMonitor();
          /// Constructeur avec passage obligatoire du nom de classe. 
   DescValueMonitor(char*);
          /// Constructeur par copie. Les méthodes déclenchées lors des accès et des modifications de valeurs ne sont pas copiées, c'est-à-dire que le démon d'origine et sa copie possèdent les mêmes pointeurs sur ces méthodes.
   DescValueMonitor(const DescValueMonitor&);   
          /// Destructeur.
   ~DescValueMonitor();  
//@}                              

/**@name [SC]Identification[/SC] */
//@{     
          /// Renvoie le type des descripteurs auxquels le démon peut être attaché.\index{méthodes!DescriptorType - DescValueMonitor}
   ValueTypeId        DescriptorType() const;
          /// Etablit le type des descripteurs auxquels le démon peut être attaché.
   void        DescriptorType(ValueTypeId);
                           
          /// Affecte à la classe le nom donné en argument. Ce nom de classe sera hérité par toutes les instances de la classe (en fait normalement une seule). Cette méthode doit être appelée, soit indirectement, en invoquant le constructeur de [C]DescValueMonitor[/C] (lequel reçoit le nom de classe en argument), soit directement dans le constructeur des sous-classes). 
   void            ClassName(char*);      
          /// Renvoie le nom de classe du démon.\index{méthodes!ClassName - DescValueMonitor}
   char*           ClassName() const;
//@}                             

/* *@name OPERATORS */
// @{
         // Opérateurs < et > (pour la forme).
  Boolean         operator < (const DescValueMonitor&) const {return TRUE;};
  Boolean         operator > (const DescValueMonitor&) const {return TRUE;}; 
// @}                             

/**@name [SC]Attachement/détachement[/SC] */  
//@{ 

   void             AddInstance(DescValueMonitor*);
   void             RemoveInstance(DescValueMonitor*);

/// Le démon récepteur de ce message est attaché au descripteur de l'entité (passé en argument). Les méthodes du récepteur pourront être déclenchées lors d'événements sur la valeur du descripteur.\index{méthodes!AttachToDescriptor}
   void  AttachToDescriptor(DescriptedEntity*, UserDescriptorId);
/// Idem, mais le démon récepteur de ce message est attaché au descripteur passé en argument. 
   void  AttachToDescriptor(VariableDescriptor*);  

/// Le démon récepteur de ce message est détaché du descripteur de l'entité (passé en argument). Les méthodes du récepteur ne pourront plus être déclenchées lors d'événements sur la valeur du descripteur.\index{méthodes!DetachFromDescriptor}
   void  DetachFromDescriptor(DescriptedEntity*,UserDescriptorId);
/// Idem, mais le démon récepteur de ce message est détaché du descripteur passé en argument.
   void  DetachFromDescriptor(VariableDescriptor*);
//@}

         // Renvoie la méthode dont le symbole de classe est passé en argument.
   Method*             GetMethod(UserMethodId); 
         // Renvoie la méthode dont le nom de classe est passé en argument.
   Method*             GetMethod(char*);
         // Renvoie la table des méthodes.
   pMethodTab          MethodTab() const;
         // 
   TableauInt          DualMethodTab() const;
 

/* *@name Exécution des méthodes. 

Les méthodes [Y]ExecVoidCoreMethod[/Y], [Y]ExecIntCoreMethod[/Y] \ldots exécutent le corps de la méthode dont le symbole est passé en argument. \index{méthodes!ExecVoidCoreMethod - Monitor}
*/ 
// @{ 
         // 
   void            ExecVoidCoreMethod(UserMethodId);   
         // 
   int             ExecIntCoreMethod(UserMethodId);   
         // 
   float           ExecFloatCoreMethod(UserMethodId);  
         // 
   double          ExecDoubleCoreMethod(UserMethodId); 
         // 
   char*           ExecStringCoreMethod(UserMethodId);
         // 
   BasicEntity*         ExecEntityCoreMethod(UserMethodId);
         // 
   pEntityTab*     ExecEntityTabCoreMethod(UserMethodId);
         // 
   DescValueSpec*     ExecDescValueSpecCoreMethod(UserMethodId);
         // 
   pDescValueSpecTab*     ExecDescValueSpecTabCoreMethod(UserMethodId);
          
// @} 

/* *@name  [SC]Affectation de méthodes spécifiques[/SC]
*/
// @{
         // Ajoute la méthode à la liste des méthodes du démon, seulement si c'est une méthode de démons.
   void                AddMethod(Method*);
// @}

    // comptage Const/Dest
    static int   getNbConst() {return _nbConst;}
    static int   getNbDest() {return _nbDest;}     

    // comptage instances
   //static int                   InstanceNumber();// - JPR 071109 et al. class
   //static pDescValueMonitorTab  InstanceTab();  // - JPR 071109 et al. class

   static int                   mInstanceNumber;         // + JPR 980202
   static pDescValueMonitorTab  mpInstanceTab;

protected: 

   int static   _nbConst;
   int static   _nbDest;

   char*           mThisMonitorClassName;

   ValueTypeId     mDescType;

   pMethodTab             mpMethodTab;    // + JPR 0211
   TableauInt             mpDualMethodTab;

private:
};

//@} // fin classe de base

/** Les classe dérivées pour chacun des types de descripteurs
*/
//@{

/**@name Démon sur descripteur à valeur entière*/
//@{
          ///
class IntDescValueMonitor : public DescValueMonitor 
{
public:

/**@name LIFECYCLE */
//@{
          /// Constructeur de base avec passage obligatoire du nom de classe. 
   IntDescValueMonitor(char*);
          /// Constructeur par copie. Les méthodes déclenchées lors des accès et des modifications de valeurs ne sont pas copiées, c'est-à-dire que le démon d'origine et sa copie possèdent les mêmes pointeurs sur ces méthodes.
   IntDescValueMonitor(const IntDescValueMonitor&);
//@}                             

/**@name OPERATORS */
//@{ 
         // Opérateurs < et > (pour la forme).
  Boolean         operator < (const IntDescValueMonitor&) const {return TRUE;};
  Boolean         operator > (const IntDescValueMonitor&) const {return TRUE;}; 
//@}  

         // Renvoie TRUE si le démon réagit lors d'une affectation de valeur entière ; FALSE sinon.
   Boolean  HasSetIntMethod() const; 
         // Affecte TRUE si le démon doit réagir lors d'une affectation de valeur entière.
   void     HasSetIntMethod(Boolean);



         // Renvoie TRUE si le démon réagit lors d'un accès à la valeur entière du descripteur; FALSE sinon.
   Boolean  HasGetIntMethod() const; 

         // Affecte TRUE si le démon doit réagir lors d'un accès à la valeur entière du descripteur.
   void     HasGetIntMethod(Boolean); 

/**@name Affectation des méthodes déclenchées automatiquement */ 
//@{
         ///\index{méthodes!AssignWhenSetIntMethod}
   void      AssignWhenSetIntMethod(int (*f) (MonitorMethod*));

         ///\index{méthodes!AssignWhenGetIntMethod}
   void      AssignWhenGetIntMethod(int (*f) (MonitorMethod*));
//@}

/**@name  D\'esaffectation des m\'ethodes */ 
//@{
         ///\index{méthodes!UnassignWhenSetIntMethod!}
   void      UnassignWhenSetIntMethod();

         ///\index{méthodes!UnassignWhenGetIntMethod} 
   void      UnassignWhenGetIntMethod();


//@} 

protected:
   Boolean         mHasSetIntMethod;
   Boolean         mHasGetIntMethod;

private:
};

//@} // fin classe dérivée Int

/**@name Démon sur descripteur à valeur flottante simple précision*/
//@{
          ///
class FloatDescValueMonitor : public DescValueMonitor
{
public:

/**@name LIFECYCLE */
//@{
          /// Constructeur de base avec passage obligatoire du nom de classe. 
   FloatDescValueMonitor(char*);
          /// Constructeur par copie. Les méthodes déclenchées lors des accès et des modifications de valeurs ne sont pas copiées, c'est-à-dire que le démon d'origine et sa copie possèdent les mêmes pointeurs sur ces méthodes.
   FloatDescValueMonitor(const FloatDescValueMonitor&);
//@}                             

/**@name OPERATORS */
//@{
         // Opérateurs < et > (pour la forme).
  Boolean         operator < (const FloatDescValueMonitor&) const {return TRUE;};
  Boolean         operator > (const FloatDescValueMonitor&) const {return TRUE;}; 
//@}

         // Renvoie TRUE si le démon réagit lors d'une affectation de valeur entière ; FALSE sinon.
   Boolean  HasSetFloatMethod() const;
         // Affecte TRUE si le démon doit réagir lors d'une affectation de valeur flottante.
   void     HasSetFloatMethod(Boolean); 

         // Renvoie TRUE si le démon réagit lors d'un accès à la valeur entière du descripteur; FALSE sinon.
   Boolean  HasGetFloatMethod() const; 
         // Affecte TRUE si le démon doit réagir lors d'un accès à la valeur flottante du descripteur.
   void     HasGetFloatMethod(Boolean); 

/**@name Affectation des m\'ethodes d\'eclench\'ees automatiquement */ 
//@{
         ///\index{méthodes!AssignWhenSetFloatMethod}
   void      AssignWhenSetFloatMethod(float (* function) (MonitorMethod*));

         ///\index{méthodes!AssignWhenGetFloatMethod}
   void      AssignWhenGetFloatMethod(float (* function) (MonitorMethod*));
//@}

/**@name  D\'esaffectation des m\'ethodes */ 
//@{
         ///\index{méthodes!UnassignWhenSetFloatMethod!}
   void      UnassignWhenSetFloatMethod();

         ///\index{méthodes!UnassignWhenGetFloatMethod} 
   void      UnassignWhenGetFloatMethod();
 
//@} 

protected:
   Boolean         mHasSetFloatMethod;
   Boolean         mHasGetFloatMethod;

private:
};

//@} // fin classe dérivée Float

/**@name Démon sur descripteur à valeur flottante double précision*/
//@{
          ///
class DoubleDescValueMonitor : public DescValueMonitor
{
public:

/**@name LIFECYCLE */
//@{
          /// Constructeur de base avec passage obligatoire du nom de classe. 
   DoubleDescValueMonitor(char*);
          /// Constructeur par copie. Les méthodes déclenchées lors des accès et des modifications de valeurs ne sont pas copiées, c'est-à-dire que le démon d'origine et sa copie possèdent les mêmes pointeurs sur ces méthodes.
   DoubleDescValueMonitor(const DoubleDescValueMonitor&);
//@}                             

/**@name OPERATORS */
//@{
         // Opérateurs < et > (pour la forme).
  Boolean         operator < (const DoubleDescValueMonitor&) const {return TRUE;};
  Boolean         operator > (const DoubleDescValueMonitor&) const {return TRUE;}; 
//@}

         // Renvoie TRUE si le démon réagit lors d'une affectation de valeur entière ; FALSE sinon.
   Boolean  HasSetDoubleMethod() const; 
         // Affecte TRUE si le démon doit réagir lors d'une affectation de valeur flottante double précision.
   void     HasSetDoubleMethod(Boolean);

         // Renvoie TRUE si le démon réagit lors d'un accès à la valeur entière du descripteur; FALSE sinon.
   Boolean  HasGetDoubleMethod() const;
         // Affecte TRUE si le démon doit réagir lors d'un accès à la valeur flottante double précision du descripteur.
   void     HasGetDoubleMethod(Boolean);    

/**@name Affectation des m\'ethodes d\'eclench\'ees automatiquement */ 
//@{
         ///\index{méthodes!AssignWhenSetDoubleMethod}
   void      AssignWhenSetDoubleMethod(double (* function) (MonitorMethod*));

         ///\index{méthodes!AssignWhenGetDoubleMethod}
   void      AssignWhenGetDoubleMethod(double (* function) (MonitorMethod*));
//@}

/**@name  D\'esaffectation des m\'ethodes */ 
//@{
         ///\index{méthodes!UnassignWhenSetDoubleMethod!}
   void      UnassignWhenSetDoubleMethod();

         ///\index{méthodes!UnassignWhenGetDoubleMethod} 
   void      UnassignWhenGetDoubleMethod();
 
//@} 

protected:
   Boolean         mHasSetDoubleMethod;
   Boolean         mHasGetDoubleMethod;

private:
};

//@} // fin classe dérivée Double

/**@name Démon sur descripteur à valeur chaine de caractères*/
//@{
          ///
class StringDescValueMonitor : public DescValueMonitor
{
public:

/**@name LIFECYCLE */
//@{
          /// Constructeur de base avec passage obligatoire du nom de classe. 
   StringDescValueMonitor(char*);
          /// Constructeur par copie. Les méthodes déclenchées lors des accès et des modifications de valeurs ne sont pas copiées, c'est-à-dire que le démon d'origine et sa copie possèdent les mêmes pointeurs sur ces méthodes.
   StringDescValueMonitor(const StringDescValueMonitor&);
//@}                             

/**@name OPERATORS */
//@{ 
         // Opérateurs < et > (pour la forme).
  Boolean         operator < (const StringDescValueMonitor&) const {return TRUE;};
  Boolean         operator > (const StringDescValueMonitor&) const {return TRUE;}; 
//@}

         // Renvoie TRUE si le démon réagit lors d'une affectation de valeur entière ; FALSE sinon.
   Boolean  HasSetStringMethod() const;
         // Affecte TRUE si le démon doit réagir lors d'une affectation d'une chaîne de caractères comme valeur.
   void     HasSetStringMethod(Boolean);
 
         // Renvoie TRUE si le démon réagit lors d'un accès à la valeur entière du descripteur; FALSE sinon.
   Boolean  HasGetStringMethod() const;  
         // Affecte TRUE si le démon doit réagir lors d'un accès à la chaîne de caractères qui est la valeur du descripteur.
   void     HasGetStringMethod(Boolean); 

/**@name Affectation des m\'ethodes d\'eclench\'ees automatiquement */ 
//@{
         ///\index{méthodes!AssignWhenSetStringMethod}
   void      AssignWhenSetStringMethod(char* (* function) (MonitorMethod*));

         ///\index{méthodes!AssignWhenGetStringMethod}
   void      AssignWhenGetStringMethod(char* (* function) (MonitorMethod*));
//@}

/**@name  D\'esaffectation des m\'ethodes */ 
//@{
         ///\index{méthodes!UnassignWhenSetStringMethod!}
   void      UnassignWhenSetStringMethod();

         ///\index{méthodes!UnassignWhenGetStringMethod} 
   void      UnassignWhenGetStringMethod();
 
//@} 

protected:
   Boolean         mHasSetStringMethod;
   Boolean         mHasGetStringMethod;

private:
};

//@} // fin classe dérivée String

/**@name Démon sur descripteur à valeur pointeur sur une entité*/
//@{
          ///
class EntityDescValueMonitor : public DescValueMonitor
{
public:

/**@name LIFECYCLE */
//@{
          /// Constructeur de base avec passage obligatoire du nom de classe. 
   EntityDescValueMonitor(char*);
          /// Constructeur par copie. Les méthodes déclenchées lors des accès et des modifications de valeurs ne sont pas copiées, c'est-à-dire que le démon d'origine et sa copie possèdent les mêmes pointeurs sur ces méthodes.
   EntityDescValueMonitor(const EntityDescValueMonitor&);
//@}                             

/**@name OPERATORS */
//@{
         // Opérateurs < et > (pour la forme).
  Boolean         operator < (const EntityDescValueMonitor&) const {return TRUE;};
  Boolean         operator > (const EntityDescValueMonitor&) const {return TRUE;}; 
//@}

         // Renvoie TRUE si le démon réagit lors d'une affectation de valeur entière ; FALSE sinon.
   Boolean  HasSetEntityMethod() const; 
         // Affecte TRUE si le démon doit réagir lors d'une affectation d'une instance de {\Entity} comme valeur.
   void     HasSetEntityMethod(Boolean);

         // Renvoie TRUE si le démon réagit lors d'un accès à la valeur entière du descripteur; FALSE sinon.
   Boolean  HasGetEntityMethod() const; 
         // Affecte TRUE si le démon doit réagir lors d'un accès à l'instance de [C]Entity[/C] qui est la valeur du descripteur.
   void     HasGetEntityMethod(Boolean);   

/**@name Affectation des m\'ethodes d\'eclench\'ees automatiquement */ 
//@{
         ///\index{méthodes!AssignWhenSetEntityMethod}
   void      AssignWhenSetEntityMethod(BasicEntity* (* function) (MonitorMethod*));

         ///\index{méthodes!AssignWhenGetEntityMethod}
   void      AssignWhenGetEntityMethod(BasicEntity* (* function) (MonitorMethod*));
//@}

/**@name  D\'esaffectation des m\'ethodes */ 
//@{
         ///\index{méthodes!UnassignWhenSetEntityMethod!}
   void      UnassignWhenSetEntityMethod();

         ///\index{méthodes!UnassignWhenGetEntityMethod} 
   void      UnassignWhenGetEntityMethod();
 
//@} 

protected:
   Boolean         mHasSetEntityMethod;
   Boolean         mHasGetEntityMethod;

private:
};

//@} // fin classe dérivée Entity

/**@name Démon sur descripteur à valeur tableau de pointeurs sur une entité*/
//@{
          ///
class EntityTabDescValueMonitor : public DescValueMonitor
{
public:

/**@name LIFECYCLE */
//@{
          /// Constructeur de base avec passage obligatoire du nom de classe. 
   EntityTabDescValueMonitor(char*);
          /// Constructeur par copie. Les méthodes déclenchées lors des accès et des modifications de valeurs ne sont pas copiées, c'est-à-dire que le démon d'origine et sa copie possèdent les mêmes pointeurs sur ces méthodes.
   EntityTabDescValueMonitor(const EntityTabDescValueMonitor&);
//@}                             

/**@name OPERATORS */
//@{
         // Opérateurs < et > (pour la forme).
  Boolean         operator < (const EntityTabDescValueMonitor&) const {return TRUE;};
  Boolean         operator > (const EntityTabDescValueMonitor&) const {return TRUE;}; 
//@}

         // Renvoie TRUE si le démon réagit lors d'une affectation de valeur entité ; FALSE sinon.
   Boolean  HasSetEntityTabMethod() const;  
         // Affecte TRUE si le démon doit réagir lors d'une affectation d'un tableau d'instances de {\Entity} comme valeur.
   void     HasSetEntityTabMethod(Boolean);

         // Affecte TRUE si le démon doit réagir lors d'un ajout d'une valeur entité à un tableau d'instances d'{\Entity}.
   void     HasAddEntityMethod(Boolean);
         // Renvoie TRUE si le démon réagit lors d'un ajout d'une valeur entité ; FALSE sinon.
   Boolean  HasAddEntityMethod() const;

         // Affecte TRUE si le démon doit réagir lors d'un retrait d'une valeur entité à un tableau d'instances d'{\Entity}.
   void     HasRemoveEntityMethod(Boolean);
         // Renvoie TRUE si le démon réagit lors d'un retrait d'une valeur entité ; FALSE sinon.
   Boolean  HasRemoveEntityMethod() const;

         // Renvoie TRUE si le démon réagit lors d'un accès à la valeur entité du descripteur; FALSE sinon.
   Boolean  HasGetEntityTabMethod() const; 
         // Affecte TRUE si le démon doit réagir lors d'un accès au tableau d'instances de [C]Entity[/C] qui est la valeur du descripteur.
   void     HasGetEntityTabMethod(Boolean);   

/**@name Affectation des m\'ethodes d\'eclench\'ees automatiquement */ 
//@{
         ///\index{méthodes!AssignWhenSetEntityTabMethod}
   void      AssignWhenSetEntityTabMethod(pEntityTab* (* function) (MonitorMethod*));

         ///\index{méthodes!AssignWhenGetEntityTabMethod}
   void      AssignWhenGetEntityTabMethod(pEntityTab* (* function) (MonitorMethod*));
         ///\index{méthodes!AssignWhenAddEntityMethod}
   void      AssignWhenAddEntityMethod(BasicEntity* (* function) (MonitorMethod*));
         ///\index{méthodes!AssignWhenRemoveEntityMethod}
   void      AssignWhenRemoveEntityMethod(BasicEntity* (* function) (MonitorMethod*));
//@}

/**@name  D\'esaffectation des m\'ethodes */ 
//@{
         ///\index{méthodes!UnassignWhenSetEntityTabMethod!}
   void      UnassignWhenSetEntityTabMethod();
         ///\index{méthodes!UnassignWhenGetEntityTabMethod} 
   void      UnassignWhenGetEntityTabMethod();
         ///\index{méthodes!UnassignWhenAddEntityMethod} 
   void      UnassignWhenAddEntityMethod();
         ///\index{méthodes!UnassignWhenRemoveEntityMethod} 
   void      UnassignWhenRemoveEntityMethod();
 
//@} 

protected:
   Boolean         mHasSetEntityTabMethod;
   Boolean         mHasGetEntityTabMethod;
   Boolean         mHasAddEntityMethod;
   Boolean         mHasRemoveEntityMethod;

private:
};

//@} // fin classe dérivée EntityTab

/**@name Démon sur descripteur à valeur tableau de pointeurs sur des spécifications de domaines de valeurs*/
//@{
          ///
class DescValueSpecTabDescValueMonitor : public DescValueMonitor
{
public:

/**@name LIFECYCLE */
//@{
          /// Constructeur de base avec passage obligatoire du nom de classe. 
   DescValueSpecTabDescValueMonitor(char*);
          /// Constructeur par copie. Les méthodes déclenchées lors des accès et des modifications de valeurs ne sont pas copiées, c'est-à-dire que le démon d'origine et sa copie possèdent les mêmes pointeurs sur ces méthodes.
   DescValueSpecTabDescValueMonitor(const DescValueSpecTabDescValueMonitor&);
//@}                             

/**@name OPERATORS */
//@{
         // Opérateurs < et > (pour la forme).
  Boolean         operator < (const DescValueSpecTabDescValueMonitor&) const {return TRUE;};
  Boolean         operator > (const DescValueSpecTabDescValueMonitor&) const {return TRUE;}; 
//@}

         // Renvoie TRUE si le démon réagit lors d'une affectation de valeur entité ; FALSE sinon.
   Boolean  HasSetDescValueSpecTabMethod() const;  
         // Affecte TRUE si le démon doit réagir lors d'une affectation d'un tableau d'instances de [C]DescValueSpec[/C] comme valeur.
   void     HasSetDescValueSpecTabMethod(Boolean);

         // Affecte TRUE si le démon doit réagir lors d'un ajout d'une valeur à un tableau de [C]DescValueSpec[/C].
   void     HasAddDescValueSpecMethod(Boolean);
         // Renvoie TRUE si le démon réagit lors d'un ajout d'une valeur 'domaine de valeur' ; FALSE sinon.
   Boolean  HasAddDescValueSpecMethod() const;

         // Affecte TRUE si le démon doit réagir lors d'un retrait d'une valeur à un tableau de [C]DescValueSpec[/C].
   void     HasRemoveDescValueSpecMethod(Boolean);
         // Renvoie TRUE si le démon réagit lors d'un retrait d'une valeur 'domaine de valeur' ; FALSE sinon.
   Boolean  HasRemoveDescValueSpecMethod() const;

         // Renvoie TRUE si le démon réagit lors d'un accès à la valeur 'tableau de domaines de valeur' du descripteur; FALSE sinon.
   Boolean  HasGetDescValueSpecTabMethod() const; 
         // Affecte TRUE si le démon doit réagir lors d'un accès au tableau d'instances de [C]DescvalueSpec[/C] qui est la valeur du descripteur.
   void     HasGetDescValueSpecTabMethod(Boolean);   

/**@name Affectation des m\'ethodes d\'eclench\'ees automatiquement */ 
//@{
         ///\index{méthodes!AssignWhenSetDescValueSpecTabMethod}
   void      AssignWhenSetDescValueSpecTabMethod(pDescValueSpecTab* (* function) (MonitorMethod*));
         ///\index{méthodes!AssignWhenGetDescValueSpecTabMethod}
   void      AssignWhenGetDescValueSpecTabMethod(pDescValueSpecTab* (* function) (MonitorMethod*));
         ///\index{méthodes!AssignWhenAddDescValueSpecMethod}
   void      AssignWhenAddDescValueSpecMethod(DescValueSpec* (* function) (MonitorMethod*));
         ///\index{méthodes!AssignWhenRemoveDescValueSpecMethod}
   void      AssignWhenRemoveDescValueSpecMethod(DescValueSpec* (* function) (MonitorMethod*));
//@}

/**@name  D\'esaffectation des m\'ethodes */ 
//@{
         ///\index{méthodes!UnassignWhenSetDescValueSpecTabMethod!}
   void      UnassignWhenSetDescValueSpecTabMethod();

         ///\index{méthodes!UnassignWhenGetDescValueSpecTabMethod} 
   void      UnassignWhenGetDescValueSpecTabMethod();
         ///\index{méthodes!UnassignWhenAddDescValueSpecMethod} 
   void      UnassignWhenAddDescValueSpecMethod();
         ///\index{méthodes!UnassignWhenRemoveDescValueSpecMethod} 
   void      UnassignWhenRemoveDescValueSpecMethod();
 
//@} 

protected:
   Boolean         mHasSetDescValueSpecTabMethod;
   Boolean         mHasGetDescValueSpecTabMethod;
   Boolean         mHasAddDescValueSpecMethod;
   Boolean         mHasRemoveDescValueSpecMethod;

private:
};

//@} // fin classe dérivée DescValueSpecTab

//@} // fin classes dérivées

//@} // fin DescValueMonitor

// INLINE METHODS

// EXTERNAL REFERENCES



#endif  // _DESCVALUEMONITOR_HH
