//============================================================================
// FILE: EntitySpec.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe des spécifications d'ensembles
//              d'entités
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _ENTITYSPEC_HH
#define _ENTITYSPEC_HH

// SYSTEM INCLUDES
//
//#include <>                                 

// PROJECT INCLUDES
//
#include "CoreTypes.h"

/**@name CDPFORTEXSpécifications d'ensembles d'entités
\begin{itemize}\item Directive d'inclusion : 
[P]
\#include EntitySpec.h
[/P]
\end{itemize}\index{spécification d'entités}
Cette classe permet de spécifier un ensemble d'entités, c'est-à-dire de définir cet ensemble sans en expliciter les éléments {\em a priori}.\\
L'invocation de sa méthode prédéfinie dite d'{\em expansion} ([Y]Expand[/Y])\index{expansion} permet de générer de façon dynamique l'ensemble d'entités correspondant à la spécification dans le contexte courant. Les éléments de la liste générée seront nécessairement des instances, directes ou indirectes, d'une même sous-classe de [C]Entity[/C], appelée ici {\em classe de référence}\index{classe de référence}. Pour que la liste générée contienne des entités de différentes classes, la classe de référence doit en être une sur-classe commune. \\
L'utilisateur crée une instance de [C]EntitySpec[/C] pour chaque spécification différente dans son domaine. Cette instance porte alors la spécification. Il peut aussi créer une sous-classe [C]EntitySpec[/C], et coder dans son constructeur la spécification. Il suffit alors d'instancier la sous-classe, par un simple appel au constructeur. Les constructeurs des sous-classes seront placées, typiquement, dans [F]UserEntitySpec.cc[/F].

{\large {\tt Trois manières de spécifier un ensemble d'entités}}

La définition de l'ensemble d'entités est donnée par l'utilisateur sous l'une des trois formes suivantes, mutuellement exclusives et liées à deux valeurs différentes du {\em mode} de la spécification.
\begin{itemize}
\item un {\em prédicat de sélection} qui, appliqué à une instance de la {\em classe de référence}, renvoie {\em vrai} si l'instance doit figurer dans l'ensemble. La méthode d'{\em expansion} invoque successivement ce prédicat pour toutes les instances (directes et indirectes) de la {\em classe de référence}. Le {\em mode} affecté est {\small {\tt RETURN_INSTANCES}}. On peut souhaiter que, parmi les instances de la {\em classe de référence} satisfaisant le {\em prédicat de sélection}, toutes figurent dans la liste expansée (on construit alors la spécification avec le mot-clé {\small {\tt ALLENT}}), ou bien seulement au plus les {\em n} premières rencontrées (avec le mot-clé {\small {\tt NFIRST}} et un {\em effectif} {\em n} strictement positif). La méthode d'{\em expansion} n'invoque plus le prédicat dès que {\em n} entités ont été sélectionnées.\label{docEntitySetSpec} 
\item une {\em fonction de construction} qui crée elle-même une instance de la {\em classe de référence}. Le {\em mode} affecté  est {\small {\tt INSTANTIATE}}. La méthode d'{\em expansion} invoque {\em n} fois cette fonction, {\em n} étant la valeur de l'{\em effectif}, crée la liste des instances créées et la retourne. 
\item une {\em fonction de construction} qui crée elle-même la liste des {\em n} instances de la {\em classe de référence}, {\em n} étant la valeur de l'{\em effectif}. Le {\em mode} affecté est {\small {\tt RETURN_INSTANCES}} ou {\small {\tt INSTANTIATE}}. La méthode d'{\em expansion} invoque une seule fois cette fonction, et retourne la liste de ces instances. 
\end{itemize}
 {\large {\tt Un premier complément}} de la spécification d'un ensemble d'entités est l'{\em entité de référence}. En effet, on peut vouloir sélectionner parmi les instances de la {\em classe de référence}, celles qui ont un certain lien fonctionnel ou structurel avec une autre, appelée l'{\em entité de référence}. Le moteur de \D passe cette information au {\em prédicat de sélection} ou à la {\em fonction de construction} sous la forme d'un argument prédéfini, de type [C]Entity[/C] bien entendu.\\

 {\large {\tt Un second complément}} sont les {\em spécifications de domaines de valeurs}\index{spécification de domaines de valeurs} (voir la classe [C]DescValueSpec[/C] à la page 'La classe de base des spécifications de domaines de valeurs'). L'utilisateur ajoute des spécifications de domaines de valeurs à la spécification d'entités en invoquant la méthode [Y]AddDescValueSpec[/Y].  
\begin{itemize}
\item Si le {\em mode} est {\small {\tt RETURN_INSTANCES}}, la méthode d'{\em expansion} commence par exploiter, en conjonction, les {\em spécifications de domaines de valeurs} pour déterminer si une instance peut être retenue sur la base de ses valeurs courantes pour certains descripteurs. Le {\em prédicat de sélection} intervient ensuite comme une contrainte supplémentaire. Lorsqu'il utilisera des spécifications de domaines de valeurs, l'utilisateur mentionnera cependant fréquemment un prédicat renvoyant toujours {\em vrai}.  
\item Si le {\em mode} est {\small {\tt INSTANTIATE}}, les {\em spécifications de domaines de valeurs} permettent à la méthode d'{\em expansion} de donner des valeurs particulières aux attributs des instances créées. Pour un attribut donné, la valeur affectée est choisie arbitrairement dans l'ensemble de valeurs spécifié (par la méthode [/Y]GetValueInDomain[/Y]).
\end{itemize}

{\large {\tt Les étapes du codage}}

Les éléments de code qui permettent la sélection ou la création d'entités sur la base d'une spécification sont les suivants. Dans cet exemple, on s'intéresse à des instances de la classe {\small {\tt A_CLASS}}.
\begin{enumerate}
\item L'éventuelle création d'un domaine de valeurs pour un ou plusieurs descripteurs de {\small {\tt A_CLASS}}. Avec la ligne suivante, on impose que le descripteur de symbole {\small {\tt DESC_1}} ait sa valeur à l'intérieur ({\small {\tt IN}}) de l'intervalle [1 5] :
\begin{verbatim}
[P]DescIntValueSpec* mySelectorDomain = new DescIntValueSpec(DESC_1, IN, 1, 5);[/P]
\end{verbatim} 

\item L'éventuelle création d'une fonction de sélection (dans le cas d'une sélection d'entités existantes) ou d'une fonction de construction (pour des instances qui n'existent pas encore). On place cette fonction dans [F]UserEntitySpec.cc[/F]. Dans le code ci-dessous, on sélectionne les entités qui, en plus de satisfaire d'éventuelles contraintes spécifiées comme ci-dessus, ont 1 comme valeur du descripteur de symbole {\small {\tt DESC_2}}, en excluant une instance désignée, passée en argument :
\begin{verbatim}
[P]bool mySelectorPredicate(EntitySpecMethod* pM) {
  Entity* pEref = pM->GetEntityArgValue(REFERENCEENTITY_ARGUMENT);
  Entity* pE = pM->GetEntityArgValue(CANDIDATEENTITY_ARGUMENT);
  if((pE->GetIntValue(DESC_2) == 1)
     &&
     (pE != pEref))
    return TRUE;
  else
    return FALSE;
}[/P]
\end{verbatim}

Noter que cette fonction est le corps d'une méthode [C]EntitySelector[/C] (sous-classe de [C]EntitySpecMethod[/C]) dont une instance est un descripteur fonctionnel de la classe [C]EntitySpec[/C]. 

\item La création d'une instance de [C]EntitySpec[/C], en précisant la classe de référence, celle des entités renvoyées :
\begin{verbatim}
[P]  EntitySpec pES = new EntitySpec(A_CLASS);[/P]
\end{verbatim}

\item L'éventuel attachement des domaines de valeurs et/ou de la fonction (de sélection ou de construction) créés :
\begin{verbatim}
[P]  pES->AssignSelectorPredicate(mySelectorPredicate);
  pES->AddDescValueSpec(mySelectorDomain);[/P]
\end{verbatim}

\item La sélection, puis la récupération du résultat s'opèrent par :
\begin{verbatim}
[P]  Entity* pEref = GetNamedEntity(A_CLASS, "aClass_245");
  pES->SetEntityArgValue(ENTITY_SELECTOR, REFERENCEENTITY_ARGUMENT, pEref);
  pES->Expand();
  pEntityTab* result = pES->GetExpansion();
  Entity* pE_0 = result->get_nth(0);
  int val1 = pE_0->GetIntValue(DESC_1);
  int val2 = pE_0->GetIntValue(DESC_2);
  printf("\npour '%s' DESC_1 = %d DESC_2 = %d", pE_0->InstanceName(), val1, val2);[/P]
\end{verbatim}
\begin{verbatim}
>>
pour 'aClass_247' DESC_1 = 4 DESC_2 = 1
\end{verbatim}

Noter qu'un {\em corps} n'a pas été donné à la méthode [Y]QuantityEvaluator[/Y], fixant un nombre maximum. On sélectionne donc toutes les entités satisfaisant la spécification.

\end{enumerate}

{\large {\tt Spécifications alternatives}}

Soit la fonction suivante de détermination du nombre maximum d'instances a créer ou sélectionner :
\begin{verbatim}
[P]int myQuantityEvaluator(EntitySpecMethod* pM) {
  int result;
  ... // valuation du nombre requis maximum
  return result;
}[/P]
\end{verbatim}

Exemple de fonction d'instanciation :
\begin{verbatim}
[P]  EntitySpec pES = new EntitySpec(A_CLASS);
  pES->AssignEntityInstantiator(myEntityInstantiator);

BasicEntity* myEntityInstantiator(EntitySpecMethod* pM) {
  Entity* result;
  Entity* pEref = pM->GetEntityArgValue(REFERENCEENTITY_ARGUMENT);
  UserClassId id = pM->GetIntArgValue(CLASSID_ARGUMENT);

  result = App_NewEntityFromID(id);
  result->SetIntVarValue(DESC_1, pEref->GetIntVarValue(DESC_1));
  return result;
}[/P]
\end{verbatim}

Exemple de fonction de création de liste d'instances existantes :
\begin{verbatim}
[P]  EntitySpec pES = new EntitySpec(A_CLASS);
  pES->AssignEntityListInstantiator(myEntityListInstantiator);
  pES->AssignQuantityEvaluator(myQuantityEvaluator);

pEntityTab* myEntityListInstantiator(EntitySpecMethod* pM) {
  EntitySpec* pES = pM->DescribedEntitySpec();
  pEntityTab* result = pES->GetExpansion();
  result->erase();
  //Entity* pEref = pM->GetEntityArgValue(REFERENCEENTITY_ARGUMENT);
  UserClassId id = pM->GetIntArgValue(CLASSID_ARGUMENT);
  int N = pM->GetIntArgValue(LISTSIZE_ARGUMENT);

  pEntityTab* instanceList = GetEntityList(id); // les instances existantes
  for(int k=0;k<N;k++) {
    Entity* pE_k = instanceList->get_nth(k);
    result->push_back(pE_k);
  }
  return result;
}[/P]
\end{verbatim}

Dans ce code ci-dessus, la valeur passée par l'argument de symbole {\small {\tt LISTSIZE_ARGUMENT}} a été établie par le moteur en invoquant la méthode [Y]QuantityEvaluator[/Y], puisqu'un {\em corps} lui a été donné ( par [Y]AssignQuantityEvaluator[/Y]).

Exemple de fonction de création de liste de nouvelles instances :
\begin{verbatim}
[P]  EntitySpec pES = new EntitySpec(A_CLASS);
  pES->AssignEntityListInstantiator(myEntityListInstantiator);
  pES->IntQuantity(3);

pEntityTab* myEntityListInstantiator(EntitySpecMethod* pM) {
  EntitySpec* pES = pM->DescribedEntitySpec();
  pEntityTab* result = pES->GetExpansion();
  result->erase();
  //Entity* pEref = pM->GetEntityArgValue(REFERENCEENTITY_ARGUMENT);
  UserClassId id = pM->GetIntArgValue(CLASSID_ARGUMENT);
  int N = pM->GetIntArgValue(LISTSIZE_ARGUMENT);

  for(int k=0;k<N;k++) {
    Entity* pE_k = App_NewEntityFromID(id);  // création nouvelle instance
    result->push_back(pE_k);
  }
  return result;
}[/P]
\end{verbatim}

Dans le code ci-dessus, la valeur passée par l'argument de symbole {\small {\tt LISTSIZE_ARGUMENT}} résulte du message [Y]IntQuantity[/Y] adressé à {\small {\tt pES}}, avec la valeur {\em 3}.\\

Note importante : dans les deux fonctions de création de liste, ci-dessus, on reconstruit la liste qui est le membre de la classe [C]EntitySpec[/C] renvoyé par le service [Y]GetExpansion[/Y]. Avant de la reconstruite, on prend soin de la vider (par [Y]erase[/Y]).
*/
//@{
           ///
class EntitySpec 
{
public:

  /**@name LIFECYCLE */
//@{
          /// Constructeur de base. La valeur RETURN_INSTANCES est affectée au mode. La valeur ALLENT est affectée au {\em quantificateur}. Aucune méthode n'est dotée d'un corps.
  EntitySpec();  // _no swig_

          /// Constructeur de base, avec le symbole de la classe puits. La valeur RETURN_INSTANCES est affectée au {\em mode}. La valeur ALLENT est affectée au {\em quantificateur}. Aucune méthode n'est dotée d'un corps.
   EntitySpec(UserClassId);

          /// Constructeur de base, avec un {\em prédicat de sélection} en quatrième argument. La valeur RETURN_INSTANCES est affectée au {\em mode}. Le deuxième argument est NFIRST ou ALLENT. Le troisième argument est l'effectif {\em n}. Il doit être strictement positif si et seulement si le second argument est {\em NFIRST} et qu'aucune fonction d'évaluation de l'effectif, assignée par la méthode [Y]AssignQuantityEvaluator[/Y], n'est spécifiée en cinquième argument (valeur NULL pour {\em h}). Le sixième argument est l'entité de référence. Le prédicat de sélection est assigné par la méthode [Y]AssignSelectorPredicate[/Y].
   EntitySpec(UserClassId, QuantificatorId, int n,                                              bool(* predicat) (EntitySpecMethod*),                                           int (* h) (EntitySpecMethod*),                                                  BasicEntity*);  // _no swig_

          /// Constructeur de base, avec une {\em fonction de construction} en quatrième argument ({\em f}). La valeur INSTANTIATE est affectée au {\em mode}. Le deuxième argument est NFIRST. Le troisième argument est l'effectif {\em n}. Il doit être strictement positif si aucune fonction d'évaluation de l'effectif, assignée par la méthode [Y]AssignQuantityEvaluator[/Y], n'est spécifiée en cinquième argument (valeur NULL pour {\em h}). Le sixième argument est l'entité de référence. La fonction de construction est assignée par la méthode [Y]AssignEntityInstantiator[/Y]. Elle construit les intances une à une et est donc invoquée {\em n} fois lors de l'expansion. Si {\em f} a la valeur NULL, [Y]Expand[/Y] invoque [Y]NewEntityFromId[/Y]({\em id}) (voir page 'Fonctions globales/Création d'instances d'entités')
   EntitySpec(UserClassId id, QuantificatorId, int n,                                           BasicEntity* (* f) (EntitySpecMethod*),                                         int (* h) (EntitySpecMethod*),                                                  BasicEntity*);  // _no swig_

          /// Constructeur de base, avec une {\em fonction de construction} en quatrième argument ({\em g}). Cette fonction construit elle-même la liste des {\em n} intances qui sera renvoyée par [Y]Expand[/Y], après examen des spécifications de domaines de valeurs. La valeur INSTANTIATE est affectée au mode. Le deuxième argument est NFIRST. Le troisième argument est l'effectif {\em n}. Il doit être strictement positif si aucune fonction d'évaluation de l'effectif, assignée par la méthode [Y]AssignQuantityEvaluator[/Y], n'est spécifiée en cinquième argument (valeur NULL pour {\em h}). Le sixième argument est l'entité de référence. La fonction de construction est assignée par la méthode [Y]AssignEntityListInstantiator[/Y]. 
   EntitySpec(UserClassId, QuantificatorId, int n,                                              pEntityTab* (* g) (EntitySpecMethod*),                                          int (* h) (EntitySpecMethod*),                                                  BasicEntity*);  // _no swig_

          /// Constructeur par copie. Les entités de la liste expansée (renvoyée par [Y]GetExpansion[/Y]) ne sont pas copiées. Autrement dit, cette liste est vide dans la copie. Une nouvelle liste peut être générée par un appel à [Y]Expand[/Y] appliqué à la copie. Par contre, la copie possède les mêmes spécifications de domaines de valeurs que l'instance d'origine copiée.
   EntitySpec(const EntitySpec&);  // _no swig_
   
          /// Destructeur. Ni les entités de la liste expansée ni les spécifications de domaines de valeurs éventuellement attachées ne sont détruites.
   ~EntitySpec();   
   /// Affecte à la classe le nom donné en argument. Cette méthode doit être invoquée dans le constructeur de [C]EntitySpec[/C] qui reçoit le nom de classe en argument ou explicitement dans le constructeur des sous-classes. Il est nécessaire d'affecter un nom de classe dans le cas où un corps d'une méthode est externalisé (voir la section sur la classe [C]Swig[/C], page 'Externalisation de données fonctionnelles').
   void            ClassName(char*);  // _no swig_      
          /// Renvoie le nom de classe de la spécification.\index{méthodes!ClassName - EntitySpec}
   char*           ClassName() const; 

//@}                             

// @Man: OPERATORS
// @{

         // Opérateurs < et > (pour la forme).
  Boolean         operator < (const EntitySpec&) const {return TRUE;};
  Boolean         operator > (const EntitySpec&) const {return TRUE;};

// @}  

  /**@name [SC]Identification[/SC] */
//@{  
         //  Renvoie le symbole de classe de la classe dans laquelle sont recherchés les entités à placer dans la liste expansée.
  UserClassId        ReferenceClassId(); // _no swig_  
         /// Renvoie le symbole de classe de la classe dans laquelle sont recherchés les entités à placer dans la liste expansée. Si un symbole n'a pas été précisé, on l'établit à partir du nom de classe. Si celui-ci n'a pas été fourni, l'exception [Y]NoInformationForReferenceClass[/Y] est levée.\index{méthodes!GetReferenceClassId}
  UserClassId        GetReferenceClassId(); // _no swig_
         /// Etablit le symbole de classe de la classe dans laquelle sont recherchés les entités à placer dans la liste expansée.\index{méthodes!ReferenceClassId}
  void               ReferenceClassId(UserClassId); 
         //  Renvoie le nom de classe de la classe dans laquelle sont recherchés les entités à placer dans la liste expansée.
  char*              ReferenceClassName(); // _no swig_
         /// Etablit le nom de classe de la classe dans laquelle sont recherchés les entités à placer dans la liste expansée.\index{méthodes!ReferenceClassName}
  void               ReferenceClassName(char*);
         //
  EntitySpecMode     GetMode(); // _no swig_
          /// Etablit le {\em mode} de la spécification, qui détermine si les instances à retourner sont à créer ou non.\index{méthodes!SetMode}
  void               SetMode(EntitySpecMode);
         //
  QuantificatorId    GetQuantificatorId(); // _no swig_
         /// Etablit le {\em quantificateur} de la spécification, qui détermine si toutes les instances sélectionnées sont à retourner ou bien seulement les {\em n} premières trouvées.\index{méthodes!SetQuantificatorId}
  void               SetQuantificatorId(QuantificatorId);
         //
  int                IntQuantity(); // _no swig_
         /// Etablit la taille maximale de la liste d'entités à construire, parmi les entités satisfaisant la spécification.\index{méthodes!IntQuantity}
  void               IntQuantity(int);
         //
  BasicEntity*            ReferentEntity(); // _no swig_
         /// Etablit l'{\em entité de référence} de la spécification.\index{méthodes!ReferentEntity}
  void               ReferentEntity(BasicEntity*);
         /// Renvoie la liste des entités qui correspondent à la spécification. Cette liste est construite par un appel à [Y]Expand[/Y]. Attention ! Il ne s'agit pas d'une copie : cette liste ne doit être modifiée qu'en connaissance de cela. Notamment, les corps de la méthode [Y]EntityListInstantiator[/Y] vide généralement cette liste avant de la recompléter.\index{méthodes!GetExpansion}
  pEntityTab*        GetExpansion();
         //
  int                ExpandedTabSize(); // _no swig_

//@}                            

/**@name OPERATIONS */
//@{   
   void                  AddInstance(EntitySpec*); // _no swig_
   void                  RemoveInstance(EntitySpec*);  // _no swig_ 

          /// Ajoute à la présente spécification une spécification de domaine de valeur, après celles éventuellement déjà ajoutées. \index{méthodes!AddDescValueSpec}
  void AddDescValueSpec(DescValueSpec*);

          /// Génère la liste des entités qui correspondent à la spécification. Affecte cette valeur à l'attribut de la présente spécification renvoyé par un appel à [Y]GetExpansion[/Y]. Selon le mode de la spécification, [Y]Expand[/Y] invoque le prédicat de sélection sur les instances existantes de la classe de référence, ou bien la fonction d'instanciation (celle assignée par [Y]AssignEntityInstantiator[/Y] ou par [Y]AssignEntityListInstantiator[/Y]).\index{méthodes!Expand}
  void  Expand();
//@}                              

// ACCESS TO FLAGS

         // Renvoie TRUE si la spécification est dotée d'un prédicat de sélection ; FALSE sinon.
   Boolean  HasEntitySelectorMethod() const; // _no swig_  

         // Renvoie TRUE si la spécification est dotée d'une fonction de construction ; FALSE sinon.
   Boolean  HasEntityInstantiatorMethod() const;  // _no swig_ 

         // Renvoie TRUE si la spécification est dotée d'une fonction de construction de liste ; FALSE sinon.
   Boolean  HasEntityListInstantiatorMethod() const; // _no swig_

         // Renvoie TRUE si la spécification est dotée d'une fonction qui renvoie le cardinal de la liste d'entités à construire ; FALSE sinon.
   Boolean  HasQuantityEvaluatorMethod() const;    // _no swig_       
 
/**@name [SC]Affectation des méthodes[/SC] */
//@{    
          /// Affecte à la spécification un prédicat qui détermine si une entité correspond à la spécification. L'entité est la valeur de l'argument de {\em pM} de symbole CANDIDATEENTITY_ARGUMENT. Un autre argument, de symbole REFERENCEENTITY_ARGUMENT, a pour valeur une entité quelconque qui peut servir de référence dans le corps du prédicat.\index{méthodes!AssignSelectorPredicate}
void  AssignSelectorPredicate(bool(* function) (EntitySpecMethod* pM)); // _no swig_

          /// Affecte à la spécification une fonction qui construit une entité d'une certaine classe, celle dont le symbole est la valeur de l'argument de {\em pM} dont le symbole est CLASSID_ARGUMENT. Un autre argument, de symbole REFERENCEENTITY_ARGUMENT, a pour valeur une entité quelconque qui peut servir de référence dans le corps de la fonction.\index{méthodes!AssignEntityInstantiator}
void  AssignEntityInstantiator(BasicEntity* (* function) (EntitySpecMethod* pM)); // _no swig_

          /// Cette fonction est analogue à la précédente, à ceci près qu'elle affecte à la spécification une fonction qui construit la liste complète des entités de la classe dont le symbole est la valeur de l'argument de {\em pM} dont le symbole est CLASSID_ARGUMENT. Un autre argument de {\em pM}, de symbole LISTSIZE_ARGUMENT, est la taille de la liste à construire.\index{méthodes!AssignEntityListInstantiator}
void  AssignEntityListInstantiator(pEntityTab* (* function)(EntitySpecMethod* pM)); // _no swig_

          /// Affecte à la spécification une fonction qui renvoie le nombre d'entités que doiAffecte à la spécification une fonction qui renvoie le nombre d'entités que doit contenir, au maximum, la liste renvoyée par [Y]Expand[/Y]. Cette fonction n'est invoquée que si l'effectif {\em n}, celui passé en paramètre du constructeur ou établi par par [Y]IntQuantity[/Y], n'est pas strictement positif. L'entité qui est la valeur de l'argument de {\em pM} de symbole REFERENCEENTITY_ARGUMENT est une entité quelconque susceptible de renseigner la détermination de l'effectif.\index{méthodes!AssignQuantityEvaluator}
  void  AssignQuantityEvaluator(int (*function) (EntitySpecMethod* pM)); // _no swig_


         // Renvoie la méthode dont le symbole de classe est passé en argument.
   Method*             GetMethod(UserMethodId);  // _no swig_
         // Renvoie la méthode dont le nom de classe est passé en argument.
   Method*             GetMethod(char*); // _no swig_
         // Renvoie la table des méthodes.
   pMethodTab          MethodTab() const; // _no swig_
         // 
   TableauInt          DualMethodTab() const;  // _no swig_
//@}   

// @Man: Affectation d'un corps aux m\'ethodes et de valeurs aux arguments. 
/* *Les méthodes [Y]ExecCorePredicate[/Y] ... [Y]ExecEntityTabCoreMethod[/Y] exécutent le corps de la méthode dont le symbole est passé en argument. \index{méthodes!ExecCorePredicate - Monitor}\index{méthodes!ExecEntityTabCoreMethod - Monitor}
*/ 
// @{ 
         // 
   bool            ExecCorePredicate(UserMethodId); // _no swig_  
         // 
   int             ExecIntCoreMethod(UserMethodId);  // _no swig_ 
         // 
   BasicEntity*         ExecEntityCoreMethod(UserMethodId); // _no swig_ 
         // 
   pEntityTab*     ExecEntityTabCoreMethod(UserMethodId); // _no swig_
          
// @}    

// @Man: Affectation de m\'ethodes 
/* 
 */
// @{
         // Ajoute la méthode à la liste des méthodes de la spécification.
   void                AddMethod(Method*); // _no swig_
// @} 
 
    // comptage Const/Dest
    static int   getNbConst() {return _nbConst;}
    static int   getNbDest() {return _nbDest;}     

    // comptage instances
   //static int                   InstanceNumber(); 
   //static pEntitySpecTab        InstanceTab();    
   static EntitySpec*           GetInstance(int);   

   static int                    mInstanceNumber;        
   static pEntitySpecTab         mpInstanceTab; 

protected:

   int static   _nbConst;
   int static   _nbDest; 

   UserClassId            mReferenceClassId;
   char*                  mReferenceClassName;
   EntitySpecMode         mEntitySpecMode;
   QuantificatorId        mQuantId;
   int                    mQuantity;
   BasicEntity*           mpReferentEntity;

   pDescValueSpecTab      mpDescValueSpecTab;
   pEntityTab*            mpExpandedEntityTab;
   // pEntityTab* pour pouvoir être directement la valeur d'un appel à 
   // ExecEntityTabCoreMethod(ENTITYLIST_INSTANTIATOR)

   pMethodTab             mpMethodTab;    // + JPR 0211
   TableauInt             mpDualMethodTab; 

   char*           mThisEntitySpecClassName;

   Boolean         mHasEntitySelectorMethod;
   Boolean         mHasEntityInstantiatorMethod;
   Boolean         mHasEntityListInstantiatorMethod;
   Boolean         mHasQuantityEvaluatorMethod;

private:
};  // _end class_

//@}  // fin classe

// INLINE METHODS

// EXTERNAL REFERENCES


//-X///////////////X/////////////////////////////X////////////////////////////


#endif  // _ENTITYSPEC_HH
