//============================================================================
// FILE: Event.h
//
// TYPE: C++CLASS 
// DESCRIPTION: entête de la classe générique des événements
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _EVENT_HH
#define _EVENT_HH

// SYSTEM INCLUDES
//                               
  
// PROJECT INCLUDES
//
#include "CoreTypes.h"
#include "CoreTime.h"

/**@name Les \'ev\'enements 

L'occurrence d'un événement provoque l'exécution, ou bien le déclenchement et l'entretien, de processus de changement d'état du système simulé. En conséquence, un système qui ne serait soumis à l'occurrence d'aucun événement resterait dans son état initial. Un événement qui ne comporterait aucune directive sur un processus ne peut être placé dans l'agenda de la simulation.\\

Les sous-classes d'événements propres au système simulé sont déclarées dans le fichier typiquement nommé [F]UserEvent.h[/F] et réalisées dans le fichier [/F]UserEvent.cc[/F]. Leurs constructeurs contiennent essentiellement l'attribution d'une date d'occurrence, d'un degré de priorité\index{degré de priorité} par rapport aux autres événements survenant à cette date, la fabrication de la liste des couples \{processus, action\} qui en sont la conséquence, et l'affectation, si besoin est, des méthodes de postconséquence et d'autogénération.  Attention ! Plus la valeur du degré de priorité est faible, plus la priorité est forte (un événement de degré 0 est plus prioritaire d'un autre de degré 1). La priorité par défaut est 99.\\

Les méthodes de postconséquence et d'autogénération sont des instances de la classe [Y]EventMethod[/Y]. Elles sont affectées à l'événement, avec un corps vide, dans le constructeur de base de l'événement. Un corps spécifique leur est ensuite affecté par invocation d'une des méthodes [Y]SetPostConsequence[/Y], [Y]Set\-Generate\-NextEvent[/Y], [Y]SetUpdateNextEvent[/Y] et [Y]SetDefineNextEventClockTime[/Y].\\ 

L'utilisateur crée des instances d'événements au moyen d'un constructeur (voir section LIFECYCLE) et les range dans l'agenda d'événements. Cette création peut avoir lieu avant le début de la simulation, ou bien pendant la simulation comme partie de l'effet de processus. Le moteur de simulation peut lui-aussi créer des instances de sous-classes de [C]Event[/C], dans trois situations :
\begin{itemize}

\item Dans le cas où l'utilisateur a écrit une fonction de génération automatique \index{génération automatique} (par l'une ou l'autre des méthodes [Y]SetGenerateNextEvent[/Y], [Y]SetUpdateNextEvent[/Y] et [Y]SetDefineNextEventClockTime[/Y]). Le première fonction code, à partir d'un événement, la génération d'un nouvel événement d'une classe différente (dont le nom de classe est renvoyé par [Y]NextEventClassName()[/Y]) ou de la même classe (dans ce dernier cas : obligatoirement par [Y]RawCopy[/Y] appliquée à l'événement d'origine). Les deux autres fonctions sont d'utilisation exclusive l'une de l'autre. 
\noLF
L'une ou l'autre opère sur un événement généré par [Y]SetGenerateNextEvent[/Y], ou bien, par défaut, par le moteur de simulation. Tout cela permet à l'utilisateur de définir un enchaînement d'événements dans le temps, sans avoir à donner explicitement la liste complète des événements. La fonction de génération automatique, si elle est présente, est appelée par le moteur après l'exécution des conséquences (en termes de processus) de l'occurrence de l'événement. Typiquement, cette fonction peut contenir un appel tel que :
\begin{verbatim}
[P]  RandomInteger(self->MaxDelayNextEvent() - self->MinDelayNextEvent());[/P]
\end{verbatim}
où {\small {\tt self}} est lié à l'instance de l'événement qui vient de survenir, et [Y]RandomInteger[/Y] est une fonction qui renvoie un nombre entier aléatoire entre 0 et son argument. Ce nombre sert à définir la date du nouvel événement.

\item Dans le cas où un processus de l'événement a vu sa précondition\index{précondition} (d'initialisation ou d'exécution) non satisfaite. Le système crée alors un nouvel événement de la même classe pour un instant ultérieur (mise en attente du processus, voir la méthode [Y]SetPrecondition[/Y] de la classe [C]Process[/C]). Si l'événement est doté d'une fonction d'autogénération, celle-ci ne sera invoquée qu'en l'instant où la précondition deviendra satisfaite, et non pas en l'instant d'occurrence initialement programmé (dont la valeur est perdue et ne peut donc pas être exploitée dans la fonction d'autogénération).

\item Dans le cas où l'utilisateur a écrit une fonction de {\em postconséquence}\index{postconséquence} (par la méthode [Y]SetPostConséquence[/Y]). Le corps d'une telle fonction est libre. Il est donc possible d'y coder l'occurrence d'un événement, mais on préfèrera pour cela utiliser les fonctions présentées plus haut.

\end{itemize}
En résumé, le cycle de vie d'un événement est le suivant :
\begin{enumerate}
\item création et insertion dans l'agenda
\item dépilement de l'événement lorsqu'il arrive en la position de tête : {\em current} 
\item tentative exécution des directives sur les processus
  \begin{itemize}
  \item si l'événement est différé (précondition non satisfaite)
    \begin{itemize}
    \item création d'un événement {\em new} par 
      \begin{itemize}
      \item copie des processus de {\em current} (en conservant les mêmes méthodes)
      \item copie de {\em current} (en conservant les mêmes méthodes), avec les processus copiés
      \end{itemize}
    \item suppression des processus de {\em current} (sans supprimer les méthodes), et suppression de {\em current} par [Y]RawCopy[/Y] (sans supprimer les méthodes, lesquelles sont désormais attachées à {\em new})
    \item insertion de {\em new} dans l'agenda
    \end{itemize}
  \item si les directives sur les processus sont exécutées (précondition satisfaite)
    \begin{itemize}
    \item exécution de la post-conséquence
    \item tentative d'autogénération si c'est spécifié
       \begin{itemize}
       \item si un nouvel événement {\em new} a été généré 
          \begin{itemize}
	  \item suppression des processus de {\em current} (sans supprimer les méthodes), et suppression de {\em current} (sans supprimer les méthodes)
	  \item insertion de {\em new} dans l'agenda
          \end{itemize}
       \item sinon 
          \begin{itemize}
	  \item suppression des processus de {\em current} (et de leurs méthodes), et suppression de {\em current} (et de ses méthodes)
          \end{itemize}
       \end{itemize}
    \end{itemize}
  \end{itemize} 
\item fin cycle
\end{enumerate}
*/
//@{  // fermeture dans CoreEvent.h

/**@name CDPFORTEXLa classe des évéenements
\begin{itemize} \item Directive d'inclusion : 
[P]
\#include Event.h
[/P]
\end{itemize}
*/
//@{
        ///
class Event 
{
public:

  /**@name LIFECYCLE */
//@{
          // Constructeur de base.
  Event();  // _no swig_ 
          // Constructeur de base sans méthode. Appel exclusif dans RawCopy avec l'argument 1. Les méthodes sont affectées dans la suite de RawCopy.
   Event(bool);  // _no swig_                 
          /// Constructeur avec affectation d'un nom de classe par [Y]ClassName[/Y].
   Event(char*);  // _no swig_                  
          /// Constructeur avec affectation d'un symbole de classe par [Y]ClassSymbol[/Y] et d'un nom de classe par [Y]ClassName[/Y].
   Event(UserEventId, char*);  // _no swig_                 
          // Constructeur avec affectation d'un nom de classe. Le second argument indique si l'événement est autogénérable. Le troisième argument indique si l'événement a des post-conséquences. Sert à ne pas créer d'instances de méthodes initilement (e.g. pour les événements propres au moteur (voir CoreEvent.h). 
   Event(char*, Boolean, Boolean);  // _no swig_                           
          /// Constructeur avec affectation d'un degré de priorité et d'un nom de classe par [Y]ClassName[/Y]. Lève l'exception [Y]UnexpectedPriorityDegree[/Y].\label{affectNomEvent}             
   Event(short, char*);   // _no swig_                    
          /// Constructeur par copie profonde (y compris copie profonde des processus associés) avec copie des méthodes.              
   Event(const Event&);  // _no swig_ 
          /// Constructeur par copie superficielle, c'est-à-dire sans copie des méthodes, car celles-ci deviennent aussi attachées à la copie. Les processus discrets associés sont copiés superficiellement par [Y]RawCopy[/Y] (c'est-à-dire sans copier leurs méthodes). Les processus continus associés sont copiés par leurs constructeur par copie (c'est-à-dire en copiant les méthodes).\index{méthodes!RawCopy - Event}          
   Event* RawCopy();

          /// Destructeur superficiel. Ne détruit pas les méthodes de l'événement. Les processus associés ne sont pas détruits. A utiliser sur un événement qui gère un processus aussi géré par d'autres événements. Les processus qui, de ce fait, resteraient encore en vie en fin de simulation seront détruits dans le destructeur profond de la simulation.
   ~Event();     // _no swig_ 
          /// Méthode de destruction. Détruit les instances de [C]Method[/C] attachées à l'événement. Les processus associés ne sont pas détruits, pour la même raison que ci-dessus.
   void DeleteInDepth();   // _no swig_

void               AddInstance(UserEventId id, char* name);
//@}

/**@name OPERATORS
*/
//@{
          /// Opérateur d'affectation. L'événement à gauche pointe sur les mêmes couples \{processus, action\} que l'événement à droite. Invoque les opérateurs d'affectation des méthodes. Renvoie une référence à l'entité d'origine.
   const
   Event&          operator=(const Event&); 

         // Opérateurs < et >.
  Boolean         operator < (const Event&) const;
  Boolean         operator > (const Event&) const;     
//@}

/**@name [SC]Identification et accès[/SC]  */
//@{ 
          /// Renvoie le symbole de classe de l'événement (une des valeurs énumérées de {\em EventId}). Cette énumération doit être faite par l'utilisateur dans le fichier [F]UserTypes.h[/F], sous le nom {\em EventId}.\index{méthodes!ClassSymbol - Event}
   UserEventId     ClassSymbol() const;
          /// Affecte à la classe le nom donné en argument. Ce nom de classe sera hérité par toutes les instances de la classe. Cette méthode doit être appelée dans les constructeurs d'événements (en invoquant un constructeur de [C]Event[/C] qui reçoit le nom de classe en argument ou explicitement dans le constructeur des sous-classes). C'est à l'intérieur de cette méthode qu'est réalisée la liaison de l'événement avec une instance de [C]EventOutputSpec[/C] dans le cas où l'utilisateur a demandé la sauvegarde de l'information sur la date de déclenchement (voir page 'Sauvegarde de la chronique des événements'). Pour une exploitation dans le fichier de spécifications de sorties, le nom ne doit comporter que des caractères ASCII\index{nom de classe (événement)}.
   void            ClassName(char*); // _no swig_      
          /// Renvoie le nom de classe de l'événement.\index{méthodes!ClassName - Event}
   char*           ClassName() const;
          // Renvoie le nom d'instance de l'événement.
   char*           InstanceName() const;   // _no swig_
          // Affecte le nom d'instance standard au récepteur du message
   void            SetInstanceName(char*, int);   // _no swig_
          /// Affecte un nom d'instance au récepteur du message, qui surcharge le nom d'instance par défaut attribué par le moteur\index{nom par défaut - Event}. Ce nom par défaut est la concaténation du nom de classe et d'un nombre entier positif unique sur l'ensemble de toutes les instances d'événements.\index{nom d'instance (événement)}\index{méthodes!SetInstanceName - Event}
   void            SetInstanceName(char*);
//@}

/**@name [SC]Priorité, probabilité[/SC]  */
//@{
          /// Affecte un degré de priorité donné en argument. La valeur doit être comprise entre {\em 0} (priorité forte) et {\em 99} (priorité faible). Lève l'exception [Y]UnexpectedPriorityDegree[/Y].\index{méthodes!SetPriorityDegree}
   void            SetPriorityDegree(short);
          /// Renvoie le degré de priorité de l'événement.\index{méthodes!PriorityDegree}
   short           PriorityDegree() const;

          /// Affecte une probabilité d'occurrence à l'événement. La valeur par défaut est {\em 1.0}.\index{méthodes!SetOccurrenceProbability}
   void            SetOccurrenceProbability(float);  
          /// Affecte une probabilité d'occurrence à l'événement (valeur flottante). L'argument doit être entre 0 et 100. La valeur affectée par défaut est {\em 1.0}.
   void            SetOccurrenceProbability(int);  
          /// Renvoie la probabilité d'occurrence de l'événement.\index{méthodes!GetOccurrenceProbability}
   float           GetOccurrenceProbability() const;
//@}

/**@name [SC]Date d'occurrence[/SC]  */
//@{ 
          /// Renvoie le moment d'occurrence de l'événement, en nombre d'unités de temps simulé depuis le début de la simulation.\index{méthodes!OccurrenceClockTime}
   int             OccurrenceClockTime() const;
          /// Renvoie le moment au plus tôt d'occurrence de l'événement, en nombre d'unités de temps simulé depuis le début de la simulation.\index{méthodes!MinOccurrenceClockTime}
   int             MinOccurrenceClockTime() const;  // + JPR 090727
          /// Renvoie le moment au plus tard d'occurrence de l'événement, en nombre d'unités de temps simulé depuis le début de la simulation.\index{méthodes!MaxOccurrenceClockTime}
   int             MaxOccurrenceClockTime() const;  // + JPR 090727   
   
          /// Détermine le moment d'occurrence de l'événement, en nombre d'unités de temps simulé depuis le début de la simulation.\index{méthodes!SetOccurrenceClockTime}
   void            SetOccurrenceClockTime(int);   
          /// Détermine le moment au plus tôt d'occurrence de l'événement, en nombre d'unités de temps simulé depuis le début de la simulation.\index{méthodes!SetMinOccurrenceClockTime}
   void            SetMinOccurrenceClockTime(int);  
          /// Détermine le moment au plus tard d'occurrence de l'événement, en nombre d'unités de temps simulé depuis le début de la simulation. Si cette méthode est invoquée (nécessairement après une invocation de [Y]SetMinOccurrenceClockTime[/Y]), le moment d'occurrence est calculé par la fonction assignée par [Y]SetOccurrenceClockTimeFunction[/Y].\index{méthodes!SetMaxOccurrenceClockTime}
   void            SetMaxOccurrenceClockTime(int);
          /// Calcule (et renvoie) la structure [C]tm[/C] décrivant la date à laquelle l'événement est prévu.\index{méthodes!OccurrenceTmDate}
   tm*             OccurrenceTmDate() const;
          /// Affecte une date à l'événement, en nombre d'unités de temps simulé depuis le début de la simulation. Les arguments sont, dans l'ordre le jour, le mois (ou le {\em ShortMonthName}) et le millésime (par exemple 2001). La valeur de l'argument 'millésime' peut être -9. Dans ce cas, le millésime sera supposé être celui de l'horloge de la simulation au moment de l'appel de la méthode.\index{méthodes!SetOccurrenceDate}
   void            SetOccurrenceDate(int, int, int);   // _no swig_ 
          /// Affecte une date à l'événement, en nombre d'unités de temps simulé depuis le début de la simulation. Les arguments sont, dans l'ordre le jour, le mois (ou le {\em ShortMonthName}), le millésime (par exemple 2001), l'heure, la minute et la seconde. La valeur de l'argument 'millésime' peut être -9. Dans ce cas, le millésime sera supposé être celui de l'horloge de la simulation au moment de l'appel de la méthode.
   void            SetOccurrenceDate(int, int, int, int, int, int);
          // Affecte une date à l'événement, en nombre d'unités de temps simulé depuis le début de la simulation. Les arguments sont, dans l'ordre l'heure, la minute et la seconde. Le jour de la date affectée est le jour où la simulation est effectuée.\index{méthodes!ThisDayOccurrenceDate}
   void            SetThisDayOccurrenceDate(int, int, int); // caché JPR 110607            MaxDelayNextEvent() const;

          /// Dote l'événement d'un corps de méthode qui détermine et renvoie le moment aléatoire d'occurrence de l'événement. Le moment est établi entre la date au plus tôt et la date au plus tard. Par défaut d'affectation d'un corps, la date sera déterminée sur la base d'une loi de probabilité uniforme entre les deux bornes. Le moment d'occurence est calculé lors de l'affectation (par [Y]SetMaxOccurrenceClockTime[/Y]) de la borne supérieure (il faut alors que la date au plus tôt ait été spécifiée), puis affecté par [Y]SetOccurenceClockTime[/Y]. \index{méthodes!SetRandomizeThisEventClockTime}
  void             SetRandomizeThisEventClockTime (int (* f)(EventMethod*)); // _no swig_
//@}


/**@name [SC]Processus et directives[/SC]  */
//@{ 
          /// Ajoute un élément en fin du tableau des couples \{processus, action\} qui sont la conséquence de l'événement. Lève l'exception [Y]UnexpectedAddToListProcessAction[/Y].\index{méthodes!AddProcessAction}
   void            AddProcessAction(Process*, ActionType);
          /// A le même effet que le service ci-dessus. Le processus est spécifié par un nom de classe : le premier processus trouvé est retenu. Lève l'exception [Y]UnfoundProcessForAddingToEvent[/Y].
   void            AddProcessAction(char*, ActionType); // _no swig_
          /// Insère un élément en bonne place dans tableau des couples \{processus, action\} qui sont la conséquence de l'événement. Cette insertion se fait selon des critères d'ordre définis par la méthode attachée à la simulation par la méthode [Y]AssignBeforeInAgendaMethod[/Y] (classe [C]Simulation[/C]). Lève l'exception [Y]UnexpectedInsertInListProcessAction[/Y].\index{méthodes!InsertProcessAction}
   void            InsertProcessAction(Process*, ActionType);
          /// A le même effet que le service ci-dessus. Le processus est spécifié par un nom de classe : le premier processus trouvé est retenu. Lève l'exception [Y]UnfoundProcessForAddingToEvent[/Y].
   void            InsertProcessAction(char*, ActionType); // _no swig_
          /// Affiche à l'écran le  tableau des couples \{processus, action\} qui sont la conséquence de l'occurrence de l'événement.\index{méthodes!DisplayListProcessAction}
   void            DisplayListProcessAction() const;
          /// Renvoie le tableau des couples \{processus, action\} qui sont la conséquence de l'occurrence de l'événement.\index{méthodes!ListpProcessAction}
   StructProcessActionTab
                   ListpProcessAction() const;
          /// Renvoie le nombre de couples \{processus, action\} qui sont la conséquence de l'occurrence de l'événement.\index{méthodes!LengthListProcessAction}
   int             LengthListProcessAction() const;
          /// Renvoie le premier des processus dont le contrôle est une conséquence de l'occurrence de l'événement, ou NULL si aucun processus n'a encore été attaché.\index{méthodes!FirstProcess}
   Process*        FirstProcess();
          /// Renvoie le type de l'action de contrôle du  premier processus lié à l'événement, ou UNSPECIFIED si aucune directive sur processus n'a encore été édictée.\index{méthodes!FirstActionLabel}
   ActionType      FirstActionLabel(); // - const JPR 110616
          /// Enlève tous les éléments du tableau des couples \{processus, action\} qui sont la conséquence de l'événement.\index{méthodes!ClearListProcessAction}
   void            ClearListProcessAction(); 
          /// Supprime les processus liés à cet événement (avec leurs méthodes).
   void            DeleteProcesses();       // _no swig_
//@}

/**@name [SC]L'entité attachée au(x) processus[/SC]  */
//@{ 
          /// Etablit que l'entité fournie en argument est celle sur laquelle le processus dont le symbole de classe est en argument opère un changement d'état. Lève l'exception [Y]BadProcessedEntityClass[/Y].
   void                ProcessedEntity(UserProcessId, BasicEntity*);  // _no swig_
          /// Renvoie l'entité sur laquelle le processus dont le symbole de classe est en argument opère un changement d'état. Lève l'exception [Y]DoNotHaveProcessedEntity[/Y].\index{méthodes!ProcessedEntity - Event}
   BasicEntity*         ProcessedBasicEntity(UserProcessId);
          /// {\em idem} avec une entité supposée hériter de la classe [C]Entity[/C].
   Entity*             ProcessedEntity(UserProcessId);
//@}

// début code gestion OutputSpec  
          // Détermine si l'occurrence de l'événement doit être enregistrée dans une instance de [C]EventOutputSpec[/C].
   void              IsRecorded(Boolean);   // _no swig_  
          // Renvoie TRUE si l'occurrence de l'événement doit être enregistrée dans une instance de [C]EventOutputSpec[/C].\index{méthodes!Event - IsRecorded}
   Boolean           IsRecorded() const;   // _no swig_   
          // Renvoie l'instance de [C]OutputSpec[/C] dans laquelle l'occurrence de l'événement est enregistrée.\index{méthodes!Event - GetOutputSpec}
   EventOutputSpec*  GetOutputSpec() const;   // _no swig_ 
// fin code gestion OutputSpec 

   /**@name [SC]Sp\'ecification des post-cons\'equences d'un \'ev\'enement[/SC]
Le message : [P]<event>->SetPostConsequence(f);[/P] est normalement envoyé par l'utilisateur dans le constructeur de l'événement\index{méthodes!SetPostConsequence}. Il affecte à la méthode instance de [Y]EventPostConsequence[/Y] attachée à tout événement un corps {\em f} qui sera invoqué par la méthode [Y]Run[/Y], après le traitement du tableau renvoyé par [Y]ListpProcessAction[/Y], seulement si tous les processus de cette liste ont vu leurs préconditions\index{précondition} satisfaites. 
\noLF
La fonction affectée est normalement réalisée dans le fichier [F]UserEvent.cc[/F] (en tous cas avant sa première utilisation). Le corps de la fonction {\em f} est libre. Son argument est l'instance de [C]EventMethod[/C] dont elle est le corps. L'événement lui-même est renvoyé par une invocation de [Y]DescribedEvent[/Y] sur cette instance de [C]EventMethod[/C]. Par exemple {\em f} peut être :
\begin{verbatim}
[P]void MyEventClass_Fct1(EventMethod* pM) { 
  Event* pCurrentEvent = pM->DescribedEvent();
  ...
}[/P]
\end{verbatim}
  */  
  //@{  
          /// Dote l'événement d'un corps de méthode qui code sa post-conséquence.\index{méthodes!SetPostConsequence}
   void     SetPostConsequence (void (* f)(EventMethod* pM)); // _no swig_

          // Appelée par le moteur. Affecte la valeur TRUE lors de l'affectation  d'une méthode (par [Y]SetPostConsequence[/Y]). La valeur FALSE est attribuée par défaut.
   void     HasPostConsequence(Boolean);   // _no swig_
  //@}

  /**@name [SC]D\'efinition de l'\'ev\'enement autog\'en\'er\'e[/SC]
Un événement est autogénérable si on fait appel, dans son constructeur, au corps de l'un ou l'autre des trois services commentés ci-après. Dans les trois cas, le corps affecté à la méthode appropriée est normalement réalisé dans le fichier [F]UserEvent.cc[/F] (en tous cas avant sa première utilisation), et la valeur TRUE est affectée par [Y]IsAutoGenerate[/Y]. La génération d'un nouvel événement à partir d'un autre qui vient d'avoir lieu est alors réalisée dans la méthode [Y]Run[/Y] de [C]Simulation[/C], après le traitement du tableau renvoyé par [Y]ListpProcessAction[/Y], et après l'invocation éventuelle de la méthode affectée par [Y]SetPostConsequence[/Y]. 
\begin{itemize}
\item [Y]SetGenerateNextEvent[/Y]({\em f}). La fonction {\em f} doit coder elle-même la création du nouvel événement. Seulement si c'est un événement de la même classe (même valeur retournée par [Y]ClassSymbol[/Y]), la création doit se faire par un appel à la méthode [Y]RawCopy[/Y] de la classe [C]Event[/C]. La fonction {\em f} doit attacher l'événement généré à l'événement en cours par le message [Y]NextEvent[/Y] (il sera alors inséré automatiquement dans l'agenda d'événements) et renvoyer {\em true}. 
La fonction doit retourner {\em false} seulement si un nouvel événement n'a pas été généré (par choix du modélisateur). Comme pour tout corps de méthode, l'argument de la fonction {\em f} est la méthode dont elle est le corps (instance de [C]GenerateNextEvent[/C]). L'événement à partir duquel on réalise l'autogénération d'un autre est renvoyé par une invocation de [Y]DescribedEvent[/Y] sur cette instance de [C]EventMethod[/C]. Par exemple, {\em f} peut être :
\begin{verbatim}
[P]bool MyEventClass_Fct2(EventMethod* pM) { 
  Event* pCurrentEvent = pM->DescribedEvent();
  Event* pNewEvent = pCurrentEvent->RawCopy();
  ...
  pCurrentEvent->SetNextEvent(pNewEvent);
  return true;
}[/P]
\end{verbatim}

Noter que les pointeurs sur les processus associés à l'événement sont utilisables dans la fonction, puisque les processus ne sont éventuellement détruits qu'après l'invocation de [Y]SetGenerateNextEvent[/Y] (voir la remarque page 'La classe des processus ponctuels').

\item [Y]SetUpdateNextEvent[/Y]({\em f}). La fonction {\em f} opère sur un événement créé par la fonction assignée par [Y]GenerateNextEvent[/Y] ou sur un événement que le moteur s'est chargé de créer par copie ([Y]RawCopy[/Y]) de l'événement qui vient d'avoir lieu. La fonction {\em f} est alors invoquée par le moteur avec en argument la méthode (instance de [C]UpdateNextEvent[/C] dont elle est le corps). L'événement à partir duquel on réalise l'autogénération d'un autre est renvoyé par une invocation de [Y]DescribedEvent[/Y] sur cette instance de [C]EventMethod[/C].  
\noLF
L'unique argument (instance de [C]Argument[/C]) de celle-ci a pour symbole de classe PREVIOUS_EVENT_ARGUMENT. Sa valeur, affectée par le moteur, est le pointeur sur l'événement à partir duquel on a réalisé l'autogénération. L'événement autogénéré est alors renvoyé par [Y]NextEvent()[/Y]. On peut ainsi modifier ce dernier à partir du précédent. Les processus associés à l'événement sont copiés puis associés dans le même ordre au nouvel événement avec les mêmes directives. Par exemple, {\em f} peut être : 
\begin{verbatim}
[P]void MyEventClass_Fct3(EventMethod* pM) {
  Event* pCurrentEvent = pM->DescribedEvent(); 
  Event* pNewEvent = pM->GetEventArgValue(AUTOGENERATED_EVENT_ARGUMENT);
  ...
}[/P]
\end{verbatim}

\item [Y]SetDefineNextEventClockTime({\em f})[/Y]. Même origine du nouvel événement sur lequel opère {\em f}. La fonction {\em f} est alors invoquée par le moteur avec en argument la méthode (instance de [C]DefineNextEventClockTime[/C]) dont elle est le corps. L'événement à partir duquel on réalise l'autogénération est la valeur de l'argument PREVIOUS_EVENT_ARGUMENT. La date de cet événement est renvoyée par une invocation de [Y]OccurrenceClockTime[/Y]. La valeur renvoyée par {\em f} est alors affectée, par le moteur, à la date du nouvel événement. Les autres attributs du nouvel événement sont inchangés. Les processus associés à l'événement sont copiés puis associés dans le même ordre au nouvel événement avec les mêmes directives. Par exemple, {\em f} peut être : 
\begin{verbatim}
[P]int MyEventClass_Fct4(EventMethod* pM) {
  Event* pCurrentEvent = pM->DescribedEvent();
  int currentDate = pCurrentEvent->OccurrenceClockTime();
  ...
  return currentDate;
}[/P]
\end{verbatim}

\end{itemize}
  */  
  //@{  
 
          /// Spécifie que l'événement possède ou ne possède pas de fonction d'autogénération. La valeur FALSE est attribuée par défaut par le moteur. La valeur TRUE est attribuée lors de l'affectation d'un corps à l'une des méthodes d'autogénération (voir plus loin les méthodes [Y]SetGenerateNext\-Event[/Y], [Y]SetDefineNextEventClockTime[/Y] et [Y]SetUpdateNextEvent[/Y]). Peut aussi être utilisée dans la fonction d'autogénération elle-même avec l'argument FALSE pour inhiber définitivement l'autogénération. \index{méthodes!IsAutoGenerate}
   void            IsAutoGenerate(Boolean);   

          /// Dote l'événement d'un corps de méthode qui détermine complètement le nouvel événement (création, avec éventuellement celles des processus associés, et définition des attributs).\index{méthodes!SetGenerateNextEvent}
   void            SetGenerateNextEvent (bool (* f)(EventMethod*)); // _no swig_

          /// Dote l'événement d'un corps de méthode qui modifie le nouvel événement que le moteur s'est chargé de créer.\index{méthodes!SetUpdateNextEvent}
   void            SetUpdateNextEvent (void (* f)(EventMethod*));  // _no swig_

          /// Dote l'événement d'un corps de méthode qui détermine et renvoie la date du nouvel événement. Le moteur crée le nouvel événement et lui affecte (par [Y]SetOccurenceClockTime[/Y]) la date renvoyée.\index{méthodes!SetDefineNextEventClockTime}
   void            SetDefineNextEventClockTime (int (* f)(EventMethod*)); // _no swig_
          //
   Boolean         GenerateNextEventCore();   // _no swig_
          /// Renvoie l'événement suivant autogénéré.\index{méthodes!NextEvent}
  Event*           NextEvent() const;
          /// Affecte à l'événement qui reçoit ce message l'événement suivant autogénéré. Cette information est utilisée par la méthode [Y]Run[/Y] pour placer le nouvel événement dans l'agenda.\index{méthodes!SetNextEvent}
  void             SetNextEvent(Event*);  
          /// Détermine la classe (par son nom de classe) de l'éventuel événement post généré.\index{méthodes!SetNextEventClassName - Event}
   void            SetNextEventClassName(char*);
          /// Renvoie le nom de la classe de l'éventuel événement post généré.\index{méthodes!NextEventClassName - Event}
   char*           NextEventClassName() const;

          /// Affecte la probabilité d'occurrence de l'événement généré par l'invocation du corps de méthode affecté par [Y]SetGenerateNextEvent[/Y] ou, par défaut, de l'événement généré par le moteur.\index{méthodes!SetNextEventOccurrenceProbability}
   void            SetNextEventOccurrenceProbability(float);

          /// Affecte la probabilité d'occurrence de l'événement généré par l'invocation du corps de méthode affecté par [Y]SetGenerateNextEvent[/Y] ou, par défaut, de l'événement généré par le moteur. L'argument doit être entre 0 et 100.
   void            SetNextEventOccurrenceProbability(int);

          /// Renvoie la probabilité d'occurrence de l'événement généré par l'invocation du corps de méthode affecté par [Y]SetGenerateNextEvent[/Y] ou, par défaut, de l'événement généré par le moteur.\index{méthodes!NextEventOccurrenceProbability}
   float           NextEventOccurrenceProbability() const;

          /// Affecte le nombre d'unités de temps minimum avant lequel le prochain événement autogénéré du même type ne peut pas intervenir.\index{méthodes!SetMinDelayNextEvent}
   void            SetMinDelayNextEvent(int);

          /// Renvoie le nombre d'unités de temps minimum avant lequel le prochain événement autogénéré du même type ne peut pas intervenir.\index{méthodes!MinDelayNextEvent}
   int             MinDelayNextEvent() const;

          /// Affecte le nombre d'unités de temps maximum au delà duquel le prochain événement autogénéré du même type ne peut pas intervenir.\index{méthodes!SetMaxDelayNextEvent}
   void            SetMaxDelayNextEvent(int);

          /// Renvoie le nombre d'unités de temps maximum au delà duquel le prochain événement autogénéré du même type ne peut pas intervenir.\index{méthodes!MaxDelayNextEvent}
   int             MaxDelayNextEvent() const;

          /// Dote l'événement d'un corps de méthode qui détermine et renvoie le moment aléatoire d'occurrence d'un événement généré automatiquement par la fonction affectée par [Y]SetGenerateNextEvent[/Y]. Par défaut d'invocation de la présente méthode (ou si elle a été définie mais non utilisée dans les méthodes d'autogénération), et si les dates au plus tôt et au plus tard ont été spécifiées, la date sera déterminée sur la base d'une loi de probabilité uniforme entre les deux bornes. L'appel qui réalise ce tirage aléatoire est [P]ExecIntCoreMethod(RANDOMIZE_NEXTEVENTCLOCKTIME)[/P].\index{méthodes!SetRandomizeNextEventClockTime}
  void             SetRandomizeNextEventClockTime (int (* f)(EventMethod*)); // _no swig_
//@}

/**@name [SC]Gestion des m\'ethodes sp\'ecifiques[/SC]
Outre les méthodes prédéfinies liées à la postconséquence et à l'autogénération, il est possible de décrire un événement par des méthodes spécifiques, en fonction des besoins de l'application. Il faut alors construire une instance de [C]Method[/C] et l'ajouter au tableau des méthodes par une invocation de [Y]AddMethod[/Y].\\
\begin{verbatim}
[P]int f(EventMethod* pM) {
  int result = 0;
  Event* theEvent = pM->DescribedEvent();
   ...
  return result;
}
MyMethod::MyMethod() { // e.g.  IntEventMethod
  ...
  MyArgument* anArgument = new MyArgument();  // e.g. IntArgument
  AddArgument(anArgument);
  //AssignBody(f);  // reporté dans le constructeur de MyEvent
  ...
}
MyEvent::MyEvent() {
  ...
  MyMethod* aMeth = new MyMethod();
  AssignBody(MY_METHOD, f);
  AddMethod(aMeth);
  ...
}
int main(...) {
  ...
  MyEvent* anEvent = ...;
  SetIntArgValue(MY_METHOD, MY_ARGUMENT, 7); 
  int x = anEvent->ExecIntMethod(MY_METHOD);
  ...
}[/P]
\end{verbatim}
  
Le service [Y]AssignBody[/Y]\index{méthodes!AssignBody - Event} affecte un corps à une méthode (au sens d'une instance de [C]Method[/C]) dont le symbole de classe est donné en premier argument.\\

Les méthodes [Y]ExecVoidMethod[/Y], [Y]ExecPredicate[/Y] et [Y]ExecIntMethod[/Y] exécutent le corps de la méthode dont le symbole est passé en argument. La présente version n'offre que ces trois possibilités de type de méthode ({\em void}, {\em bool} et {\em int}), et ces deux possibilités de type d'argument ({\em int} et {\em Event*}).\index{méthodes!ExecVoidMethod - Process}\index{méthodes!ExecPredicate - Process}\index{méthodes!ExecIntMethod - Process}
 */
//@{ 
          /// Renvoie la méthode dont le symbole de classe est passé en argument.\index{méthodes!GetMethod - Event}
   Method*         GetMethod(UserMethodId);    // _no swig_

          /// Renvoie la méthode dont le nom de classe est passé en argument.
   Method*         GetMethod(char*);   // _no swig_
         /// Ajoute la méthode à la liste des méthodes de l'événement.\index{méthodes!AddMethod - Event}
   void            AddMethod(Method*); // _no swig_
          // Renvoie la table des méthodes.
   pMethodTab      MethodTab() const;   // _no swig_
          // 
   TableauInt      DualMethodTab() const;   // _no swig_
         /// 
   void            AssignBody(UserMethodId, void (*f)(EventMethod*)); // _no swig_
         /// 
   void            AssignBody(UserMethodId, bool (*f)(EventMethod*)); // _no swig_
         /// 
   void            AssignBody(UserMethodId, int (*f)(EventMethod*)); // _no swig_
         /// 
   void            SetIntArgValue(UserMethodId, UserArgumentId, int);        
         /// 
   void            SetEventArgValue(UserMethodId, UserArgumentId, Event*);     
         /// 
   void            ExecVoidMethod(UserMethodId);   
         /// 
   bool            ExecPredicate(UserMethodId);    
         /// 
   int             ExecIntMethod(UserMethodId);     
         // 
   void            ExecVoidCoreMethod(UserMethodId); // _no swig_   
         // 
   bool            ExecCorePredicate(UserMethodId); // _no swig_    
         // 
   int             ExecIntCoreMethod(UserMethodId); // _no swig_      

//@}

/* *@name [SC]Pour OpenDiese, gestion de la suspension des dépilements[/SC] */
  
          // Détermine si l'événement doit provoquer l'arrêt des dépilements quand il est trouvé en tête de l'agenda. \index{méthodes!IsPoppingToSuspend}
   void              SetIsPoppingToSuspend(Boolean);   // _no swig_
  
          // Renvoie vrai si l'événement doit provoquer l'arrêt des dépilements quand il est trouvé en tête de l'agenda.
   Boolean           GetIsPoppingToSuspend();   // _no swig_   
          // Détermine si l'événement doit être dépilé de l'agenda d'événements, quelle que soit la valeur que pourrait renvoyer la méthode [Y]IsPoppingToSuspend[/Y]. \index{méthodes!IsPoppingToResume}
   void              SetIsPoppingToResume(Boolean);   // _no swig_
          // Renvoie vrai si et seulement si le moteur doit reprendre les dépilements d'événements, quelle que soit la valeur que pourrait renvoyer la méthode [Y]IsPoppingToSuspend[/Y].
   Boolean           GetIsPoppingToResume();   // _no swig_
          // Dote l'événement d'un corps de méthode qui ......... L'appel qui réalise ce tirage aléatoire est [P]ExecPredicate(IS_POPPING_TO_SUSPEND)[/P].\index{méthodes!IsPoppingToSuspend}
   void                 SetIsPoppingToSuspendMethod (bool (* f)(EventMethod*)); // _no swig_ 
// @}

    // comptage
    static int   getNbConst() {return _nbConst;}
    static int   getNbDest() {return _nbDest;}

   //static pEventTab  mpInstanceTab;         // + JPR 040113 // - JPR 071109
   static int      mInstanceNumber;         // + JPR 020412

protected:

   int static      _nbConst;
   int static      _nbDest;
   UserEventId     mThisEventClassId;
   char*           mThisEventClassName;
   int             mThisEventNumber;
   char*           mThisEventInstanceName;
 
   int             mMinOccurrenceClockTime;
   int             mMaxOccurrenceClockTime;
   int             mOccurrenceClockTime;
   //Date*           mOccurrenceDate;  // JPR : unused from 110607 (D5.8)

   float           mOccurrenceProbability;
   short           mPriorityDegree;               // max short : 65535

   Boolean         mIsAutoGenerate;
   char*           mNextEventClassName;   // + JPR 040216
   Event*          mNextEvent;
   int             mMinDelayNextEvent;
   int             mMaxDelayNextEvent;
   float           mNextEventOccurrenceProbability;

   Boolean         mHasPostConsequence;

   StructProcessActionTab 
                   mListpProcessAction;
 
// début code gestion OutputSpec  
   Boolean          mIsRecorded;
   EventOutputSpec* mpOutputSpec;
// fin code gestion OutputSpec  

// début ajout 100630 pour gestion arrêt dépilement
   Boolean         mPoppingToSuspend;
   Boolean         mPoppingToResume;
// fin ajout 100630 pour gestion arrêt dépilement

   void            ClassSymbol(UserEventId);

   Boolean         IsAutoGenerate() const; 
   Boolean         HasPostConsequence() const;
   Event*          ApplyConsequences();
   void            DeleteProcessesIfNeeded(bool);

   pMethodTab      mpMethodTab;    // + JPR 0204
   TableauInt      mpDualMethodTab;

private:
   friend class Simulation; // pour y utiliser ApplyConsequences() 
};  // _end class_

//@}  // fin classe Event

// INLINE METHODS

// EXTERNAL REFERENCES

#endif  // _EVENT_HH
       
