//============================================================================
// FILE: StructureMonitor.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe des démons
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _STRUCTUREMONITOR_HH
#define _STRUCTUREMONITOR_HH

// SYSTEM INCLUDES
//
//#include <>                                    //

// PROJECT INCLUDES
//
#include "CoreTypes.h"

/**@name CDPFORTEXLes d\'emons sur la structure des entit\'es
\begin{itemize} \item Directive d'inclusion :
[P]
\#include StructureMonitor.h}
[/P]
 \end{itemize}

Un démon\index{démon} de cette classe est une structure analogue à un démon sur les descripteurs d'entités, à ceci près qu'il permettent de spécifier des actions qui doivent être déclenchées lorsqu'on modifie la structure d'une entité (par [Y]AddElement[/Y], \ldots). La spécification est, là encore, faite dans le démon lui-même, puis le démon est attaché directement à l'entité concernée (et non à un descripteur de cette entité).\\

Un tel démon est déclaré et défini comme une instance de la classe [C]StructureMonitor[/C]. La version actuelle de DIESE ne permet que de spécifier des actions à déclencher lors de l'ajout d'un élément à une entité ou bien lors de son retrait (à l'exception donc des opérations sur les composants, la super-entité ou les sur-ensembles).\\

Les éléments de code qui permettent le déclenchement automatique d'actions lors de la modification de la structure sont les mêmes que pour les démons liés aux descripteurs variables.

{\large {\tt Remarques}} :
\begin{itemize}
\item Un même démon peut être attaché à plusieurs entités différentes. C'est bien clair lorsqu'il s'agit d'instances de la même classe. C'est aussi vrai pour des instances de classes différentes, à condition que l'effet de bord de la fonction ait un sens identique pour les différentes classes. Par exemple, un démon dont l'effet de bord est de modifier le nombre d'éléments lors de l'ajout d'un élément, pourra être attaché à toute classe d'entités.   
\begin{verbatim}
[P]void App_DefineMonitors() {
  InputOutputMonitor = new StructureMonitor("inputOutputMonitor");
  InputOutputMonitor->AssignWhenAddElementMethod(InputOutput_whenAdd);
  InputOutputMonitor->AssignWhenRemoveElementMethod(InputOutput_whenRemove);
  ...
};[/P]
\end{verbatim}

\item Il n'est pas possible d'attacher plusieurs démons à la même entité. Si l'idée d'attacher plusieurs démons était justifiée par la multiplicité des effets de bords à déclencher, alors le codage de ces différents effets doit être assemblé dans les fonctions liées aux méthodes prédéfinies d'un seul démon.   
\begin{verbatim}
[P]Herd::Herd() {
  ...
  ElementClassId(ANIMAL);
  BirthDeathMonitor->AttachToEntity(this); // inopérant
  InputOutputMonitor->AttachToEntity(this);
  ...
}[/P]
\end{verbatim} 

\item Il est par contre possible, pour une entité, de lui attacher directement un démon sur sa structure, et d'attacher en outre des démons sur ses descripteurs variables. 

\item On peut vouloir modifier la structure d'une entité, sans déclencher, à titre exceptionnel, les effets de bord du démon attaché à l'entité. Il suffit pour cela d'utiliser les méthodes normales de modification ([Y]AddElement[/Y], [Y]AddElements[/Y], \ldots) avec l'argument supplémentaire {\em FALSE}. Par exemple :
\begin{verbatim}
[P]pHerd_1->AddElement(pAnimal_24, FALSE);[/P]
\end{verbatim} 

\item Il est possible d'ôter une capacité de réaction à un démon, en lui envoyant un message approprié, correspondant à l'événement visé : par exemple [Y]UnassignWhenAddElementMethod[/Y], sans argument, pour supprimer la réaction à la modification du tableau des éléments :
\begin{verbatim}
[P]InputOutputMonitor->UnassignWhenAddElementMethod();[/P]
\end{verbatim} 

De même il est possible de détacher un démon d'une entité en envoyant au démon le message [Y]DetachFromEntity[/Y], avec l'adresse de l'entité en argument :
\begin{verbatim}
[P]InputOutput->DetachFromEntity(pHerd_1);[/P]
\end{verbatim} 
\end{itemize} 

*/
//@{
          ///
class StructureMonitor 
{
public:

  /**@name LIFECYCLE */
//@{
          // Constructeur de base. 
  //StructureMonitor();
          /// Constructeur avec passage du nom de classe. 
   StructureMonitor(char*);
          /// Constructeur par copie. Les méthodes déclenchées lors des modifications de structure ne sont pas copiées, c'est-à-dire que le démon d'origine et sa copie possèdent les mêmes pointeurs sur ces méthodes.
   StructureMonitor(const StructureMonitor&);
          /// Destructeur.
   ~StructureMonitor();  
          /// Affecte à la classe le nom donné en argument. Ce nom de classe sera hérité par toutes les instances de la classe (en fait normalement une seule). Cette méthode doit être appelée en invoquant le constructeur de [C]StructureMonitor[/C] (ui reçoit le nom de classe en argument)ou explicitement dans le constructeur des sous-classes). 
   void            ClassName(char*);      
          /// Renvoie le nom de classe du démon.\index{méthodes!ClassName - StructureMonitor}
   char*           ClassName() const;  
//@}                             

/**@name OPERATORS */
//@{
         /// Opérateur d'affectation. Renvoie une référence à l'entité d'origine.
   const
   StructureMonitor&      operator=(const StructureMonitor&);  

         // Opérateurs < et > (pour la forme).
  Boolean         operator < (const StructureMonitor&) const {return TRUE;};
  Boolean         operator > (const StructureMonitor&) const {return TRUE;}; 
//@}                             

/**@name OPERATIONS */
//@{ 
         /// Le démon récepteur de ce message est attaché à l'entité passée en argument. Les méthodes du récepteur pourront être déclenchées lors d'événements sur la structure de l'entité.\index{méthodes!AttachToEntity}
   void  AttachToEntity(BasicEntity*); 
         /// Le démon récepteur de ce message est détaché de l'entité passée en argument. Les méthodes du récepteur ne pourront plus être déclenchées lors d'événements sur la structure de l'entité.\index{méthodes!DetachFromEntity}
   void  DetachFromEntity(BasicEntity*);
//@}       

   void             AddInstance(StructureMonitor*);
   void             RemoveInstance(StructureMonitor*);                        

// ACCESS TO FLAGS

         // Renvoie TRUE si le démon réagit lors de l'ajout d'un élément à l'entité ; FALSE sinon.
   Boolean  HasAddElementMethod() const; 

         // Affecte TRUE si le démon doit réagir lors de l'ajout d'un élément à l'entité.
   void     HasAddElementMethod(Boolean);
         // Renvoie TRUE si le démon réagit lors du retrait d'un élément de l'entité ; FALSE sinon.
   Boolean  HasRemoveElementMethod() const; 

         // Affecte TRUE si le démon doit réagir lors du retrait d'un élément de l'entité.
   void     HasRemoveElementMethod(Boolean);


   /**@name [SC]Affectation des fonctions déclenchées automatiquement[/SC]
\\

Ces méthodes ci-dessous réalisent la liaison entre une fonction écrite par l'utilisateur et la méthode prédéfinie du démon connue pour être déclenchée lors de la modification du tableau des éléments (ajout ou retrait).\\

Cette liaison est réalisée par des messages tels que [Y]AssignWhenAddElementMethod[/Y] : la fonction passée en argument devient celle qui sera exécutée lors de l'ajout d'un élément à l'entité.  
\begin{verbatim}
[P]  InputOutputMonitor->AssignWhenAddElementMethod(InputOutput_whenAdd);[/P]
\end{verbatim}

Le premier argument (CURRENTENTITYCONTAINER_ARGUMENT) de la fonction sera lié, lors de l'appel, à l'entité à laquelle est attaché le démon, et le second (CANDIDATEENTITYELEMENT_ARGUMENT) à l'élément ajouté au tableau par [Y]AddElement[/Y] ou retiré par [Y]RemoveElement[/Y]. L'ajout ou le retrait ne s'opèrent réellement que si l'entité renvoyée n'est pas le pointeur NULL. C'est l'entité renvoyée par la fonction qui sera ajoutée du tableau. Dans le cas général, l'utilisateur programme le renvoi de l'entité passée en paramètre. La fonction déclenchée lors d'un retrait le sera immédiatement avant le retrait. L'entité retirée du tableau (celle qui est passée en argument) ne peut être ni modifiée ni changée par le corps de cette fonction.
*/
//@{   
         ///\index{méthodes!AssignWhenAddElementMethod}
   void      AssignWhenAddElementMethod(BasicEntity* (*f) (MonitorMethod*));
         ///\index{méthodes!AssignWhenRemoveElementMethod}
   void      AssignWhenRemoveElementMethod(BasicEntity* (*f) (MonitorMethod*));

//@} 

/**@name  [SC]Désaffectation des fonctions déclenchées automatiquement[/SC]
\\

Ces méthodes défont la liaison entre la fonction écrite par l'utilisateur et la méthode prédéfinie du démon. Par exemple, le message [Y]UnassignWhenAddElementMethod[/Y] fait que la fonction actuellement liée à l'ajout d'une entité au tableau des éléments ne sera plus exécutée lors d'un ajout ultérieur. La fonction elle-même n'est pas détruite, c'est-à-dire qu'elle pourra être à nouveau liée (par un [Y]AssignWhenAddElementMethod[/Y]).
\begin{verbatim}
[P]InputOutputMonitor->UnassignWhenAddElementMethod();[/P]
\end{verbatim} 
*/
//@{
         ///\index{méthodes!UnassignWhenAddElementMethod}
   void      UnassignWhenAddElementMethod();
         ///\index{méthodes!UnassignWhenRemoveElementMethod}
   void      UnassignWhenRemoveElementMethod();

//@} 

         // Renvoie la méthode dont le symbole de classe est passé en argument.
   Method*             GetMethod(UserMethodId); 
         // Renvoie la méthode dont le nom de classe est passé en argument.
   Method*             GetMethod(char*);
         // Renvoie la table des méthodes.
   pMethodTab          MethodTab() const;
         // 
   TableauInt          DualMethodTab() const;
 

// @Man: Affectation d'un corps aux m\'ethodes et de valeurs aux arguments. 
/* *Les méthodes [Y]ExecVoidCoreMethod[/Y] et [Y]ExecEntityCoreMethod[/Y] exécutent le corps de la méthode dont le symbole est passé en argument. \index{méthodes!ExecVoidCoreMethod - Monitor}
*/ 
// @{ 
         // 
   void            ExecVoidCoreMethod(UserMethodId);  
         // 
   BasicEntity*         ExecEntityCoreMethod(UserMethodId);
          
// @} 

// @Man: Affectation de m\'ethodes 
/* 
 */
// @{
         // Ajoute la méthode à la liste des méthodes du démon, seulement si c'est une méthode de démons.
   void                AddMethod(Method*);
// @}

    // comptage Const/Dest
    static int   getNbConst() {return _nbConst;}
    static int   getNbDest() {return _nbDest;}      

    // comptage instances
   //static int                   InstanceNumber(); 
   //static pStructureMonitorTab  InstanceTab();

   static int                   mInstanceNumber;         // + JPR 980202
   static pStructureMonitorTab  mpInstanceTab;               // + JPR 980202

protected:

   int static   _nbConst;
   int static   _nbDest;

   char*           mThisMonitorClassName;

   Boolean         mHasAddElementMethod;
   Boolean         mHasRemoveElementMethod;

   pMethodTab             mpMethodTab;    // + JPR 0211
   TableauInt             mpDualMethodTab;

private:
};

//@} // fin classe

// INLINE METHODS

// EXTERNAL REFERENCES

//@}  // ouverture dans DescValueMonitor

#endif  // _STRUCTUREMONITOR_HH
