//============================================================================
// FILE: C_AvailabilityConstraint.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête des contraintes de disponibilité des ressources
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_AVAILABILITYCONSTRAINT_HH
#define _C_AVAILABILITYCONSTRAINT_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "Entity.h"

#include "C_CoreTypes.h"
#include "C_SingleResource.h"

// EXTERNAL REFERENCES

// /X///////////////X/////////////////////////////X///////////////////////////
//
// CONTRAINTES DE DISPONIBILITE D'UNE RESSOURCE
//
// /X///////////////X/////////////////////////////X///////////////////////////

/**@name Contraintes de disponibilité d'une ressource\\

\begin{itemize} \item Directive d'inclusion : 
[P]\#include C_AvailabilityConstraint.h[/P]
\end{itemize}
Pour pouvoir être allouée à une activité quelconque, une ressource doit satisfaire toutes les contraintes de disponibilité qui lui sont attachées.
\begin{itemize}
\item Une ressource à état discret peut se voir attacher des [C]TimeAvailabilityConstraints[/C], des [C]StateAvailabilityConstraints[/C] et des [C]RecurrentAvailabilityTimetable[/C].
\item Une ressource à capacité peut se voir attacher, en plus, des [C]CapacityAvailabilityConstraints[/C].
\end{itemize}

Pour attacher une contrainte à une ressource, on ajoute une valeur au descripteur approprié de la ressource, parmi les suivants. La valeur est une instance de la classe mentionnée :
\begin{itemize}
\item descripteur [C]TimeAvailabilityAttribute[/C], à valeur [C]TimeAvailabilityConstraint[/C]
\item descripteur [C]StateAvailabilityAttribute[/C], à valeur [C]StateAvailabilityConstraint[/C] 
\item descripteur [C]AvailabilityTimetableAttribute[/C], à valeur [C]RecurrentAvailabilityTimetable[/C]
\item descripteur [C]CapacityAvailabilityAttribute[/C], à valeur [C]CapacityAvailabilityConstraint[/C]
\end{itemize}

{\large {\tt Attention :}} bien noter que la satisfaction de ces contraintes est vérifiée avant la tentative d'allocation de la ressource à une activité. Si la ressource est disponible, et après la tentative d'allocation, d'autres types de contraintes sont testées : celles représentées par des instances de [C]InconsistencyCondition[/C] et de [C]ActivitiesResourcesInconsistentCommitment[/C]. Ces dernières portent sur le contexte de l'allocation de la ressource : l'allocation conjointe d'autres ressources et les propriétés de l'activité réquérante. Ces contraintes sont documentées dans une section dédiée.
*/ 
//@{

/**@name La classe de base

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]AvailabilityConstraint[/C]} & {\tiny AVAILABILITY_CONSTRAINT} & {\tiny availabilityConstraint} & {\tiny } & {\tiny }
\end{tabular}

Cette classe n'est pas destinée à recevoir des instances directes.\\

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
 & {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny C} & {\tiny pEntity*} & {\tiny [C]DescribedResource[/C] DESCRIBED_RESOURCE describedResource} & {\tiny NULL} & {\tiny la ressource qui détient cette contrainte}
\end{tabular}
 */
//@{
          ///
class AvailabilityConstraint : public Entity
{
public:
          // Constructeur de base.
  AvailabilityConstraint();
          // Constructeur par copie.
  AvailabilityConstraint(const AvailabilityConstraint&);
          // Destructeur.
  ~AvailabilityConstraint();

/**@name [SC]Test de la contrainte[/SC] 

Selon que la contrainte est une instance de l'une ou l'autre des trois sous-classes, le présent service invoque le service du même nom attaché à la sous-classe.

*/
//@{
          /// Renvoie {\em vrai} si la contrainte recevant ce message est satisfaite en l'instant d'invocation, et {\em faux} sinon.\index{méthodes!Check (contrainte dispo.)}
bool Check();
//@}
};

//@}  // fin classe de base

/**@name Contraintes de disponibilité liée au temps

De manière générale, une telle contrainte est spécifiée par un domaine de valeurs d'instants, formellement une disjonction d'intervalles d'entiers, et un opérateur relationnel qui détermine si la ressource est disponible ou bien indisponible dans l'intervalle de temps défini par le domaine. La valeur par défaut du domaine est [0 infini], et la valeur par défaut de l'opérateur est IN.\\

Il existe deux manière d'exprimer la disjonction d'intervalles :
\begin{itemize}
\item on positionne explicitement dans le temps les bornes de chaque intervalle. Par exemple {[0 10] [15 15]}. Dans ce cas, on crée une instance de la sous-classe [C]TimeAvailabilityConstraint[/C] de [C]AvailabilityConstraint[/C].\\

\item fonctionnellement : les valeurs pour les bornes du premier intervalle sont calculées par une fonction, et les valeurs pour les bornes des intervalles suivants sont calculées par une fonction sur les intervalles précédents. Dans ce cas, on crée une instance de [C]RecurrentAvailabilityTimetable[/C], dont les éléments sont des instances de [C]AvailabilityTimeSlot[/C].
\end{itemize}
*/
//@{ 

/**@name Expression explicite d'un domaine temporel

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]TimeAvailabilityConstraint[/C]} & {\tiny TIME_AVAILABILITY_CONSTRAINT} & {\tiny timeAvailabilityConstraint} & {\tiny } & {\tiny }
\end{tabular}

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
 & {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny C} & {\tiny pIntIntervalTab*} & {\tiny [C]TimeDomainAttribute[/C] TIME_DOMAIN_ATTRIBUTE TimeDomainAttribute} & {\tiny NULL} & {\tiny la disjonction d'intervalles}
\end{tabular}

La valeur du descripteur [C]TimeDomainAttribute[/C] est un tableau d'intervalles d'entiers disjoints, rangés dans l'ordre croissant des bornes.

Une telle contrainte est valeur du descripteur [C]TimeAvailabilityAttribute[/C] de la ressource visée.\\

On peut construire un tel domaine de la manière programmée suivante :
[P]
\begin{verbatim}
{
  // ...
  TimeAvailabilityConstraint* pTAC = new TimeAvailabilityConstraint();
  pTAC->AddIntDomainValueElement(0, 10);
  pTAC->AddIntDomainValueElement(15, 15);
  // ...
  SingleWorker* pW = new SingleWorker();
  pW->AddEntityValueElement(TIME_AVAILABILITY_ATTRIBUTE, pTAC);
\end{verbatim}
[/P]

Et de la manière externalisée suivante :

[P]
\begin{verbatim}
+ I timeAvailabilityConstraint pTAC_1,
	timeDomainAttribute << (0 10);
	timeDomainAttribute << (15 15);
;
+ I singleWorker pW_1
	timeAvailabilityAttribute << <I><, pTAC_1>;
;
\end{verbatim}
[/P]

Dans les deux cas, le test de satisfaction peut être codé comme suit :

[P]
\begin{verbatim}
{
  TimeAvailabilityConstraint* pTAC = new TimeAvailabilityConstraint();
  // ...
  bool isSatisfied = pTAC->Check();
  printf("\nla valeur de l'horloge est %d\n\t--> satisfaction de %s = %d", 
         pCurrentSim->Clock(), pTAC->InstanceName(), isSatisfied);
  // ...
}
\end{verbatim}
[/P]
>>
\begin{verbatim}
la valeur de l'horloge est 12 
    --> satisfaction de timeAvailabilityConstraint_386 = 0
\end{verbatim}
*/ 
//@{
          ///
class TimeAvailabilityConstraint : public AvailabilityConstraint
{
public:
          // Constructeur de base.
  TimeAvailabilityConstraint();
          // Constructeur par copie.
  TimeAvailabilityConstraint(const TimeAvailabilityConstraint&);
          // Destructeur.
  ~TimeAvailabilityConstraint();

/**@name  [SC]Test de la contrainte[/SC] */
//@{
          /// Renvoie {\em vrai} si et seulement si l'instant courant est dans un des intervalles du tableau valeur du descripteur [C]TimeDomainAttribute[/C].
bool Check();
//@}
};

//@}   // fin classe 

/**@name Expression fonctionnelle d'un intervalle de disponibilité\\

Une instance de cette classe est destinée à être un élément d'une instance de la classe [C]RecurrentAvailabilityTimetable[/C]. Se reporter à la documentation de cette dernière pour une explication de l'utilité et de l'utilisation de ces classes.

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]AvailabilityTimeSlot[/C]} & {\tiny AVAILABILITY_TIME_SLOT} & {\tiny availabilityTimeSlot} & {\tiny } & {\tiny }
\end{tabular}

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
 & {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny V} & {\tiny int} & {\tiny [C]BegDate[/C] BEG_DATE begDate} & {\tiny -9} & {\tiny borne inférieure de l'intervalle}\hline
{\tiny V} & {\tiny int} & {\tiny [C]EndDate[/C] END_DATE endDate} & {\tiny -9} & {\tiny borne supérieure de l'intervalle}
\end{tabular}
  
  {\bf Méthodes :}

Les méthodes suivantes de la série [C]Set.TimeSlot.Value[/C] servent au moteur de simulation à déterminer fonctionnellement les bornes (descripteurs [C]BegDate[/C] et [C]EndDate[/C]) des intervalles de temps composant le [C]RecurrentAvailabilityTimetable[/C]. Elles n'ont pas de corps prédéfini, et ce corps est en général attribué dans le code de l'application. S'il ne l'est pas, les descripteurs visés par la méthode restent à leur valeur par défaut (-9).\\

On qualifie de "courant" l'intervalle dans lequel on se trouve, en l'instant courant de l'horloge de simulation, ou bien le suivant si on est entre deux intervalles.

\begin{itemize}
\item le message [C]SetNextTimeSlotBegValue[/C], adressé à l'intervalle courant, détermine la borne inférieure du prochain intervalle, ou du premier intervalle
\item le message [C]SetThisTimeSlotBegValue[/C], adressé au prochain intervalle, détermine sa borne inférieure, ou la borne inférieure du premier intervalle 
\item le message [C]SetThisTimeSlotEndValue[/C], adressé à un intervalle, détermine la borne supérieure de cet intervalle 
  \item [C]OpeningPredicate[/C] est codé pour renvoyer {\em vrai} si et seulement si l'instant courant est dans l'intervalle courant (au sens ci-dessus)
\item [C]ClosingPredicate[/C] est codé pour renvoyer {\em vrai} si et seulement si l'instant courant vient de sortir de l'intervalle courant
\end{itemize}

Lorsque on a donné un corps à [C]OpeningPredicate[/C] et à [C]ClosingPredicate[/C], ce sont eux qui détermine la dynamique de "traversée" du [C]AvailabilityTimeSlot[/C]. Sinon, le moteur de simulation effectue pour cela des comparaisons entre l'instant courant et les valeurs établies par les trois autres méthodes.\\

\begin{tabular}{ccccl}
{\tiny type} & {\tiny classe, symbole, nom} & {\tiny corps par défaut} & {\tiny classes des arguments}\hline
\hline
{\tiny int} & {\tiny [C]SetNextTimeSlotBegValue[/C] SET_NEXT_TIME_SLOT_BEG_VALUE setNextTimeSlotBegValue} & {\tiny} & {\tiny } \hline
{\tiny int} & {\tiny [C]SetThisTimeSlotBegValue[/C] SET_THIS_TIME_SLOT_BEG_VALUE setThisTimeSlotBegValue} & {\tiny} & {\tiny } \hline
{\tiny int} & {\tiny [C]SetThisTimeSlotEndValue[/C] SET_THIS_TIME_SLOT_END_VALUE setThisTimeSlotEndValue} & {\tiny} & {\tiny } \hline
{\tiny bool} & {\tiny [C]OpeningPredicate[/C] OPENING_PREDICATE openingPredicate} & {\tiny} & {\tiny } \hline
{\tiny bool} & {\tiny [C]ClosingPredicate[/C] CLOSING_PREDICATE closingPredicate} & {\tiny} & {\tiny }

\end{tabular}
*/ 
//@{
          ///
class AvailabilityTimeSlot : public Entity
{
public:
          // Constructeur de base.
  AvailabilityTimeSlot();
          // Constructeur par copie.
  AvailabilityTimeSlot(const AvailabilityTimeSlot&);
          // Destructeur.
  ~AvailabilityTimeSlot();

/**@name  OPERATION */
// @{
// @}
};

//@}   // fin classe 

/**@name Programme récurrent de disponibilité

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]RecurrentAvailabilityTimetable[/C]} & {\tiny RECURRENT_AVAILABILITY_TIMETABLE} & {\tiny recurrentAvailabilityTimetable} & {\tiny AVAILABILITY_TIME_SLOT} & {\tiny }
\end{tabular}

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
 & {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline

{\tiny C} & {\tiny int} & {\tiny [C]AvailabilityMode[/C] AVAILABILITY_MODE availabilityMode} & {\tiny IN} & {\tiny lire le commentaire ci-dessous}
\end{tabular}

Lorsqu'une ressource est contrainte par une instance de [C]RecurrentAvailabilityTimetable[/C], c'est qu'on a exprimé la contrainte temporelle par un ensemble d'intervalles récurrents sur l'axe du temps. Si la valeur du descripteur [C]AvailabilityMode[/C] est IN, la contrainte est vérifiée si l'instant courant est dans un de ces intervalles. Si la valeur est NOT_IN, la contrainte est vérifiée si l'instant courant est en dehors de tous ces intervalles.


*/ 
//@{
          ///
class RecurrentAvailabilityTimetable : public Entity // + JPR 100517
{
public:
          // Constructeur de base.
  RecurrentAvailabilityTimetable();
          // Constructeur par copie.
  RecurrentAvailabilityTimetable(const RecurrentAvailabilityTimetable&);
          // Destructeur.
  ~RecurrentAvailabilityTimetable();

/**@name  OPERATION */
//@{
          /// Renvoie 1 si le programme de disponibilité contient un [C]AvaibalilityTimeSlot[/C] contenant l'instant courant dans sa spécification. Et 0 sinon.\index{méthodes!GetCurrentAvailability}
int GetCurrentAvailability();

          /// Renvoie 1 si le programme de disponibilité contient un [C]AvaibalilityTimeSlot[/C] contenant l'instant passé en argument (supposé futur) dans sa spécification. Et 0 sinon.\index{méthodes!GetAvailabilityAtDate}
int GetFutureAvailability(int clock);

          /// \index{méthodes!GetCurrentAvailability}
void AttachToResource(SingleResource* rsc);
//@}
};

//@}   // fin classe 

//@}  // fin cte liée au temps

/**@name Contraintes de disponibilité liée à l'état de la ressource

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]StateAvailabilityConstraint[/C]} & {\tiny STATE_AVAILABILITY_CONSTRAINT} & {\tiny stateAvailabilityConstraint} & {\tiny } & {\tiny }
\end{tabular}

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
 & {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny C} & {\tiny DescValueSpecTab*} & {\tiny [C]StateDomain[/C] STATE_DOMAIN stateDomain} & {\tiny tableau vide} & {\tiny conjonction de domaines de valeurs}
\end{tabular}
  
  {\bf Méthodes :}

\begin{itemize}
\item [C]StateRelatedPredicate[/C] renvoie {\em vrai} dans toutes les situations dans lesquelles la contrainte est jugée satisfaite.
\end{itemize}

\begin{tabular}{ccccl}
{\tiny type} & {\tiny classe, symbole, nom} & {\tiny corps par défaut} & {\tiny classes des arguments}\hline
\hline
{\tiny Entity*} & {\tiny [C]StateRelatedPredicate[/C] STATE_RELATED_PREDICATE stateRelatedPredicate} & {\tiny gAlwaysTrue} & {\tiny}
\end{tabular}

Une telle contrainte est une des valeurs du descripteur [C]StateAvailabilityAttribute[/C] de la ressource. Elle est spécifiée de deux manières complémentaires :
\begin{itemize}
\item en surchargeant le corps de sa méthode [C]StateRelatedPredicate[/C], avec une fonction qui fait une partition entre les situations nécessaires de satisfaction et de non-satisfaction ;\\ 
\item par une liste d'instances de [C]DescValueSpec[/C] (plus exactement par des instances d'une de ses spécialisations liées au type de descripteur en jeu : INT, FLOAT, \ldots). En effet, l'état de la ressource est vu ici au travers de certains de ses descripteurs, et la disponibilité requiert que leurs valeurs soient, chacune, dans un certain domaine.\\
Cette liste d'instances est placée comme valeur du descripteur [C]StateDomain[/C] de la contrainte.
\end{itemize}

En l'absence de valeur pour [C]StateDomain[/C], le prédicat [C]StateRelatedPredicate[/C] décide seul de la satisfaction. Si le prédicat n'est pas surchargé, la contrainte est vérifiée par la ressource si toutes les invocations du service [Y]IsInDomain[/Y] sur les différentes instances de [C]DescValueSpec[/C] renvoient {\em vrai}. En présence des deux spécifications, la conjonction de leur satisfaction est nécessaire.
*/ 
//@{
          ///
class StateAvailabilityConstraint : public AvailabilityConstraint
{
public:
          // Constructeur de base.
  StateAvailabilityConstraint();
          // Constructeur par copie.
  StateAvailabilityConstraint(const StateAvailabilityConstraint&);
          // Destructeur.
  ~StateAvailabilityConstraint();

/**@name  OPERATION */
//@{
          /// Renvoie {\em vrai} si conjointement, d'une part il n'y a pas de prédicat de satisfaction ou s'il renvoie {\em vrai}, d'autre part si [C]StateDomain[/C] n'a pas d'éléments ou si tous ses éléments répondent {\em vrai} au message [Y]IsInDomain[/Y].
bool Check();
//@}
};

//@}  // fin cte liée à l'état

/**@name Contraintes de disponibilité liée à la capacité de la ressource

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]CapacityAvailabilityConstraint[/C]} & {\tiny CAPACITY_AVAILABILITY_CONSTRAINT} & {\tiny capacityAvailabilityConstraint} & {\tiny } & {\tiny }
\end{tabular}
  
  {\bf Méthodes :}

\begin{itemize}
\item [C]CapacityOfferDemandComparison[/C] renvoie {\em vrai} si la contrainte de capacite est satisfaite au vu de la comparaison 'offre/demande' sur la ressource a laquelle la contrainte est attachee.
\end{itemize}

\begin{tabular}{ccccl}
{\tiny type} & {\tiny classe, symbole, nom} & {\tiny corps par défaut} & {\tiny classes des arguments}\hline
\hline
{\tiny Entity*} & {\tiny [C]CapacityOfferDemandComparison[/C] CAPACITY_OFFER_DEMAND_COMPARISON capacityOfferDemandComparison} & {\tiny gAlwaysTrue} & {\tiny}
\end{tabular}

Une telle contrainte intervient éventuellement dans une opération de transfert (voir page dédiée). Elle est une des valeurs du descripteur [C]CapacityAvailabilityAttribute[/C] de la ressource. Elle est spécifiée en surchargeant le corps de sa méthode [C]CapacityOfferDemandComparison[/C], avec une fonction qui fait une partition entre les situations de satisfaction et de non-satisfaction.\\

\CDb fournit une fonction standard pour surcharger [C]CapacityOfferDemandComparison[/C] : [Y]standardCapacityAvailabilityConstraint_offerDemandComparison_Body[/Y]. La sous-classe [C]StandardCapacityAvailabilityConstraint[/C], possède par construction cette fonction comme corps de [C]CapacityOfferDemandComparison[/C].
\begin{itemize}
*/ 
//@{

/**@name Classe de base
*/ 
//@{
          ///
class CapacityAvailabilityConstraint : public AvailabilityConstraint
{
public:
          // Constructeur de base.
  CapacityAvailabilityConstraint();
          // Constructeur par copie.
  CapacityAvailabilityConstraint(const CapacityAvailabilityConstraint&);
          // Destructeur.
  ~CapacityAvailabilityConstraint();

/**@name  OPERATION */
//@{
          /// Renvoie {\em vrai} s'il n'y a pas de prédicat de satisfaction ou s'il renvoie {\em vrai}, au vu de la comparaison 'offre/demande'.
bool Check();
//@}

bool standardCapacityAvailabilityConstraint_offerDemandComparison_Body(EntityMethod*);

};
//@}  // fin classe de base cte liée à la capacité

/**@name La spécialisation standard prédéfinie

Sous-classe de [C]CapacityAvailabilityConstraint[/C] dont la méthode [C]CapacityOfferDemandComparison[/C] est surchargée avec [Y]standardCapacityAvailabilityConstraint_offerDemandComparison_Body[/Y].\\

Cette fonction renvoie {\em vrai} si la "demande" est plus élevée que l'"offre", dans toutes ses dimensions (on rappelle qu'une capacité est possiblement multi-dimensionnelle).\\
La "demande", émanant de la source de l'opération de transfert, est spécifiée dans l'instance de [C]TransferInformation[/C] attachée au transfert.\\
L'"offre", par la destination de l'opération de transfert, est fonction de la valeur des descripteurs sur lesquels portent le transfert.\\

*/ 
//@{
          ///
class StandardCapacityAvailabilityConstraint : public CapacityAvailabilityConstraint
{
public:
          // Constructeur de base.
  StandardCapacityAvailabilityConstraint();
          // Constructeur par copie.
  StandardCapacityAvailabilityConstraint(const StandardCapacityAvailabilityConstraint&);
          // Destructeur.
  ~StandardCapacityAvailabilityConstraint();
};

//@}  // fin spécialisation std

//@}  // fin cte liée à la capacité

//@}  // fin Contraintes de disponibilité d'une ressource


// @} // fin Les ressources (C_Resource.h) --> dans C_InconsistentRscCommitment

#endif  // _C_AVAILABILITYCONSTRAINT_HH
