//============================================================================
// FILE: C_System.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe générique des systemes (C-DIESE)
//              (général, de décision, opérant, biophysique)
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_SYSTEM_HH
#define _C_SYSTEM_HH


#include "C_CoreTypes.h"

/**@name Les systèmes
\label{classeSystem}\index{système en général} 
\begin{itemize} \item Directive d'inclusion : 
[P]
\#include C_System.h
[/P]
\end{itemize} 

{\large  {\tt La classe de base [C]System[/C]}}

La classe [C]System[/C] est une classe abstraite, au sens où aucune instance de cette classe ne doit être créée. Elle est une sous-classe directe de [C]Entity[/C]. Un système réel doit être une instance de l'une des quatre sous-classes ci-dessous. Elle ne possède aucun descripteur ([C]Descriptor[/C]) propre, ni aucune méthode ([C]Method[/C]) propre, ni aucun service prédéfini.
\begin{itemize} 
\item [C]ProductionSystem[/C], 
\item [C]ControlledSystem[/C], 
\item [C]OperatingSystem[/C],  
\item [C]Manager[/C]
\end{itemize}  

{\large  {\tt L'entité simulée est généralement un système}}

Après la construction de l'instance de [C]Simulation[/C] (variable globale [Y]pCurrentSim[/Y], voir le manuel de \D), le développeur lui attache une entité particulière, appelée l'entité simulée, par le message [Y]SimulatedEntity[/Y], argumenté par cette entité.\\

L'entité simulée est une instance de n'importe quelle sous-classe de [C]Entity[/C], mais lorsque la couche \CD est exploitée, cette entité est généralement une instance de [C]System[/C], et plus précisément de [C]ProductionSystem[/C].\\

La construction de ce système simulé passe par la construction de ses composants et/ou de ses éléments, jusqu'aux composants et éléments atomiques, ainsi que de ses descripteurs et méthodes.
*/
//@{

//===========================================================================
//
// SYSTEM
//
//===========================================================================
/**@name Les systèmes en général\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]System[/C]} & {\tiny SYSTEM} & {\tiny system} & {\tiny } & {\tiny }
\end{tabular}
*/
//@{
          ///
//----------------------------------------------------------------------------
class System : public Entity
{
public:

/**@name  LIFECYCLE */
//@{ 
          //  Constructeur de base.
System();
/*
          // Constructeur par copie profonde. 
System(const System&);
          //  Destructeur.
~System();
*/
//@} 

/**@name  ACCESS */
//@{ 
           // / Renvoie {\em vrai} si le système est contrôlé et {\em faux} sinon.\index{méthodes!IsControlled}
//Boolean  IsControlled();
//@}

protected: 

private:
};
//@}
   

//===========================================================================
//
// PRODUCTION_SYSTEM
//
//===========================================================================

/**@name Le système de production et ses composants
\label{classeProductionSystem}\index{système de production} 

Un système de production est représenté par la classe [C]ProductionSystem[/C], dont toute instance est destinée à recevoir trois composants :
\begin{itemize}
\item une instance de la classe [C]ControlledSystem[/C]
\item une instance de la classe [C]OperatingSystem[/C]
\item une instance de la classe [C]Manager[/C]
\end{itemize}

Cette classe ne possède pas de descripteur, méthode ou service propre.

Une simulation ne gère généralement qu'un seul système de production, qui est alors l'"entité simulée" attachée à l'instance [Y]pCurrentSim[/Y].

Dans un fichier de données externe, la déclaration s'opère sur l'exemple suivant :

[P]
\begin{verbatim}
+ I myEntity, pEA,
    // ...
;
+ I controlledSystem pCS
	attachedEntity = <I><, pEA>;
;
+ I operatingSystem pOS
    // ...
;

+ I myManager pSD
    // ...
;
+ I productionSystem pSP  
	<- C <I><, pCS>;   
	<- C <I><, pOS>; 
	<- C <I><, pSD>;
;

ENTITE_SIMULEE <I><,pSP>; 
\end{verbatim}
[/P]

Si la construction est fait dans le code C++, on écrit par exemple :

[P]
\begin{verbatim} {
  //...
  Entity* pEA = new MyEntity();
  Entity* pCS = new ControlledSystem();
  pCS->SetEntityConstValue(ATTACHED_ENTITY, pEA);
  Entity* pOS = new OperatingSystem();
  Entity* pSD = new MyManager();
  Entity* pSP = new ProductionSystem();
  pSP->AddComponent(pCS);
  pSP->AddComponent(pOS);
  pSP->AddComponent(pSD);
  pCurrentSim->SimulatedEntity(pSP);
  //...
}
\end{verbatim}
[/P]

*/
//@{

/**@name La classe des systèmes de production\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]ProductionSystem[/C]} & {\tiny PRODUCTION_SYSTEM} & {\tiny productionSystem} & {\tiny } & {\tiny MANAGER OPERATING_SYSTEM CONTROLLED_SYSTEM}
\end{tabular}
*/
//@{
          ///
class ProductionSystem : public System
{
public:

  /**@name LIFECYCLE */
//@{
          //  Constructeur de base
ProductionSystem();
          /// Constructeur par copie profonde. 
ProductionSystem(const ProductionSystem&); 
          /// Destructeur superficiel. Utiliser [Y]DeleteInDepth[/Y] (classe [C]Entity[/C] de \D) pour une destruction en profondeur des composants ou éléments.
~ProductionSystem();
//@}

protected:

private:
};
 
//@}  // fin classe système de production
   
//===========================================================================
//
// OPERATING_SYSTEM
//
//===========================================================================

/**@name Le système opérant
\label{classeOperatingSystem}\index{système opérant}\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]OperatingSystem[/C]} & {\tiny OPERATING_SYSTEM} & {\tiny operatingSystem} & {\tiny } & {\tiny RESOURCE_POOL_DISJUNCTION}
\end{tabular}

Le système opérant est un composant d'un système de production. Il est caractérisé par deux choses :\\
\begin{itemize}
\item Son unique composant, qui est l'ensemble des ressources ({\em opérateurs} et {\em outils}) mobilisables pour la conduite du système, comme agent ou ressources propres des opérations spécifiées dans les activités.\\
 La classe de ce composant est [C]ResourcePoolDisjunction[/C]. C'est dire que les ressources doivent etre partitionnées en sous-ensembles (éventuellement un seul), dont chacun sera affecté à un "bloc activités-ressources" (instance de [C]ActivitiesResourcesBlock[/C]) de la stratégie ([C]Strategy[/C]) du système décisionnel ([C]Manager[/C]). Chacun de ces sous-ensembles peut être lui-même une disjonction d'ensembles de ressources.\\
L'expression "susceptibles d'intervenir" sous-entend que ni tous les opérateurs, ni tous les outils dans ce composant ne sont forcément utilisés pour agir sur le système avec une stratégie donnée. 

\item Une liste de contraintes posées sur l'exploitation des ressources. Cette liste est la valeur du descripteur [C]ResourceConstraintList[/C] du système opérant. Elle contient des instances de [C]InconsistencyCondition[/C] ou de [C]ActivitiesResourcesInconsistentCommitment[/C]. \\

Toutes les contraintes de la liste doivent être respectées conjointement par le mécanisme d'allocation des ressources aux activiés. On peut déclarer des contraintes sans les placer dans la liste : elles seront alors inactives.
\end{itemize}

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
 & {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny C} & {\tiny pEntityTab*} & {\tiny [C]ResourceConstraintList[/C] RESOURCE_CONSTRAINT_LIST resourceConstraintList} & {\tiny ()} & {\tiny contraintes actives sur les ressources}
\end{tabular}
  
  {\bf Méthodes :}
*/
//@{
          /// 
class OperatingSystem : public System
{
public:

  /**@name LIFECYCLE */
//@{
          //  Constructeur de base
OperatingSystem();
          /// Constructeur par copie profonde. 
OperatingSystem(const OperatingSystem&); 
          /// Destructeur superficiel. Utiliser [Y]DeleteInDepth[/Y] (classe [C]Entity[/C] de \D) pour une destruction en profondeur des composants.
~OperatingSystem();
//@}

/**@name [SC]Accès sélectif aux contraintes[/SC]\\\\ 

Ces services opèrent sur la liste qui est la valeur du descripteur [C]ResourceConstraintList[/C].\\
Important : pour chacun de ces services, le code client doit se charger de la destruction du tableau alloué et renvoyé.
*/
//@{
          /// Construit et renvoie la sous-liste des instances d'[C]ActivityInconsistencyCondition[/C]. 
pEntityTab* GetAICList();
          /// Construit et renvoie la sous-liste des instances de [C]ResourceSharingViolationCondition[/C].
pEntityTab* GetRSVCList();
          /// Construit et renvoie la sous-liste des instances d'[C]ActivitiesResourcesInconsistentCommitment[/C].
pEntityTab* GetARICList();
//@}

/**@name OPERATIONS
\begin{itemize}
\item A partir d'un ensemble de jeux d'activités primitives, possiblement inconsistants, [Y]PruneInconsistentSets[/Y] renvoie un sous-ensemble après application de règles éliminatoires successives :\\
\begin{itemize}
\item Si un des jeux possède une activité dont la mise en oeuvre est "requise", alors ceux qui n'ont pas cette propriété sont éliminés.
\item Dans chaque jeu, on élimine des activités primitives dont l'opération a sa condition de faisabilité (méthode [C]FeasibilityCondition[/C]) non satisfaite.
\end{itemize}

Après cet élagage, et dans chaque jeu successivement, le moteur de simulation ordonne les activités du jeu sur la base d'une fonction d'ordre. \\
Si une telle fonction d'ordre a été attachée au bloc activités-ressources, elle est exploitée (c'est alors le corps de la méthode [C]LocalCandidateActivitySetSortMethod[/C]).\\
Sinon, et si une a été attachée au système décidionnel ([C]Manager[/C]), elle est exploitée (c'est alors le corps de la méthode [C]CandidateActivitySetSortMethod[/C]).\\
Sinon encore, c'est une fonction prédéfinie qui est exploitée. Celle-ci range les activités par degrés décroissants (de 0 à 99) de priorité d'allocation (descripteur [C]AllocationPriorityDegree[/C]), puis repouse en fin de liste, en conservant leur ordre, les activités dont une des spécifications repose sur l'opérateur MAX.
 
\item Sur l'ensemble de jeux d'activités possiblement inconsistants, défini ci-dessus et passé en second argument, [Y]SetAllFeasibleAlternatives[/Y] réalise l'allocation des ressources et renvoie un ensemble de jeux (éventuellement vide) de jeux alloués de manière consistante. Le premier argument est le bloc activités-ressources au titre duquel l'allocation est effectuée.
 
\item [Y]SelectBestAlternative[/Y] sélectionne et renvoie le meilleur jeu d'activités parmi l'ensemble de jeux généré par [Y]SetAllFeasibleAlternatives[/Y]. En présence d'un jeu de critères ([C]CriterionSet[/C]), applique la méthode de Choquet de choix multi-critères. Sinon, invoque la méthode de sélection (surchargeable par le développeur) attachée soit au bloc-activités-ressources le cas échéant (corps de la méthode [/C]LocalBestActivitySetSelector[/C]), soit attachée au [C]Manager[/C] ([/C]BestActivitySetSelector[/C]). Dans les deux cas, en cas d'ex-aequo, un tirage aléatoire uniforme est fait.
 
\item Le service[Y]SelectAlternativesOnMaxOperationPriority[/Y] permet d'appliquer une règle standard de sélection d'un jeu d'activités. Il renvoie les jeux {\small {\tt set_k}} tels que la plus grande valeur de la priorité de l'opération, parmi les activités {\small {\tt k_i}} du jeu {\small {\tt k}}, est maximale sur les jeux. Pour exemple, soit les trois jeux suivants : \\

\begin{verbatim}
(A  B  C) (D  E  F) (A  B  F)
\end{verbatim}
avec dans le même ordre, les priorités suivantes pour les opérations :\\
\begin{verbatim}
 10 40 30  60 30 70  10 40 70
\end{verbatim}
alors, l'ensemble de jeux renvoyé sera :\\
\begin{verbatim}
(A  B  C) (A  B  F)
\end{verbatim}

Voici un exemple de mise en oeuvre de ce service : \\
[P]
\begin{verbatim}
BasicEntity* monManager_bestActivitySetSelector_Body(EntityMethod* pM) {
  Entity* pManager = pM->DescribedEntity();

  Entity* canditateSetsDisjunction 
               = pM->GetEntityArgValue(ACTIVITY_SETS_ARGUMENT);
  BasicEntity* result = NULL;

  InstructionSetDisjunction* currentResult = NULL;
  //
  // regle 1 : priorite au jeu qui contient l'activite de plus forte priorite
  // -----------------------------------------------------------------------
  Entity* pPS = pManager->GetSuperEntity();
  OperatingSystem* pOS = (OperatingSystem*)pPS->GetComponent(OPERATING_SYSTEM);
  InstructionSetDisjunction* pISD 
	= (InstructionSetDisjunction*)canditateSetsDisjunction;
  currentResult = pOS->SelectAlternativesOnMaxOperationPriority(pISD);

  // regle 2 :
  // ---------
  // ...
  // currentResult = ...

  // on tire aleatoirement un jeu parmi les restants ex-aequo
  // ---------------------------------------------------------
  int nbSets = currentResult->GetNumberOfElements();
  int randomIndex = RandomInteger(nbSets);
  result = currentResult->GetElement(randomIndex);

  return result;
};
\end{verbatim}
[/P]
\end{itemize}
*/
//@{
          /// 
InstructionSetDisjunction* 
  PruneInconsistentSets(InstructionSetDisjunction* p);

          /// 
InstructionSetDisjunction* 
  SetAllFeasibleAlternatives(ActivitiesResourcesBlock*,
							 InstructionSet*);

          /// 
InstructionSetConjunction* 
  SelectBestAlternative(ActivitiesResourcesBlock*,
						InstructionSetDisjunction*);

          /// 
InstructionSetDisjunction* 
  SelectAlternativesOnMaxOperationPriority(InstructionSetDisjunction*);// + JPR 111208
//@}

protected:

private:
};
//@}  // fin classe système opérant
   

//===========================================================================
//
// CONTROLLED_SYSTEM
//
//===========================================================================

/**@name Le système contrôlé, proprement dit 
\label{classeControlledSystem}\index{système contrôlé}\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]ControlledSystem[/C]} & {\tiny CONTROLLED_SYSTEM} & {\tiny controlledSystem} & {\tiny } & {\tiny }
\end{tabular}

Le système contrôlé est un composant d'un système de production ([C]ProductionSystem[/C]), à côté du système opérant et du système décisionnel.\\

On peut voir ce composant comme un point d'attache de l'entité dont on étudie le contrôle (le pilotage) par simulation. Cette entité peut avoir été développée (sur le socle BASIC_DIESE), préalablement à, et/ou indépendamment du questionnement sur le contrôle.\\

Plus précisément, le point d'attache est le descripteur [C]AttachedEntity[/C] du système contrôlé. Un exemple de déclaration externe est le suivant :
[P]
\begin{verbatim}
+ I myEntity, pEA,    // classe développée indépendamment
    // ...
;
+ I controlledSystem pCS
	attachedEntity = <I><, pEA>;  // attachement
;
// ...
+ I productionSystem pSP  
	<- C <I><, pCS>;
    // ...
;
\end{verbatim}
[/P]

Si la construction est fait dans le code C++, on écrit par exemple :

[P]
\begin{verbatim} {
  //...
  Entity* pEA = new MyEntity();
  Entity* pCS = new ControlledSystem();
  pCS->SetEntityConstValue(ATTACHED_ENTITY, pEA);
  // ...
  Entity* pSP = new ProductionSystem();
  pSP->AddComponent(pCS);
  // ...
}
\end{verbatim}
[/P]

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
 & {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny C} & {\tiny Entity*} & {\tiny [C]AttachedEntity[/C] ATTACHED_ENTITY attachedEntity} & {\tiny NULL} & {\tiny entité pilotée par le système décisionnel}
\end{tabular}
  
  {\bf Méthodes :}
*/
//@{
          /// 
class ControlledSystem : public System
{
public:

  /**@name LIFECYCLE */
//@{
          //  Constructeur de base
ControlledSystem();
          /// Constructeur par copie profonde. 
ControlledSystem(const ControlledSystem&); 
          /// Destructeur superficiel. Utiliser [Y]DeleteInDepth[/Y] (classe [C]Entity[/C] de \D) pour une destruction en profondeur des composants ou éléments.
~ControlledSystem();
//@}

protected:

private:
};
//@}  // fin classe système contrôlé

//===========================================================================
//
// MANAGER
//
//===========================================================================

/**@name Le système de décision (ou "manager")\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]Manager[/C]} & {\tiny MANAGER} & {\tiny manager} & {\tiny } & {\tiny STRATEGY}
\end{tabular}

  {\bf Descripteurs :}\\

Dans la table ci-dessous, les enregistrements évoqués sont ceux des "fichiers de données séquentielles mémorisées et prédites" (classe [C]BufferizedDataFile[/C]), développés dans l'application.

\begin{tabular}{ccccl}
 & {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny V} & {\tiny int} & {\tiny [C]MemoryScope[/C] MEMORY_SCOPE memoryScope} & {\tiny 0} & {\tiny nombre d'enregistrements mémorisés à partir du présent}\hline
{\tiny V} & {\tiny int} & {\tiny [C]PredictionScope[/C] PREDICTION_SCOPE predictioScope} & {\tiny 1} & {\tiny nombre d'enregistrements prédits à partir du suivant}
\end{tabular}
  
  {\bf Méthodes :}

\begin{itemize}
\item [C]BestActivitySetSelector[/C] sélectionne et renvoie un jeu, interprété ensuite comme le meilleur, parmi un ensemble de jeux d'activités alloués avec succès.
\item [C]CandidateActivitySetSortMethod[/C] modifie l'ordre des activités dans le jeu passé en second argument.
\end{itemize}

\begin{tabular}{ccccl}
{\tiny type} & {\tiny classe, symbole, nom} & {\tiny corps par défaut} & {\tiny classes des arguments}\hline
\hline
{\tiny Entity*} & {\tiny [C]BestActivitySetSelector[/C] BEST_ACTIVITY_SET_SELECTOR bestActivitySetSelector} & {\tiny} & {\tiny [C]ActivitiesResourcesBlockArgument[/C] [C]ActivitySetsArgument[C]} \hline
{\tiny Entity*} & {\tiny [C]CandidateActivitySetSortMethod[/C] CANDIDATE_ACTIVITY_SET_SORT_METHOD candidateActivitySetSortMethod} & {\tiny } & {\tiny [C]ActivitiesResourcesBlockArgument[/C] [C]ActivitySetArgument[/C]}
\end{tabular}
  
  {\bf Arguments des méthodes :}

\begin{tabular}{ccccl}
{\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny domaine}\hline
\hline
{\tiny Entity*} & {\tiny [C]ActivitiesResourcesBlockArgument[/C] ACTIVITIES_RESOURCES_BLOCK_ARGUMENT activitiesResourcesBlockArgument} & {\tiny NULL} & {\tiny instance de [C]ActivitiesResourcesBlock[/C]} \hline
{\tiny Entity*} & {\tiny [C]ActivitySetsArgument[/C] ACTIVITY_SETS_ARGUMENT activitySetsArgument} & {\tiny NULL} & {\tiny  instance de [C]InstructionSetDisjunction[/C]} \hline
{\tiny Entity*} & {\tiny [C]ActivitySetArgument[/C] ACTIVITY_SET_ARGUMENT activitySetArgument} & {\tiny NULL} & {\tiny  instance de [C]InstructionSetConjunction[/C]} 
\end{tabular}

*/
//@{
          /// 
//----------------------------------------------------------------------------
class Manager : public System
{
public:

  /**@name LIFECYCLE */
//@{
          //  Constructeur de base
Manager();
          /// Constructeur par copie profonde. 
Manager(const Manager&); 
          /// Destructeur superficiel. Utiliser [Y]DeleteInDepth[/Y] (classe [C]Entity[/C] de \D) pour une destruction en profondeur des composants (stratégie, \ldots).
~Manager();
//@}

/**@name  ACCESS */
//@{
           /// Renvoie le système dont le présent système de décision est un composant.\index{méthodes!GetPS} 
  ProductionSystem*  GetPS();   
           /// Renvoie le système de opérant du système dont le présent système de décision est un composant.\index{méthodes!GetOS}   
  OperatingSystem*  GetOS();   
           /// Renvoie le système contrôlé du système dont le présent système de décision est un composant.\index{méthodes!GetCS}
  ControlledSystem*  GetCS();
//@}  

protected: 

private:
};
//@} // fin classe
 
//@}  // système de production

//----------------------------------------------------------------------------
 
//@}  // fin systèmes





#endif  // _C_SYSTEM_HH
